#include "gcc-plugin.h"

#include "config.h"
#include "system.h"
#include "coretypes.h"
#include "tree.h"
#include "intl.h"
#include "tm.h"

#include "gimple.h"
#include "tree-pass.h"
#include "plugin-version.h"
#include "langhooks.h"
#include <string.h>
#include "tree-flow.h"
#include "tree-dump.h"
#include "tree-pass.h"
#include "basic-block.h"
#include "toplev.h"
#include "tree-inline.h"
#include "tree-iterator.h"
#include "real.h"
#include "fixed-value.h"
#include "tree-ssa-operands.h"
#if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6)
#include "c-family/c-common.h"
#else
#include "c-common.h"
#endif
#include <assert.h>

int plugin_is_GPL_compatible;

int serialize_state;
int GimpleSSA_dump_nr;
FILE *GimpleSSA_dump_file;
/* Information about a node to be dumped.  */
typedef struct serialize_node_info
{
  /* The index for the node.  */
  unsigned int index;
  /* Nonzero if the node is a binfo.  */
  unsigned int binfo_p : 1;
  unsigned int gimple_p : 1;
  unsigned int statement_p : 1;
} *serialize_node_info_p;

/* A serialize_queue is a link in the queue of things to be dumped.  */
typedef struct serialize_queue
{
  /* The queued tree node.  */
  splay_tree_node node;
  /* The next node in the queue.  */
  struct serialize_queue *next;
} *serialize_queue_p;

/* A serialize_info gives information about how we should perform the serialization
   and about the current state of the serialization.  */
struct serialize_info
{
  /* The stream on which to dump the information.  */
  FILE *stream;
  /* The original node.  */
  const_tree node;
  /* The next unused node index.  */
  unsigned int index;
  /* The next column.  */
  unsigned int column;
  /* The first node in the queue of nodes to be written out.  */
  serialize_queue_p queue;
  /* The last node in the queue.  */
  serialize_queue_p queue_end;
  /* Free queue nodes.  */
  serialize_queue_p free_list;
  /* The tree nodes which we have already written out.  The
     keys are the addresses of the nodes; the values are the integer
     indices we assigned them.  */
  splay_tree nodes;
};

typedef struct serialize_info *serialize_info_p;

/* Serialize the CHILD and its children.  */
#define serialize_child(field, child) \
  queue_and_serialize_index (di, field, child, DUMP_NONE)
/* Serialize the GIMPLE CHILD and its children.  */
#define serialize_gimple_child(field, child) \
  queue_and_serialize_gimple_index (di, field, child, DUMP_NONE)
/* Serialize the STATEMENT CHILD and its children.  */
#define serialize_statement_child(field, child) \
  queue_and_serialize_statement_index (di, field, child, DUMP_NONE)

#define LOCATION_COLUMN(LOC) ((expand_location (LOC)).column)
#define EXPR_COLUMNNO(NODE) LOCATION_COLUMN (EXPR_CHECK (NODE)->exp.locus)


static unsigned int queue (serialize_info_p, const_tree, int);
static unsigned int queue_gimple (serialize_info_p di, gimple t, int flags);
static unsigned int queue_statement (serialize_info_p di, struct control_flow_graph * cfg, int flags);
static void serialize_index (serialize_info_p, unsigned int);
static void queue_and_serialize_index (serialize_info_p di, const char *field, const_tree t, int);
static void queue_and_serialize_gimple_index (serialize_info_p di, const char *field, gimple t, int);
static void queue_and_serialize_statement_index (serialize_info_p di, const char *field, struct control_flow_graph * cfg, int);
static void queue_and_serialize_type (serialize_info_p di, const_tree t);
static void dequeue_and_serialize (serialize_info_p);
static void dequeue_and_serialize_gimple (serialize_info_p);
static void dequeue_and_serialize_statement (serialize_info_p);
static void serialize_new_line (serialize_info_p);
static void serialize_maybe_newline (serialize_info_p);
static void serialize_pointer (serialize_info_p di, const char *field, void *ptr);
static void serialize_int (serialize_info_p di, const char *field, int i);
static void serialize_wide_int (serialize_info_p di, const char *field, unsigned HOST_WIDE_INT i);
static void serialize_real (serialize_info_p di, tree t);
static int serialize_with_double_quote(const char * input, FILE * output, int length);
static int serialize_with_escape(const char * input, FILE * output, int length);
static void serialize_fixed (serialize_info_p di, const char *field, const FIXED_VALUE_TYPE *f);
static void serialize_string (serialize_info_p di, const char *string);
static void serialize_string_field (serialize_info_p di, const char *field, const char *str);
static void serialize_symbols (serialize_info_p di, const char * field, bitmap syms);
static void serialize_vops (serialize_info_p di, gimple stmt);
static void serialize_node (const_tree t, FILE *stream);

static tree gimple_call_expr_arglist (gimple g);
static tree build_custom_function_call_expr (location_t loc, tree fn, tree arglist);

static bool
is_alwaysTrue (void)
{
  return 1;
}

