/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                 URL: http://trac.ws.dei.polimi.it/panda
 *                      Microarchitectures Laboratory
 *                       Politecnico di Milano - DEI
 *             ***********************************************
 *              Copyright (c) 2004-2011 Politecnico di Milano
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the 
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
*/
/**
 * @file printf_verilog_generator.cpp
 * @brief Snippet for the printf dynamimc generator.
 *
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * @author Alessandro Nacci <alenacci@gmail.com>
 * @author Gianluca Durelli <durellinux@gmail.com>
 * $Revision: 10403 $
 * $Date: 2012-01-19 15:47:48 +0100 (Thu, 19 Jan 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
std::string fsm = "";
std::string fsm_last = "";
std::string display_cmd="";
std::string case_statement="";

std::string debug_display="";

for(int idx=0;idx<_np;idx++){
   debug_display+="$display(\""+_p[idx].name+": %d\","+_p[idx].name+");\n";
}

std::stringstream _npString;
_npString<<(_np);
std::string selector_dimension=_npString.str();

std::stringstream _np1String;
_np1String<<(_np-1);
std::string selector_left=_np1String.str();

int selector=1;
for(int i=0;i<_np;i++){
   std::stringstream _selector_string;
   _selector_string<<selector;
   std::stringstream _index_string;
   _index_string<<(i+1);
   case_statement+="            "+selector_dimension+"'d"+_selector_string.str()+":\n             begin\n               data1="+_p[i].name+";\n               data1_size=BITSIZE_in"+_index_string.str()+";\n             end\n";
   selector*=2;
}
case_statement+="            default:\n\
             begin\n\
               data1 = 64'b0;\n\
               data1_size = 8'b0;\n\
             end\n";

std::string sensitivity;
for(int i=0;i<_np;i++){
  sensitivity += " or " + _p[i].name;
}
if(_np>1){
  case_statement="case (_present_selector)\n"+case_statement+"          endcase";
}
else{
  case_statement="";
}

for(int i=1;i<_np;i++){
        if(i==1)
                display_cmd=_p[i].name;
        else
                display_cmd += ", " + _p[i].name;
}

fsm="\
reg [BITSIZE_Mout_addr_ram-1:0] _present_pointer, _next_pointer;\n\
reg [BITSIZE_Mout_addr_ram-1:0] _present_pointer1, _next_pointer1;\n\
reg done_port;\n\
reg Mout_oe_ram;\n\
reg Mout_we_ram;\n\
reg [BITSIZE_Mout_addr_ram-1:0] Mout_addr_ram;\n\
reg [BITSIZE_Mout_Wdata_ram-1:0] Mout_Wdata_ram;\n\
reg [7:0] Mout_data_ram_size;\n\
\n\
parameter [3:0] S_0 = 4'd0,\n\
  S_1 = 4'd1,\n\
  S_2 = 4'd2,\n\
  S_3 = 4'd3,\n\
  S_4 = 4'd4,\n\
  S_5 = 4'd5,\n\
  S_6 = 4'd6,\n\
  S_7 = 4'd7;\n\
reg [3:0] _present_state, _next_state;\n\
reg ["+selector_left+":0] _present_selector, _next_selector;\n\
reg [63:0] data1;\n\
reg [7:0] _present_data2, _next_data2;\n\
reg [7:0] data1_size;\n\
reg write_done;\n\
\n\
  always @(posedge clock or posedge reset)\n\
    if (reset)\n\
      begin\n\
        _present_state <= S_0;\n\
        _present_pointer <= 32'd0;\n\
        _present_pointer1 <= 32'd0;\n\
        _present_selector <="+selector_dimension+"'d0;\n\
        _present_data2 <= 8'b0;\n\
      end\n\
    else\n\
      begin\n\
        _present_state <= _next_state;\n\
        _present_pointer <= _next_pointer;\n\
        _present_pointer1 <= _next_pointer1;\n\
        _present_selector <= _next_selector;\n\
        _present_data2 <= _next_data2;\n\
      end\n\
\n\
  always @(_present_state or _present_pointer or _present_pointer1 or _present_selector or start_port or M_DataRdy or Min_we_ram or Min_oe_ram or Min_Wdata_ram or Min_addr_ram or Min_data_ram_size" + sensitivity + " or _present_data2 or M_Rdata_ram)\n\
      begin\n\
        Mout_we_ram = Min_we_ram;\n\
        Mout_Wdata_ram = Min_Wdata_ram;\n\
        Mout_oe_ram=Min_oe_ram;\n\
        Mout_addr_ram=Min_addr_ram;\n\
        Mout_data_ram_size=Min_data_ram_size;\n\
        done_port = 1'b0;\n\
        _next_state = _present_state;\n\
        _next_pointer = _present_pointer;\n\
        _next_pointer1 = _present_pointer1;\n\
        _next_selector = _present_selector;\n\
        _next_data2 = _present_data2;\n\
        "+ case_statement + "\n\
        case (_present_state)\n\
          S_0:\n\
            if(start_port)\n\
              begin\n\
                _next_pointer=32'd0;\n\
                _next_pointer1=32'd0;\n\
                _next_state=S_1;  \n\
                _next_selector="+selector_dimension+"'d2;\n \
              end\n\
            \n\
         S_1:\n\
           begin\n\
             Mout_addr_ram[31:0]=in1+_present_pointer;\n\
             Mout_data_ram_size=8'd8;\n\
             Mout_oe_ram=1'b1;\n\
             if(M_DataRdy)\n\
             begin\n\
                _next_data2 = M_Rdata_ram[7:0];\n\
                _next_state=S_2;\n\
// synthesis translate_off\n\
                write_done=1'b0;\n\
// synthesis translate_on\n\
             end\n\
           end\n\
         S_2:\n\
           begin\n\
             _next_pointer=_present_pointer+1'd1;\n\
             if((_present_data2!=8'd0)&&(_present_data2!=8'd37))\n\
             begin\n\
// synthesis translate_off\n\
               if(!write_done)\n\
               begin\n\
                 $write(\"%c\",_present_data2);\n\
               write_done=1'b1;\n\
               end\n\
// synthesis translate_on\n\
               _next_state=S_1;\n\
             end\n\
             else if(_present_data2==8'd37)\n\
               _next_state=S_3;\n\
             else if(_present_data2==8'd0)\n\
             begin\n\
               done_port = 1'b1;\n\
               _next_state=S_0;\n\
             end\n\
           end\n\
         S_3:\n\
           begin\n\
             Mout_addr_ram=in1[BITSIZE_Mout_addr_ram-1:0]+_present_pointer;\n\
             Mout_data_ram_size=8'd8;\n\
             Mout_oe_ram=1'b1;\n\
             if(M_DataRdy)\n\
             begin\n\
                _next_data2 = M_Rdata_ram[7:0];\n\
                _next_state=S_5;\n\
// synthesis translate_off\n\
                write_done=1'b0;\n\
// synthesis translate_on\n\
             end\n\
           end\n\
         S_5 :\n\
           begin\n\
             _next_state=S_6;\n\
             _next_pointer=_present_pointer+1'd1;\n\
             case(_present_data2)\n\
               8'd37: //%%\n\
               begin\n\
                 _next_state=S_1;\n\
// synthesis translate_off\n\
                 if(!write_done)\n\
                 begin\n\
                   $write(\"%c\",8'd37);\n\
                   write_done=1'b1;\n\
                 end\n\
// synthesis translate_on\n\
               end\n\
               8'd99: //Char\n\
               begin\n\
// synthesis translate_off\n\
                 if(!write_done)\n\
                 begin\n\
                   $write(\"%c\",data1[7:0]);\n\
                   write_done=1'b1;\n\
                 end\n\
// synthesis translate_on\n\
               end\n\
               8'd100: //Decimal %d\n\
                 if(data1_size ==8'd64)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0d\",$signed(data1));\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd32)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0d\",$signed(data1[31:0]));\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd16)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0d\",$signed(data1[15:0]));\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd8)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0d\",$signed(data1[7:0]));\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else\n\
                 begin\n\
// synthesis translate_off\n\
                   $display(\"ERROR - Decimal precision not supported (d) %d\", data1_size);\n\
                   $finish;\n\
// synthesis translate_on\n\
                 end\n\
               8'd105: //Decimal %i\n\
                 if(data1_size ==8'd64)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0d\",$signed(data1));\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd32)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0d\",$signed(data1[31:0]));\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd16)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0d\",$signed(data1[15:0]));\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd8)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0d\",$signed(data1[7:0]));\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else\n\
                 begin\n\
// synthesis translate_off\n\
                   $display(\"ERROR - Decimal precision not supported (i) %d\", data1_size);\n\
                   $finish;\n\
// synthesis translate_on\n\
                 end\n\
               8'd101: //Exponential %e\n\
               begin\n\
                 if(data1_size ==8'd64)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0e\",$bitstoreal(data1));\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd32)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
`ifdef XILINX_ISIM\n\
                     $write(\"%0e\",$bitstoreal(data1[31:0]));\n\
`elsif __ICARUS__\n\
                     $write(\"%0e\",data1[31:0]);\n\
`else\n\
                     $write(\"%0e\",$bitstoshortreal(data1[31:0]));\n\
`endif\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else\n\
                 begin\n\
// synthesis translate_off\n\
                   $display(\"ERROR - Floating point precision not supported %d\", data1_size);\n\
                   $finish;\n\
// synthesis translate_on\n\
                 end\n\
               end\n\
               8'd69: //Exponential %E\n\
               begin\n\
                 if(data1_size ==8'd64)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0e\",$bitstoreal(data1));\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd32)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
`ifdef XILINX_ISIM\n\
                     $write(\"%0e\",$bitstoreal(data1[31:0]));\n\
`elsif __ICARUS__\n\
                     $write(\"%0e\",data1[31:0]);\n\
`else\n\
                     $write(\"%0e\",$bitstoshortreal(data1[31:0]));\n\
`endif\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else\n\
                 begin\n\
// synthesis translate_off\n\
                   $display(\"ERROR - Floating point precision not supported %d\", data1_size);\n\
                   $finish;\n\
// synthesis translate_on\n\
                 end\n\
               end\n\
               8'd102: //Float %f\n\
               begin\n\
                 if(data1_size==8'd64)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0f\",$bitstoreal(data1));\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd32)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
`ifdef XILINX_ISIM\n\
                     $write(\"%0f\",$bitstoreal(data1[31:0]));\n\
`elsif __ICARUS__\n\
                     $write(\"%0f\",data1[31:0]);\n\
`else\n\
                     $write(\"%0f\",$bitstoshortreal(data1[31:0]));\n\
`endif\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else\n\
                 begin\n\
// synthesis translate_off\n\
                   $display(\"ERROR - Floating point precision not supported %d\", data1_size);\n\
                   $finish;\n\
// synthesis translate_on\n\
                 end\n\
               end\n\
               8'd70: //Float %F\n\
               begin\n\
                 if(data1_size==8'd64)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0f\",$bitstoreal(data1));\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd32)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
`ifdef XILINX_ISIM\n\
                     $write(\"%0f\",$bitstoreal(data1[31:0]));\n\
`elsif __ICARUS__\n\
                     $write(\"%0f\",data1[31:0]);\n\
`else\n\
                     $write(\"%0f\",$bitstoshortreal(data1[31:0]));\n\
`endif\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else\n\
                 begin\n\
// synthesis translate_off\n\
                   $display(\"ERROR - Floating point precision not supported %d\", data1_size);\n\
                   $finish;\n\
// synthesis translate_on\n\
                 end\n\
               end\n\
               8'd103: //Float %g\n\
               begin\n\
                 if(data1_size==8'd64)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0g\",$bitstoreal(data1));\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd32)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
`ifdef XILINX_ISIM\n\
                     $write(\"%0g\",$bitstoreal(data1[31:0]));\n\
`elsif __ICARUS__\n\
                     $write(\"%0g\",data1[31:0]);\n\
`else\n\
                     $write(\"%0g\",$bitstoshortreal(data1[31:0]));\n\
`endif\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else\n\
                 begin\n\
// synthesis translate_off\n\
                   $display(\"ERROR - Floating point precision not supported %d\", data1_size);\n\
                   $finish;\n\
// synthesis translate_on\n\
                 end\n\
               end\n\
               8'd71: //Float %G\n\
               begin\n\
                 if(data1_size==8'd64)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0g\",$bitstoreal(data1));\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd32)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
`ifdef XILINX_ISIM\n\
                     $write(\"%0g\",$bitstoreal(data1[31:0]));\n\
`elsif __ICARUS__\n\
                     $write(\"%0g\",data1[31:0]);\n\
`else\n\
                     $write(\"%0g\",$bitstoshortreal(data1[31:0]));\n\
`endif\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else\n\
                 begin\n\
// synthesis translate_off\n\
                   $display(\"ERROR - Floating point precision not supported %d\", data1_size);\n\
                   $finish;\n\
// synthesis translate_on\n\
                 end\n\
               end\n\
               8'd111: //Octal\n\
                 if(data1_size ==8'd64)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0o\",data1);\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd32)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0o\",data1[31:0]);\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd16)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0o\",data1[15:0]);\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd8)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0o\",data1[7:0]);\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else\n\
                 begin\n\
// synthesis translate_off\n\
                   $display(\"ERROR - Octal precision not supported %d\", data1_size);\n\
                   $finish;\n\
// synthesis translate_on\n\
                 end\n\
               8'd112: //%p\n\
                 begin\n\
// synthesis translate_off\n\
                 if(!write_done)\n\
                 begin\n\
                   $write(\"0x%0h\",data1);\n\
                   write_done=1'b1;\n\
                 end\n\
// synthesis translate_on\n\
               end\n\
               8'd115: //String\n\
               begin\n\
                 _next_state=S_7;\n\
                 _next_pointer1=32'd0;\n\
               end\n\
               8'd117: //unsigned int %u TO BE FIXED\n\
                 if(data1_size ==8'd64)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0d\",$unsigned(data1));\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd32)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0d\",$unsigned(data1[31:0]));\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd16)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0d\",$unsigned(data1[15:0]));\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd8)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0d\",$unsigned(data1[7:0]));\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else\n\
                 begin\n\
// synthesis translate_off\n\
                   $display(\"ERROR - Unsigned precision not supported %d\", data1_size);\n\
                   $finish;\n\
// synthesis translate_on\n\
                 end\n\
               8'd120: //Hex %x\n\
                 if(data1_size ==8'd64)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0h\",data1);\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd32)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0h\",data1[31:0]);\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd16)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0h\",data1[15:0]);\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd8)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0h\",data1[7:0]);\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else\n\
                 begin\n\
// synthesis translate_off\n\
                   $display(\"ERROR - Hex precision not supported %d\", data1_size);\n\
                   $finish;\n\
// synthesis translate_on\n\
                 end\n\
               8'd88: //Hex %X\n\
                 if(data1_size ==8'd64)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0h\",data1);\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd32)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0h\",data1[31:0]);\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd16)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0h\",data1[15:0]);\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else if(data1_size ==8'd8)\n\
                 begin\n\
// synthesis translate_off\n\
                   if(!write_done)\n\
                   begin\n\
                     $write(\"%0h\",data1[7:0]);\n\
                     write_done=1'b1;\n\
                   end\n\
// synthesis translate_on\n\
                 end\n\
                 else\n\
                 begin\n\
// synthesis translate_off\n\
                   $display(\"ERROR - Hex precision not supported %d\", data1_size);\n\
                   $finish;\n\
// synthesis translate_on\n\
                 end\n\
               default:\n\
                 _next_state=S_3;\n\
             endcase\n\
           end\n\
         S_6:\n\
           begin\n\
             _next_selector=_present_selector<<1;\n\
             _next_state=S_1;\n\
           end\n\
         S_7:\n\
           begin\n\
             Mout_addr_ram=data1[BITSIZE_Mout_addr_ram-1:0]+_present_pointer1;\n\
             Mout_data_ram_size=8'd8;\n\
             Mout_oe_ram=1'b1;\n\
             if(M_DataRdy)\n\
             begin\n\
               _next_data2 = M_Rdata_ram[7:0];\n\
               _next_state=S_4;\n\
// synthesis translate_off\n\
               write_done=1'b0;\n\
// synthesis translate_on\n\
             end\n\
           end\n\
         S_4:\n\
           begin\n\
             _next_pointer1=_present_pointer1+1'd1;\n\
             if(_present_data2!=8'd0)\n\
             begin\n\
// synthesis translate_off\n\
               if(!write_done)\n\
               begin\n\
                 $write(\"%c\",_present_data2);\n\
                 write_done=1'b1;\n\
               end\n\
// synthesis translate_on\n\
               _next_state=S_7;\n\
             end\n\
             else\n\
               _next_state=S_6;\n\
           end\n\
      endcase\n \
  end\n";

std::cout << fsm;
