dnl
dnl check gcc with plugin support enabled and plugins
dnl
AC_DEFUN([AC_CHECK_GCC_I386_VERSION],[

dnl switch to c
AC_LANG_PUSH([C])

dnl the directory where plugins will be put
PLUGIN_DIR=/opt/gcc_plugins

dnl the host compiler
HOST_GCC=`which $CC`


dnl the list of gccs to be tested
if test "x$ac_gcc" = "x"; then
   GCC_TO_BE_CHECKED="$HOST_GCC /usr/bin/gcc-4.6 /usr/bin/gcc-4.5 /opt/gcc-4.5.2/bin/gcc-4.5.2"
else
   GCC_TO_BE_CHECKED="$ac_gcc"
fi

for compiler in $GCC_TO_BE_CHECKED; do
   if test -f $compiler; then
      echo "checking $compiler..."
      dnl check for minimum gcc
      I386_GCC_VERSION=`$compiler -dumpspecs | grep \*version -A1 | tail -1`
      AS_VERSION_COMPARE($I386_GCC_VERSION, $1, echo "checking $compiler >= $1... no"; min=no, echo "checking $compiler >= $1... yes"; min=yes, echo "checking $compiler >= $1... yes"; min=yes)
      if test "$min" = "no" ; then
         continue;
      fi
      AS_VERSION_COMPARE($I386_GCC_VERSION, $2, echo "checking $compiler < $2... yes"; max=yes, echo "checking $compiler < $2... yes"; max=yes, echo "checking $compiler < $2... no"; max=no)
      if test "$max" = "no" ; then
         continue;
      fi
      I386_GCC_EXE=$compiler;
      I386_GCC_PLUGIN_DIR=`$I386_GCC_EXE -print-file-name=plugin`
      if test "x$I386_GCC_PLUGIN_DIR" = "xplugin"; then
         AC_MSG_ERROR($I386_GCC_EXE -print-file-name returns \"plugin\". Package gcc-4.x-plugin-dev missing?)
      fi
      echo "checking plugin directory...$I386_GCC_PLUGIN_DIR"
      gcc_file=`basename $I386_GCC_EXE`
      gcc_dir=`dirname $I386_GCC_EXE`
      cpp=`echo $gcc_file | sed s/gcc/cpp/`
      I386_CPP_EXE=$gcc_dir/$cpp
      if test -f $I386_CPP_EXE; then
         echo "checking cpp...$I386_CPP_EXE"
      else
         echo "checking cpp...no"
         I386_GCC_EXE=""
         continue
      fi
      gpp=`echo $gcc_file | sed s/gcc/g\+\+/`
      I386_GPP_EXE=$gcc_dir/$gpp
      if test -f $I386_GPP_EXE; then
         echo "checking g++...$I386_GPP_EXE"
         break
      else
         echo "checking g++...no"
         continue
      fi
   else
      echo "checking $compiler... not found"
   fi
done

if test x$I386_GCC_EXE = x; then
   AC_MSG_ERROR("Not found any gcc between $1 and $2);
fi

dnl set configure and makefile variables
build_I386_EMPTY_PLUGIN=yes;
build_I386_SSA_PLUGIN=yes;
I386_EMPTY_PLUGIN=$PLUGIN_DIR/plugin_dumpGimpleEmpty.so
I386_SSA_PLUGIN=$PLUGIN_DIR/plugin_dumpGimpleSSA.so
AC_SUBST(I386_EMPTY_PLUGIN)
AC_SUBST(I386_SSA_PLUGIN)
AC_SUBST(I386_GCC_PLUGIN_DIR)
AC_SUBST(I386_GCC_EXE)
AC_SUBST(I386_GCC_VERSION)
AC_SUBST(PLUGIN_DIR)
AC_DEFINE(HAVE_I386_COMPILER, 1, "Define if I386 compiler is compliant")
AC_DEFINE_UNQUOTED(I386_GCC_EXE, "${I386_GCC_EXE}", "Define the plugin gcc")
AC_DEFINE_UNQUOTED(I386_CPP_EXE, "${I386_CPP_EXE}", "Define the plugin cpp")
AC_DEFINE_UNQUOTED(I386_GPP_EXE, "${I386_GPP_EXE}", "Define the plugin g++")
AC_DEFINE_UNQUOTED(I386_EMPTY_PLUGIN, "${I386_EMPTY_PLUGIN}", "Define the filename of the GCC PandA Empty plugin")
AC_DEFINE_UNQUOTED(I386_SSA_PLUGIN, "${I386_SSA_PLUGIN}", "Define the filename of the GCC PandA SSA plugin")
AC_DEFINE_UNQUOTED(I386_GCC_VERSION, "${I386_GCC_VERSION}", "Define the gcc version")

dnl check if gcc plugin directory exists
if test -d $PLUGIN_DIR; then
  echo "checking $PLUGIN_DIR...yes"
else
  echo "checking $PLUGIN_DIR...no - creating"
  mkdir $PLUGIN_DIR || AC_MSG_ERROR(Can not create $PLUGIN_DIR)
fi

dnl switch back to old language
AC_LANG_POP([C])

])

dnl
dnl check gcc with plugin support enabled and plugins
dnl
AC_DEFUN([AC_CHECK_GCC_ARM_VERSION],[

echo "checking for arm cross compiler..."
dnl the directory where plugins will be put
ARM_PLUGIN_DIR=/opt/gcc_plugins

dnl switch to c
AC_LANG_PUSH([C])

dnl the list of gccs to be tested
GCC_TO_BE_CHECKED="$ac_arm_gcc /opt/x-tools/arm/bin/armeb-unknown-linux-gnueabi-gcc"

for compiler in $GCC_TO_BE_CHECKED; do
   if test -f $compiler; then
      echo "checking $compiler..."
      dnl check for minimum gcc
      ARM_GCC_VERSION=`$compiler -dumpspecs | grep \*version -A1 | tail -1`
      AS_VERSION_COMPARE($ARM_GCC_VERSION, $1, echo "checking $compiler >= $1... no"; min=no, echo "checking $compiler >= $1... yes"; min=yes, echo "checking $compiler >= $1... yes"; min=yes)
      if test "$min" = "no" ; then
         continue;
      fi
      AS_VERSION_COMPARE($ARM_GCC_VERSION, $2, echo "checking $compiler < $2... yes"; max=yes, echo "checking $compiler < $2... yes"; max=yes, echo "checking $compiler < $2... no"; max=no)
      if test "$max" = "no" ; then
         continue;
      fi
      ARM_GCC_VERSION=`echo $ARM_GCC_VERSION | sed 's/\./_/g'`
      echo "version is $ARM_GCC_VERSION"
      ARM_GCC_EXE=$compiler;
      ARM_GCC_PLUGIN_DIR=`$ARM_GCC_EXE -print-file-name=plugin`
      if test "x$ARM_GCC_PLUGIN_DIR" = "xplugin"; then
         ARM_GCC_EXE=""
         continue
      fi
      echo "checking plugin directory...$ARM_GCC_PLUGIN_DIR"
      ARM_GCC_CONFIGURE=`$TOPSRCDIR/macros/get_configure.sh $ARM_GCC_EXE`
      echo "checking arm gcc configure... $ARM_GCC_CONFIGURE"
      ARM_GCC_TARGET=`$TOPSRCDIR/macros/get_target.sh $ARM_GCC_EXE`
      echo "checking arm gcc target... $ARM_GCC_TARGET"
      ARM_GCC_SRC_DIR=`dirname $ARM_GCC_CONFIGURE`
      ARM_GCC_BUILT_DIR=$ARM_GCC_SRC_DIR/../../$ARM_GCC_TARGET/build/build-cc/
      echo "checking arm gcc build... $ARM_GCC_BUILT_DIR"
      if test -d $ARM_GCC_SRC_DIR; then
         echo "checking arm gcc source directory...$ARM_GCC_SRC_DIR"
      else
         echo "checking arm gcc source directory...not found"
         ARM_GCC_EXE=""
         continue
      fi
      arm_gcc_file=`basename $ARM_GCC_EXE`
      arm_gcc_dir=`dirname $ARM_GCC_EXE`
      arm_cpp=`echo $arm_gcc_file | sed s/gcc/cpp/`
      ARM_CPP_EXE=$arm_gcc_dir/$arm_cpp
      if test -f $ARM_CPP_EXE; then
         echo "checking cpp...$ARM_CPP_EXE"
      else
         echo "checking cpp...no"
         ARM_GCC_EXE=""
         continue
      fi
      arm_gpp=`echo $arm_gcc_file | sed s/gcc/gpp/`
      ARM_GPP_EXE=$arm_gcc_dir/$arm_gpp
      if test -f $ARM_GPP_EXE; then
         echo "checking g++...$ARM_GPP_EXE"
         break
      else
         echo "checking g++...no"
         break
      fi
   else
      echo "checking $compiler... not found"
   fi
done

dnl if gcc has not been found, create in /opt/gcc-4.5.2
if test "x$ARM_GCC_EXE" = x; then
  build_ARM_GCC=yes;
  ARM_GCC_EXE=/opt/x-tools/arm/bin/armeb-unknown-linux-gnueabi-gcc
  ARM_CPP_EXE=/opt/x-tools/arm/bin/armeb-unknown-linux-gnueabi-cpp
  ARM_GCC_PLUGIN_DIR=/opt/x-tools/arm/lib/gcc/armeb-unknown-linux-gnueabi/4.5.2/plugin
  ARM_GCC_SRC_DIR=/opt/x-tools/build_arm/src/gcc-4.5.2
  ARM_GCC_BUILT_DIR=/opt/x-tools/build_arm/armeb-unknown-linux-gnueabi/build/build-cc
  ARM_GCC_VERSION=4_5_2
fi

dnl set configure and makefile variables
build_ARM_EMPTY_PLUGIN=yes;
build_ARM_SSA_PLUGIN=yes;
build_ARM_RTL_PLUGIN=yes;
ARM_EMPTY_PLUGIN=$ARM_PLUGIN_DIR/ARM_plugin_dumpGimpleEmpty.so
ARM_SSA_PLUGIN=$ARM_PLUGIN_DIR/ARM_plugin_dumpGimpleSSA.so
ARM_RTL_PLUGIN=$ARM_PLUGIN_DIR/ARM_plugin_dumpRTL.so
AC_SUBST(ARM_EMPTY_PLUGIN)
AC_SUBST(ARM_SSA_PLUGIN)
AC_SUBST(ARM_RTL_PLUGIN)
AC_SUBST(ARM_GCC_PLUGIN_DIR)
AC_SUBST(ARM_GCC_EXE)
AC_SUBST(ARM_GCC_SRC_DIR)
AC_SUBST(ARM_PLUGIN_DIR)
AC_SUBST(ARM_GCC_VERSION)
AC_SUBST(ARM_GCC_BUILT_DIR)
AC_DEFINE(HAVE_ARM_COMPILER, 1, "Define if ARM cross compiler is already present")
AC_DEFINE_UNQUOTED(ARM_GCC_EXE, "${ARM_GCC_EXE}", "Define the plugin gcc")
AC_DEFINE_UNQUOTED(ARM_CPP_EXE, "${ARM_CPP_EXE}", "Define the plugin cpp")
AC_DEFINE_UNQUOTED(ARM_GPP_EXE, "${ARM_GPP_EXE}", "Define the plugin g++")
AC_DEFINE_UNQUOTED(ARM_EMPTY_PLUGIN, "${ARM_EMPTY_PLUGIN}", "Define the filename of the GCC PandA Empty plugin")
AC_DEFINE_UNQUOTED(ARM_SSA_PLUGIN, "${ARM_SSA_PLUGIN}", "Define the filename of the GCC PandA SSA plugin")
AC_DEFINE_UNQUOTED(ARM_RTL_PLUGIN, "${ARM_RTL_PLUGIN}", "Define the filename of the GCC PandA RTL plugin")
AC_DEFINE_UNQUOTED(ARM_GCC_VERSION, "${ARM_GCC_VERSION}", "Define the arm gcc version")

dnl check if gcc plugin directory exists
if test -d $ARM_PLUGIN_DIR; then
  echo "checking $ARM_PLUGIN_DIR...yes"
else
  echo "checking $ARM_PLUGIN_DIR...no - creating"
  mkdir $ARM_PLUGIN_DIR || AC_MSG_ERROR(Can not create $ARM_PLUGIN_DIR)
fi

])

dnl
dnl check gcc with plugin support enabled and plugins
dnl
AC_DEFUN([AC_CHECK_GCC_SPARC_VERSION],[
    AC_ARG_WITH(sparc-gcc,
    [  --with-sparc-gcc=GCC  the plugin compliant sparc gcc],
    [
       ac_sparc_gcc="$withval"
    ])

echo "checking for sparc cross compiler..."
dnl the directory where plugins will be put
SPARC_PLUGIN_DIR=/opt/gcc_plugins

dnl switch to c
AC_LANG_PUSH([C])

dnl the list of gccs to be tested
GCC_TO_BE_CHECKED="$ac_sparc_gcc /opt/x-tools/sparc/bin/sparc-unknown-linux-gnu-gcc"

for compiler in $GCC_TO_BE_CHECKED; do
   if test -f $compiler; then
      echo "checking $compiler..."
      dnl check for minimum gcc
      SPARC_GCC_VERSION=`$compiler -dumpspecs | grep \*version -A1 | tail -1`
      AS_VERSION_COMPARE($SPARC_GCC_VERSION, $1, echo "checking $compiler >= $1... no"; min=no, echo "checking $compiler >= $1... yes"; min=yes, echo "checking $compiler >= $1... yes"; min=yes)
      if test "$min" = "no" ; then
         continue;
      fi
      AS_VERSION_COMPARE($SPARC_GCC_VERSION, $2, echo "checking $compiler < $2... yes"; max=yes, echo "checking $compiler < $2... yes"; max=yes, echo "checking $compiler < $2... no"; max=no)
      if test "$max" = "no" ; then
         continue;
      fi
      SPARC_GCC_EXE=$compiler;
      SPARC_GCC_PLUGIN_DIR=`$SPARC_GCC_EXE -print-file-name=plugin`
      if test "x$SPARC_GCC_PLUGIN_DIR" = "xplugin"; then
         SPARC_GCC_EXE=""
         continue
      fi
      SPARC_GCC_VERSION=`echo $SPARC_GCC_VERSION | sed 's/\./_/g'`
      echo "version is $SPARC_GCC_VERSION"
      echo "checking plugin directory...$SPARC_GCC_PLUGIN_DIR"
      SPARC_GCC_CONFIGURE=`$TOPSRCDIR/macros/get_configure.sh $SPARC_GCC_EXE`
      echo "checking sparc gcc configure... $SPARC_GCC_CONFIGURE"
      SPARC_GCC_TARGET=`$TOPSRCDIR/macros/get_target.sh $SPARC_GCC_EXE`
      echo "checking sparc gcc target... $SPARC_GCC_TARGET"
      SPARC_GCC_SRC_DIR=`dirname $SPARC_GCC_CONFIGURE`
      SPARC_GCC_BUILT_DIR=$SPARC_GCC_SRC_DIR/../../$SPARC_GCC_TARGET/build/build-cc/
      echo "checking sparc gcc build... $SPARC_GCC_BUILT_DIR"
      if test -d $SPARC_GCC_SRC_DIR; then
         echo "checking sparc gcc source directory...$SPARC_GCC_SRC_DIR"
      else
         echo "checking sparc gcc source directory...not found"
         SPARC_GCC_EXE=""
         continue
      fi
      sparc_gcc_file=`basename $SPARC_GCC_EXE`
      sparc_gcc_dir=`dirname $SPARC_GCC_EXE`
      sparc_cpp=`echo $sparc_gcc_file | sed s/gcc/cpp/`
      SPARC_CPP_EXE=$sparc_gcc_dir/$sparc_cpp
      if test -f $SPARC_CPP_EXE; then
         echo "checking cpp...$SPARC_CPP_EXE"
      else
         echo "checking cpp...no"
         SPARC_GCC_EXE=""
         continue
      fi
      sparc_gpp=`echo $sparc_gcc_file | sed s/gcc/gpp/`
      SPARC_GPP_EXE=$sparc_gcc_dir/$sparc_gpp
      if test -f $SPARC_GPP_EXE; then
         echo "checking gpp...$SPARC_GPP_EXE"
         break
      else
         echo "checking gpp...no"
         break
      fi
   else
      echo "checking $compiler... not found"
   fi
done

dnl if gcc has not been found, create in /opt/gcc-4.5.2
if test "x$SPARC_GCC_EXE" = x; then
  build_SPARC_GCC=yes;
  SPARC_GCC_EXE=/opt/x-tools/sparc/bin/sparc-unknown-linux-gnu-gcc
  SPARC_CPP_EXE=/opt/x-tools/sparc/bin/sparc-unknown-linux-gnu-cpp
  SPARC_GCC_PLUGIN_DIR=/opt/x-tools/sparc/lib/gcc/sparc-unknown-linux-gnu/4.5.2/plugin
  SPARC_GCC_SRC_DIR=/opt/x-tools/build_sparc/src/gcc-4.5.2
  SPARC_GCC_BUILT_DIR=/opt/x-tools/build_sparc/sparc-unknown-linux-gnu/build/build-cc
  SPARC_GCC_VERSION=4_5_2
fi

dnl set configure and makefile variables
build_SPARC_EMPTY_PLUGIN=yes;
build_SPARC_SSA_PLUGIN=yes;
build_SPARC_RTL_PLUGIN=yes;
SPARC_EMPTY_PLUGIN=$SPARC_PLUGIN_DIR/SPARC_plugin_dumpGimpleEmpty.so
SPARC_SSA_PLUGIN=$SPARC_PLUGIN_DIR/SPARC_plugin_dumpGimpleSSA.so
SPARC_RTL_PLUGIN=$SPARC_PLUGIN_DIR/SPARC_plugin_dumpRTL.so
AC_SUBST(SPARC_EMPTY_PLUGIN)
AC_SUBST(SPARC_SSA_PLUGIN)
AC_SUBST(SPARC_RTL_PLUGIN)
AC_SUBST(SPARC_GCC_PLUGIN_DIR)
AC_SUBST(SPARC_GCC_EXE)
AC_SUBST(SPARC_GCC_SRC_DIR)
AC_SUBST(SPARC_PLUGIN_DIR)
AC_SUBST(SPARC_GCC_VERSION)
AC_SUBST(SPARC_GCC_BUILT_DIR)
AC_DEFINE(HAVE_SPARC_COMPILER, 1, "Define if the sparc cross compiler exists")
AC_DEFINE_UNQUOTED(SPARC_GCC_EXE, "${SPARC_GCC_EXE}", "Define the plugin GCC")
AC_DEFINE_UNQUOTED(SPARC_CPP_EXE, "${SPARC_CPP_EXE}", "Define the plugin CPP")
AC_DEFINE_UNQUOTED(SPARC_GPP_EXE, "${SPARC_GPP_EXE}", "Define the plugin GPP")
AC_DEFINE_UNQUOTED(SPARC_EMPTY_PLUGIN, "${SPARC_EMPTY_PLUGIN}", "Define the filename of the GCC PandA Empty plugin")
AC_DEFINE_UNQUOTED(SPARC_SSA_PLUGIN, "${SPARC_SSA_PLUGIN}", "Define the filename of the GCC PandA SSA plugin")
AC_DEFINE_UNQUOTED(SPARC_RTL_PLUGIN, "${SPARC_RTL_PLUGIN}", "Define the filename of the GCC PandA RTL plugin")
AC_DEFINE_UNQUOTED(SPARC_GCC_VERSION, "${SPARC_GCC_VERSION}", "Define the sparc gcc version")

dnl check if gcc plugin directory exists
if test -d $SPARC_PLUGIN_DIR; then
  echo "checking $SPARC_PLUGIN_DIR...yes"
else
  echo "checking $SPARC_PLUGIN_DIR...no - creating"
  mkdir $SPARC_PLUGIN_DIR || AC_MSG_ERROR(Can not create $SPARC_PLUGIN_DIR)
fi

])

