/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file BambuParameter.hpp
 * @brief 
 *
 *
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef BAMBUPARAMETER_HPP
#define BAMBUPARAMETER_HPP

#include "Parameter.hpp"
#include "refcount.hpp"

/**
 * @name Forward Declarations
 */
//@{
REF_FORWARD_DECL(BambuParameter);
//@}

class BambuParameter : public Parameter
{
   private :
      /**
       * Check the compatibility among the different parameters
       * and compute implicated parameters
       */
      void CheckParameters();

   public:

      /**
       * Default Constructor. It sets default values
       * @param program_name is the name of the executable
       */
      BambuParameter(const std::string program_name);

      /**
       * Destructor
       */
      virtual ~BambuParameter() {}

      /**
       * Execute parameter parsing
       * @param argc is the number of arguments
       * @param argv is the array of arguments given to program.
       */
      virtual int exec(int argc, char *argv[]);

      /**
       * Sets default values
       */
      void SetDefaults();

      /**
       * Print the usage of this tool
       * @param os is the stream where the message has to be printed
       */
      void PrintUsage(std::ostream &os);
};

typedef refcount<BambuParameter> BambuParameterRef;

#endif
