/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file HLS_step.hpp
 * @brief Base class for all HLS algorithms
 * 
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef _HLS_STEP_HPP_
#define _HLS_STEP_HPP_

#include "refcount.hpp"
REF_FORWARD_DECL(Parameter);
REF_FORWARD_DECL(hls);

class HLS_step
{
   protected:

      ///reference to the class containing all the parameters
      const ParameterRef Param;

      ///debug level of the class
      int debug_level;

      ///reference to the class containing all HLS information
      const hlsRef HLS;

   public:

      /**
       * Constructor
       * @param Param class containing all the parameters
       * @param HLS class containing all the HLS datastructures
       */
      HLS_step(const ParameterRef Param, const hlsRef HLS);

      /**
       * Destructor
       */
      virtual ~HLS_step();

      /**
       * Function that computes the HLS step
       * @param HLS is the data structure storing all the input and output data structures.
      */
      virtual void exec() = 0;

      /**
       * Returns an identifier for the solver
       * @return a string that represents the name for the implemented algorithm
       */
      virtual std::string get_kind_text() const = 0;

};
///refcount definition of the class
typedef refcount<HLS_step> HLS_stepRef;

#endif
