/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file classic_datapath.hpp
 * @brief Base class for usual datapath creation.
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 * $Locker:  $
 * $State: Exp $
 *
*/

#ifndef CLASSIC_DATAPATH_HPP
#define CLASSIC_DATAPATH_HPP

#include "datapath_creator.hpp"

#include "refcount.hpp"
/**
 * @name forward declarations
 */
//@{
REF_FORWARD_DECL(Parameter);
REF_FORWARD_DECL(hls);
REF_FORWARD_DECL(structural_object);
//@}

class classic_datapath: public datapath_creator
{
      /**
       * Adds the clock and reset ports to the structural description of the circuit
       * @param clock_sign is the object representing the clock signal
       * @param reset_sign is the object representing the reset signal
       */
      void add_clock_reset(structural_objectRef& clock_sign, structural_objectRef& reset_sign);

      /**
       * Adds the input/output ports of the module
       */
      void add_ports();

   public:

      /**
       * Constructor.
       * @param Param is the class containing all the parameters
       * @param HLS is the class containing all the intermediate results of the synthesis
       */
      classic_datapath(const ParameterRef Param, const hlsRef HLS);

      /**
       * Destructor.
       */
      ~classic_datapath();

      /**
       * Function that creates the datapath.
       * @param HLS is the data structure storing all the input and output data structures.
       */
      void exec();

      /**
       * Function printing the name of the algorithm performed.
       */
      std::string get_kind_text() const;

};
///Refcount definition for the class
typedef refcount<classic_datapath> classic_datapathRef;

#endif
