/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file datapath.cpp
 * @brief Base class for all datapath creation algorithms.
 *
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#include "datapath_creator.hpp"

#include "hls.hpp"

#include "polixml.hpp"
#include "xml_helper.hpp"

#include "utility.hpp"
#include "classic_datapath.hpp"
#include "dbgPrintHelper.hpp"

datapath_creator::datapath_creator(const ParameterRef _Param, const hlsRef _HLS) :
   HLS_step(_Param, _HLS)
{

}

datapath_creator::~datapath_creator()
{

}

datapath_creatorRef datapath_creator::xload(const xml_element* node, const ParameterRef Param, const hlsRef HLS)
{
   std::string algorithm;
   if (CE_XVM(algorithm, node)) LOAD_XVM(algorithm, node);
   else algorithm = "CLASSIC";

   datapath_type algorithm_t;
   if (algorithm == "CLASSIC")
      algorithm_t = CLASSIC;
   else
      THROW_ERROR("Datapath creation algorithm \"" + algorithm + "\" currently not supported");
   return create_creator(algorithm_t, Param, HLS);
}

datapath_creatorRef datapath_creator::create_creator(datapath_type _type, const ParameterRef _Param, const hlsRef _HLS)
{
   switch (_type)
   {
      case CLASSIC:
      {
         return datapath_creatorRef(new classic_datapath(_Param, _HLS));
      }
      default:
         THROW_ERROR("Datapath creation algorithm not yet supported");
   }
   PANDA_UNREACHABLE();
   return datapath_creatorRef();
}
