/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file datapath.hpp
 * @brief Base class for all datapath creation algorithms.
 *
 * This class is a pure virtual one, that has to be specilized in order to implement a particular algorithm to create
 * datapath.
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/

#ifndef _DATAPATH_CREATOR_HPP_
#define _DATAPATH_CREATOR_HPP_

#include "HLS_step.hpp"

#include "refcount.hpp"
/**
 * @name forward declarations
*/
//@{
REF_FORWARD_DECL(hls);
REF_FORWARD_DECL(datapath_creator);
class xml_element;
//@}

/**
 * @class datapath
 * Generic class managing datapath creation algorithms.
 */
class datapath_creator : public HLS_step
{

   public:

      /// datapath creation algorithms implemented
      enum datapath_type
      {
         CLASSIC = 0
      };

   protected:

      /// the current datapath type
      datapath_type type;

   public:

      /**
       * This is the constructor of the datapath class.
       */
      datapath_creator(const ParameterRef _Param, const hlsRef _HLS);

      /**
       * Destructor.
       */
      virtual ~datapath_creator();

      /**
       * Creates "creator" based on selected option
       */
      static
      datapath_creatorRef create_creator(datapath_type type, const ParameterRef Param, const hlsRef HLS);

      /**
       * Creates "creator" based on the XML configuration
       */
      static
      datapath_creatorRef xload(const xml_element* node, const ParameterRef Param, const hlsRef HLS);

};

/// RefCount definition for datapath class
typedef refcount<datapath_creator> datapath_creatorRef;

#endif
