/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file conn_binding_creator.cpp
 * @brief Base class for all interconnection binding algorithms.
 *
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/

///Autoheader include
#include "config_HAVE_EXPERIMENTAL.hpp"

#include "conn_binding_creator.hpp"

///implemented algorithms
#include "mux_connection_binding.hpp"

#include "function_behavior.hpp"
#include "behavioral_helper.hpp"

#include "hls.hpp"

#include "polixml.hpp"
#include "xml_helper.hpp"

#include "Parameter.hpp"

conn_binding_creator::conn_binding_creator(const ParameterRef _Param, const hlsRef _HLS) :
   HLS_step(_Param, _HLS), output_level(_Param->getOption<int>(OPT_output_level))
{

}

conn_binding_creator::~conn_binding_creator()
{

}

conn_binding_creatorRef conn_binding_creator::xload(const xml_element* node, const ParameterRef Param, const hlsRef HLS)
{
   std::string algorithm = "MUX_INTERCONNECTION_BINDING";
   if (CE_XVM(algorithm, node)) LOAD_XVM(algorithm, node);

   algorithm_t alg_t = MUX_INTERCONNECTION_BINDING;
   if (algorithm == "MUX_INTERCONNECTION_BINDING")
      alg_t = MUX_INTERCONNECTION_BINDING;
   else
      THROW_ERROR("Datapath creation algorithm \"" + algorithm + "\" currently not supported");
   return create_solver(alg_t, Param, HLS);
}

conn_binding_creatorRef conn_binding_creator::create_solver(algorithm_t type, const ParameterRef Param, const hlsRef HLS)
{
   switch(type)
   {
      case MUX_INTERCONNECTION_BINDING:
         return conn_binding_creatorRef(new mux_connection_binding(Param, HLS));
      default:
          THROW_ERROR("Connection type not yet supported");
   }
   PANDA_UNREACHABLE();
   return conn_binding_creatorRef();
}


void conn_binding_creator::add_parameter_ports()
{
   const behavioral_helperRef BH = HLS->FB->get_helper();

   ///list containing the parameters of the original function (representing input and output values)
   const std::list<unsigned int>& parameters = BH->get_parameters();
   PRINT_DBG_STRING(DEBUG_LEVEL_PEDANTIC, debug_level, "Parameters values: ");
   for (std::list<unsigned int>::const_iterator i = parameters.begin(); i != parameters.end(); i++)
   {
      PRINT_DBG_STRING(DEBUG_LEVEL_PEDANTIC, debug_level, BH->PrintVariable(*i) + "(");
      PRINT_DBG_STRING(DEBUG_LEVEL_PEDANTIC, debug_level, "IN)-@"+STR(*i) + " ");
      input_ports[*i] = HLS->Rconn->bind_port(*i, conn_binding::IN);
   }
   if (BH->has_return())
   {
      unsigned int var = BH->get_function_return(BH->get_function_index());
      PRINT_DBG_STRING(DEBUG_LEVEL_PEDANTIC, debug_level, "Return format " + BH->print_type(var) + " (OUT) ");
      output_ports[var] = HLS->Rconn->bind_port(var, conn_binding::OUT);
   }
   PRINT_DBG_MEX(DEBUG_LEVEL_PEDANTIC, debug_level, "");
}

