/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file conn_binding_creator.hpp
 * @brief Base class for all interconnection binding algorithms.
 *
 * @defgroup conn_binding_creator_p Interconnection Binding
 * @ingroup binding_p
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 * $Locker:  $
 * $State: Exp $
 *
*/
#ifndef _CONN_BINDING_CREATOR_HPP_
#define _CONN_BINDING_CREATOR_HPP_

#include "HLS_step.hpp"

#include "refcount.hpp"
/**
 * @name forward declarations
*/
//@{
REF_FORWARD_DECL(hls);
REF_FORWARD_DECL(conn_binding_creator);
REF_FORWARD_DECL(Parameter);
REF_FORWARD_DECL(generic_obj);
class xml_element;
//@}

#include <map>

/**
 * @defgroup interconnect Interconnection allocation and binding
 * @ingroup HLS
 *
 * Into this subproject, interconnections are computed and then allocated to connect elements into datapath.
 */

/**
 * @class conn_binding_creator
 * @ingroup conn_binding_creator_p
 * Generic class managing interconnection binding algorithms.
 */
class conn_binding_creator : public HLS_step
{
   protected:

      ///map between input port variable and generic object
      std::map<unsigned int,generic_objRef> input_ports;

      ///map between output port variable and generic object
      std::map<unsigned int,generic_objRef> output_ports;

      /**
       * Add ports representing parameter values
       */
      void add_parameter_ports();

      /// verbosity level
      int output_level;

   public:

      ///interconnection architecture style
      typedef enum
      {
         MUX = 0
      }
      connection_mode_type;

      ///interconnection type
      typedef enum
      {
         MUX_INTERCONNECTION_BINDING = 0
      }
      algorithm_t;

      /**
       * Constructor.
       * @param Param is the parameter datastructure
       * @param HLS is the datastructure containing all the intermediate HLS results
       */
      conn_binding_creator(const ParameterRef Param, const hlsRef HLS);

      /**
       * Destructor.
       */
      virtual ~conn_binding_creator();

      /**
       * Creates solver based on selected option
       * @param type is the type of connection creation that is required
       * @param Param is the parameter datastructure
       * @param HLS is the datastructure containing all the intermediate HLS results
       * @return a reference to the interconnection binding specialization
       */
      static
      conn_binding_creatorRef create_solver (algorithm_t type, const ParameterRef Param, const hlsRef HLS);

      /**
       * Creates solver based on the XML configuration
       * @param type is the type of connection creation that is required
       * @param Param is the parameter datastructure
       * @param HLS is the datastructure containing all the intermediate HLS results
       * @return a reference to the interconnection binding specialization
       */
      static
      conn_binding_creatorRef xload(const xml_element* node, const ParameterRef Param, const hlsRef HLS);

};
///refcount definition of the class
typedef refcount<conn_binding_creator> conn_binding_creatorRef;

#endif
