/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file Object.hpp
 * @brief Base class for all resources into datapath
 *
 *
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/

#ifndef CONN_ELEMENT_HPP
#define CONN_ELEMENT_HPP

#include "refcount.hpp"

#include "Variable.hpp"
class graph;

///data to be transferred <tree_node, bitsize, from, to>, where from/to can be operations or STG states
typedef std::tr1::tuple<unsigned int, unsigned int, vertex, vertex> temporary;

/**
 * @class connection_obj
 * Generic class managing elements used to interconnect generic objects into datapath
 */
class connection_obj
{
   public:

      ///resource type
      typedef enum
      {
         DIRECT_CONN,
         BY_MUX
      } element_t;

   protected:

      ///type of the connection
      element_t type;

      ///Set of variables that cross the connection
      std::set<temporary> live_variable;
      
   public:

      /**
       * Constructor.
       * @param _type is the type of the interconnection
       * @param _live_variable is the set of variables across the connection
       */
      connection_obj(element_t _type, const std::set<temporary>& _live_variable) :
         type(_type),
         live_variable(_live_variable)
      {}

      /**
       * Destructor.
       */
      virtual ~connection_obj() {}

      /**
       * Returns the name associated with the element
       * @return a string containing the name associated to element.
       */
      virtual const std::string get_string() const = 0;

      /**
       * Adds a temporary to set of ones that could cross the connection
       * @param var is the new temporary
       */
      void add_variable(unsigned int tree_index, unsigned int bitsize, vertex from, vertex to)
      {
         live_variable.insert(temporary(tree_index, bitsize, from, to));
      }

      /**
       * Gets the temporary set
       * @return the set of temporary that could cross the connection
       */
      std::set<temporary> get_variables1() const
      {
         return live_variable;
      }

      /**
       * Returns type of object used to perform connection
       * @return an integer associated to object type
       */
      unsigned int get_type() const { return type; }
};

/// RefCount definition for connection_obj class
typedef refcount<connection_obj> connection_objRef;

#endif
