/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file unique_binding.hpp
 * @brief Class to manage module binding
 *
 * This is the header file to manage module binding algorithms.
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * @version $Revision: 10608 $
 * @date $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
*/

#ifndef UNIQUE_BINDING_HPP
#define UNIQUE_BINDING_HPP

#include "refcount.hpp"

#include "fu_binding_creator.hpp"

/**
 * @name Forward declarations.
 */
//@{
REF_FORWARD_DECL(Parameter);
REF_FORWARD_DECL(hls);
//@}

/**
 * Class managing the module allocation.
 */
class unique_binding : public fu_binding_creator
{

   public:

      /**
       * This is the constructor of the class.
       */
      unique_binding(const ParameterRef Param, const hlsRef HLS);

      /**
       * Destructor.
       */
      ~unique_binding();

      /**
       * Performs module binding assigning each operation to a free resource
       */
      void exec();

      /**
       * Returns the name of the implemented algorithm
       */
      std::string get_kind_text() const;

};

#endif
