/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file fu_binding_creator.cpp
 * @brief Implementation of module binding class.
 *
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#include "fu_binding_creator.hpp"

///implemented algorithms
#include "unique_binding.hpp"
#include "cdfc_module_binding.hpp"

#include "polixml.hpp"
#include "xml_helper.hpp"
#include "Parameter.hpp"
#include "exceptions.hpp"

fu_binding_creator::fu_binding_creator(const ParameterRef _Param, const hlsRef _HLS) :
   HLS_step(_Param, _HLS),
   output_level(_Param->getOption<int>(OPT_output_level))
{

}

fu_binding_creator::~fu_binding_creator()
{

}

fu_binding_creatorRef fu_binding_creator::xload(const xml_element* node, const ParameterRef Param, const hlsRef HLS)
{
   std::string algorithm = "TTT_FAST";
   if (CE_XVM(algorithm, node)) LOAD_XVM(algorithm, node);

   type_t algorithm_t = CDFC_TTT_FAST;
   if (algorithm == "TTT_FAST")
      algorithm_t = CDFC_TTT_FAST;
   else if (algorithm == "TTT_FULL")
      algorithm_t = CDFC_TTT_FULL;
   else if (algorithm == "COLORING")
      algorithm_t = CDFC_COLORING;
   else if (algorithm == "WEIGHTED_COLORING")
      algorithm_t = CDFC_WEIGHTED_COLORING;
   else if (algorithm == "BIPARTITE_MATCHING")
      algorithm_t = CDFC_BIPARTITE_MATCHING;
   else if (algorithm == "UNIQUE")
      algorithm_t = UNIQUE;
   else
      THROW_ERROR("FU binding algorithm \"" + algorithm + "\" currently not supported");
   return create_solver(algorithm_t, Param, HLS);
}

fu_binding_creatorRef fu_binding_creator::create_solver(type_t type, const ParameterRef Param, const hlsRef HLS)
{
   switch(type)
   {
      case CDFC_TTT_FAST:
        return fu_binding_creatorRef(new cdfc_module_binding(Param, HLS, cdfc_module_binding::TTT_FAST));
      case CDFC_TTT_FULL:
        return fu_binding_creatorRef(new cdfc_module_binding(Param, HLS, cdfc_module_binding::TTT_FULL));
      case CDFC_COLORING:
        return fu_binding_creatorRef(new cdfc_module_binding(Param, HLS, cdfc_module_binding::COLORING));
      case CDFC_WEIGHTED_COLORING:
        return fu_binding_creatorRef(new cdfc_module_binding(Param, HLS, cdfc_module_binding::WEIGHTED_COLORING));
      case CDFC_BIPARTITE_MATCHING:
         return fu_binding_creatorRef(new cdfc_module_binding(Param, HLS, cdfc_module_binding::BIPARTITE_MATCHING));
      case UNIQUE:
        return fu_binding_creatorRef(new unique_binding(Param, HLS));
      default:
        THROW_ERROR("Module binding algorithm not yet supported");
   }
   PANDA_UNREACHABLE();
   return fu_binding_creatorRef();
}
