/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file fu_binding_creator.hpp
 * @brief Base class for all module binding algorithms.
 *
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef _FU_BINDING_CREATOR_HPP
#define _FU_BINDING_CREATOR_HPP

///superclass include
#include "HLS_step.hpp"

#include "refcount.hpp"
/**
 * @name forward declarations
*/
//@{
REF_FORWARD_DECL(hls);
REF_FORWARD_DECL(fu_binding_creator);
REF_FORWARD_DECL(Parameter);
class xml_element;
//@}

/**
 * Generic class managing module binding algorithms.
*/
class fu_binding_creator : public HLS_step
{
    public:

      /// Module binding algorithms
      typedef enum
      {
         CDFC_TTT_FAST = 0,
         CDFC_TTT_FULL,
         CDFC_COLORING,
         CDFC_WEIGHTED_COLORING,
         CDFC_BIPARTITE_MATCHING,
         UNIQUE
      } type_t;

      /**
        * Constructor.
        * @param Param is the class containing all the parameters
        * @param HLS is the datastructure containing all the HLS intermediate results
        */
      fu_binding_creator(const ParameterRef Param, const hlsRef HLS);

      /**
        * Destructor.
        */
      virtual ~fu_binding_creator();

      /**
        * Creates solver based on selected option
        * @param type is the desidered scheduling solver
        * @param Param is the class containing all the parameters
        * @param HLS is the datastructure containing all the HLS intermediate results
        * @return reference to scheduling specialization
        */
      static
      fu_binding_creatorRef create_solver(type_t type, const ParameterRef Param, const hlsRef HLS);

      /**
        * Creates solver based on the XML configuration
        * @param node is the XML node
        * @param Param is the class containing all the parameters
        * @param HLS is the datastructure containing all the HLS intermediate results
        * @return reference to scheduling specialization
        */
      static
      fu_binding_creatorRef xload(const xml_element* node, const ParameterRef Param, const hlsRef HLS);

   protected:

      /// verbosity level
      int output_level;
};

typedef refcount<fu_binding_creator> fu_binding_creatorRef;

#endif
