/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file conflict_based_register.hpp
 * @brief Base class specification for register allocation algorithm based on a conflict graph
 *
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef CONFLICT_BASED_REGISTER_HPP
#define CONFLICT_BASED_REGISTER_HPP

#include "reg_binding_creator.hpp"

#include "refcount.hpp"
/**
 * @name Forward declarations
 */
//@{
REF_FORWARD_DECL(storage_value_insertion);
REF_FORWARD_DECL(Parameter);
REF_FORWARD_DECL(hls);
//@}

#include <boost/graph/adjacency_list.hpp>

class conflict_based_register : public reg_binding_creator
{
   protected:

      typedef boost::adjacency_list<boost::vecS, boost::vecS, boost::undirectedS> conflict_graph;
      typedef boost::graph_traits<conflict_graph>::vertex_descriptor cg_vertex_descriptor;
      typedef boost::graph_traits<conflict_graph>::vertices_size_type cg_vertices_size_type;
      typedef boost::property_map<conflict_graph, boost::vertex_index_t>::const_type cg_vertex_index_map;

      ///conflict graph
      conflict_graph cg;

      boost::iterator_property_map < cg_vertices_size_type*, cg_vertex_index_map, cg_vertices_size_type, cg_vertices_size_type& > color;

   private:

      std::vector<cg_vertices_size_type> color_vec;

   public:

      /**
       * Constructor of the class.
       */
      conflict_based_register(const ParameterRef Param, const hlsRef HLS, const storage_value_insertionRef svi_algorithm);

      /**
       * Destructor of the class.
       */
      virtual ~conflict_based_register();

      /**
       * Create the conflict graph
       */
      void create_conflict_graph();

      /**
       * This class only computes the conflict graph. Thus, the exec method has nothing to do
       */
      virtual void exec();

      /**
       * Returns the name of the algorithm
       */
      virtual std::string get_kind_text() const;

#if HAVE_EXPERIMENTAL
      /**
       * Create the epdg conflict graph
       */
      void create_epdg_conflict_graph();

#endif
};

#endif
