/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file vertex_coloring_register.hpp
 * @brief Class specification of a coloring based register allocation algorithm
 *
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef VERTEX_COLORING_REGISTER_HPP
#define VERTEX_COLORING_REGISTER_HPP

#include "conflict_based_register.hpp"

class vertex_coloring_register : public conflict_based_register
{
   public:

      /**
       * Constructor of the class.
       */
      vertex_coloring_register(const ParameterRef _Param, const hlsRef _HLS, const storage_value_insertionRef _svi_algorithm);

      /**
       * Destructor of the class.
       */
      ~vertex_coloring_register();

      /**
       * Coloring based register allocation algorithm.
       */
      void exec();

      /**
       * Returns the name of the allocation algorithm
       */
      std::string get_kind_text() const;

};

#endif
