/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file reg_binding_creator.hpp
 * @brief Base class for all the register allocation algorithms.
 *
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef _REG_BINDING_CREATOR_HPP_
#define _REG_BINDING_CREATOR_HPP_

///Autoheader include
#include "config_HAVE_EXPERIMENTAL.hpp"

///superclass include
#include "HLS_step.hpp"

#include "refcount.hpp"
/**
 * @name Forward declarations
 */
//@{
REF_FORWARD_DECL(storage_value_insertion);
REF_FORWARD_DECL(Parameter);
REF_FORWARD_DECL(reg_binding_creator);
REF_FORWARD_DECL(hls);
class xml_element;
//@}

/**
 * Generic class managing the different register allocation algorithms.
 */
class reg_binding_creator : public HLS_step
{
   protected:

      /// verbosity level of the class
      int output_level;

      /// lower bound
      unsigned int register_lower_bound;

      /// class that represents the values to be stored in memory
      const storage_value_insertionRef svi_algorithm;

   public:

      /// Register allocation algorithms
      typedef enum
      {
         COLORING=0,
         CHORDAL_COLORING,
         WEIGHTED_COLORING,
         BIPARTITE_MATCHING,
         TTT_CLIQUE_COVERING,
#if HAVE_EXPERIMENTAL
         LEFT_EDGE,
         CYCLIC_ALLOCATION,
         K_COFAMILY,
#endif
         UNIQUE_BINDING
      } type_t;


      /**
       * Constructor
       * @param Param is the class containing all the parameters
       */
      reg_binding_creator(const ParameterRef Param, const hlsRef HLS, const storage_value_insertionRef svi_algorithm);

      /**
       * Destructor.
       */
      virtual ~reg_binding_creator();

      /**
       * Creates the solver based on the selected option
       * @param type is the desired register allocation solver
       * @param Param is the class containing all the parameters
       * @param svi_algorithm is the class managing the variables to be stored
       * @return the reference to the register allocation specialization
       */
      static
      reg_binding_creatorRef create_solver(type_t type, const ParameterRef Param, const hlsRef HLS, const storage_value_insertionRef svi_algorithm);

      /**
       * Creates the solver based on the configuration stored in the given XML node
       * @param node is the XML node where the configuration is stored
       * @param Param is the class containing all the parameters
       * @param svi_algorithm is the class managing the variables to be stored
       * @return the reference to the register allocation specialization
       */
      static
      reg_binding_creatorRef xload(const xml_element* node, const ParameterRef Param, const hlsRef HLS, const storage_value_insertionRef svi_algorithm);

};
///refcount definition of the class
typedef refcount<reg_binding_creator> reg_binding_creatorRef;

#endif
