/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file stg_chaining.cpp
 * @brief class supporting the chaining optimization in high level synthesis
 * @author Vito Giovanni Castellana <vcastellana@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
 */
#include "stg_chaining.hpp"

#include "hls.hpp"
#include "hls_constraints.hpp"

#include "function_behavior.hpp"
#include "Parameter.hpp"
#include "exceptions.hpp"

///STD include
#include <set>
#include <list>
#include <map>
#include "exceptions.hpp"

#include "StateTransitionGraph.hpp"
#include "dbgPrintHelper.hpp"

stg_chaining::stg_chaining(const ParameterRef _Param, const hlsRef _HLS) :
      chaining(_Param, _HLS)
{

}

stg_chaining::~stg_chaining()
{

}


void stg_chaining::exec()
{
   const graph *astg = HLS->STG->get_astg();
   const graph * sdg = HLS->FB->get_graph(function_behavior::SDG);

   VertexIterator s1_It, s1_ItEnd;
   for(tie(s1_It, s1_ItEnd) = boost::vertices(*astg); s1_It != s1_ItEnd; ++s1_It)
   {
      const std::list<vertex>& operations_s1 = GET_ENDING_OPERATION_LIST(astg, *s1_It);
      std::list<vertex>::const_iterator op, opend=operations_s1.end(), opbegin=operations_s1.begin();
      for(op=opbegin; op != opend; op++)
      {
         InEdgeIterator e, e_end;
         for(tie(e, e_end) = boost::in_edges(*op, *sdg);e != e_end; e++)
         {
            vertex pred = boost::source(*e, *sdg);
            if(std::find(opbegin, opend, pred)!=opend)
            {
               PRINT_DBG_MEX(DEBUG_LEVEL_PEDANTIC, debug_level, "-*-*-* operations " << GET_NAME(sdg, pred) << " and " << GET_NAME(sdg, *op) << " are chained in state " << HLS->STG->get_state_name(*s1_It));
               //std::map<vertex, std::set<vertex> >
               if(*op>pred)
                  may_be_chained[*op].insert(pred);
               else
                  may_be_chained[pred].insert(*op);
               //std::map<vertex, std::map<vertex, std::set<vertex> > >
               chained_in_state[*s1_It][*op].insert(pred);
            }
         }
      }
   }
}

std::string stg_chaining::get_kind_text() const
{
   return "stg_chaining_analysis";
}
