/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file stg_chaining.hpp
 * @brief class supporting the chaining optimization in high level synthesis
 * @author Vito Giovanni Castellana <vcastellana@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
 */

#ifndef STG_CHAINING_HPP
#define STG_CHAINING_HPP

#include "chaining.hpp"
#include "refcount.hpp"


/**
 * @name forward declarations
 */

REF_FORWARD_DECL(hls);


class stg_chaining: public chaining
{

   public:

   //FUNCTIONS

   /**
    * Constructor
    * @param Param is the refcount to the class containing all the parameters
    * @param HLS is the class containing all the HLS intermediate results
    */
   stg_chaining(const ParameterRef Param, const hlsRef HLS);

   /**
    * Destructor
    */
   ~stg_chaining();

   void exec();


   /**
    * Returns the name of the algorithms
    */
   virtual std::string get_kind_text() const;


};

typedef refcount<stg_chaining> stg_chainingRef;

#endif
