/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file chaining.cpp
 * @brief class supporting the chaining optimization in high level synthesis
 * @author Vito Giovanni Castellana <vcastellana@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
 */

#include "config_HAVE_EXPERIMENTAL.hpp"
#include "chaining.hpp"
#if HAVE_EXPERIMENTAL
#include "cdfg_chaining.hpp"
#endif
#include "stg_chaining.hpp"

#include "hls.hpp"
#include "Parameter.hpp"
#include "exceptions.hpp"
#include "dbgPrintHelper.hpp"

///STD include
#include <set>

chaining::chaining(const ParameterRef _Param, const hlsRef _HLS) :
      HLS_step(_Param, _HLS)
{


}

chaining::~chaining()
{

}


chainingRef chaining::create_solver(chaining_algorithm algorithm, const ParameterRef Param, const hlsRef HLS)
{
   switch(algorithm)
   {
      case STG_CHAINING:
         return chainingRef(new stg_chaining(Param, HLS));
#if HAVE_EXPERIMENTAL
      case CDFG_CHAINING:
         return chainingRef(new cdfg_chaining(Param, HLS));
#endif
      default:
         THROW_ERROR("Chaining method not yet supported");
   }
   PANDA_UNREACHABLE();
   return chainingRef();
}


bool chaining::is_chained_vertex(vertex v)
{
   bool cond = false;
   if(chaining_with.find(v)!=chaining_with.end()) cond = true;
   return cond;
}

std::vector<vertex> chaining::get_chaining_with_vertices(vertex op)
{
   if(!is_chained_vertex(op)) THROW_ERROR("Trying to get chaining predecessor of a non-chainable vertex");
   return chaining_with[op];
}

bool chaining::is_chained_with_vertex(vertex tgt, vertex src)
{
   if(!is_chained_vertex(tgt)) return false;
   bool cond=false;
   std::vector<vertex> temp = chaining_with[tgt];
   if(std::find(temp.begin(),temp.end(),src)!=temp.end()) cond = true;
   return cond;
}

bool chaining::may_be_chained_ops(vertex tgt, vertex src)
{
   if(tgt<=src)
      std::swap(tgt,src);
   if(may_be_chained.find(tgt)==may_be_chained.end())
      return false;
   else
      return (may_be_chained[tgt].find(src)!=may_be_chained[tgt].end());
}

bool chaining::are_chained_in_state(vertex state, vertex tgt, vertex src)
{
   if(chained_in_state.find(state)==chained_in_state.end())
      return false;
   else
   {
      std::map<vertex, std::set<vertex> > temp = chained_in_state[state];
      if(temp.find(tgt)==temp.end())
         return false;
      else
         return (temp[tgt].find(src)!=temp[tgt].end());
   }
}

