/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file virtual_hls.hpp
 * @brief Base class for simple hls algorithm
 *
 *
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/

#ifndef VIRTUAL_HLS_HPP
#define VIRTUAL_HLS_HPP

#include "refcount.hpp"

/**
 * @name Forward declarations.
*/
//@{
REF_FORWARD_DECL(Parameter);
REF_FORWARD_DECL(hls);
REF_FORWARD_DECL(call_graph);
#include "hls_flow.hpp"
//@}

/**
 * Generic class managing simple hls algorithm
*/
class virtual_hls : public hls_flow
{
   public:

      /**
       * This is the constructor of the virtual_hls class.
       * @param _Param is the set of the parameters
       * @param _HLS is the HLS datastructure
       */
      virtual_hls(const ParameterRef _Param, const hlsRef _HLS);

      /**
       * Destructor
       */
      virtual ~virtual_hls();

      /**
       * Performs the base HLS steps (i.e., scheduling, allocation and binding)
       */
      virtual void exec();

};

///
typedef refcount<virtual_hls> virtual_hlsRef;

#endif
