/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file hls_target.hpp
 * @brief Data structure representing the target information for the HLS
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef _HLS_TARGET_HPP_
#define _HLS_TARGET_HPP_

///superclass include
#include "target_manager.hpp"

/**
 * @name forward declarations
 */
//@{
REF_FORWARD_DECL(target_technology);
REF_FORWARD_DECL(technology_manager);
REF_FORWARD_DECL(target_device);
//@}

class HLS_target
{
   protected:

      ///reference to the target technology
      target_technologyRef target;

      ///reference to the container of the technology libraries
      technology_managerRef TM;

      ///reference to the information about the target device
      target_deviceRef device;

   public:

      /**
       * Constructor
       */
      HLS_target();

      /**
       * Destructor
       */
      ~HLS_target();

      /**
       * Sets the reference to the target technology manager class
       */
      void set_target_technology(const target_technologyRef _target);

      /**
       * Returns the reference to the class representing the target technology
       */
      const target_technologyRef get_target_technology() const;

      /**
       * Sets the reference to the technology manager class
       */
      void set_technology_manager(const technology_managerRef _TM);

      /**
       * Returns the reference to the class containing all the technology libraries
       */
      const technology_managerRef get_technology_manager() const;

      /**
       * Sets the reference to the target device class
       */
      void set_target_device(const target_deviceRef _device);

      /**
       * Returns the reference to the class containing information about the target device
       */
      const target_deviceRef get_target_device() const;

};
///refcount definition of class
typedef refcount<HLS_target> HLS_targetRef;

#endif
