/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file liveness_computer.cpp
 * @brief Some method for the abstract class
 *
 * This class creates the specialization to perform the computation of the liveness
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#include "liveness_computer.hpp"
#include "config_HAVE_EXPERIMENTAL.hpp"
#include "FSM_NI_SSA_liveness.hpp"
#include "memory.hpp"

#include "polixml.hpp"
#include "xml_helper.hpp"
#include "tree_helper.hpp"
#include "hls.hpp"
#include "hls_constraints.hpp"

///Behavior include
#include "application_manager.hpp"

#if HAVE_EXPERIMENTAL
#include "epdg_dataflow_analysis.hpp"
#include "dummy_liveness_analysis.hpp"
#endif

liveness_computer::liveness_computer(const ParameterRef _Param, const hlsRef _HLS) :
   HLS_step(_Param, _HLS)
{

}

liveness_computer::~liveness_computer()
{

}

liveness_computerRef liveness_computer::xload(const xml_element* node, const ParameterRef Param, const hlsRef HLS)
{
   std::string algorithm;
   if (CE_XVM(algorithm, node)) LOAD_XVM(algorithm, node);
   else algorithm = "FSM_NI_SSA_LIVENESS";

   liveness_algorithm algorithm_t;
   if (algorithm == "FSM_NI_SSA_LIVENESS")
      algorithm_t = FSM_NI_SSA_LIVENESS;
   else
      THROW_ERROR("Liveness algorithm \"" + algorithm + "\" currently not supported");
   return create_solver(algorithm_t, Param, HLS);
}

liveness_computerRef liveness_computer::create_solver(liveness_algorithm algorithm, const ParameterRef Param, const hlsRef HLS)
{
   switch(algorithm)
   {
      case FSM_NI_SSA_LIVENESS:
         return liveness_computerRef(new FSM_NI_SSA_liveness(Param, HLS));
#if HAVE_EXPERIMENTAL
//      case EPDG_DATAFLOW:
//         return liveness_computerRef(new epdg_dataflow_analysis(Param, HLS));
      case DUMMY_LIVENESS:
         return liveness_computerRef(new dummy_liveness_analysis(Param, HLS));
#endif
      default:
         THROW_ERROR("Dataflow method not yet supported");
   }
   PANDA_UNREACHABLE();
   return liveness_computerRef();
}

bool liveness_computer::is_register_compatible(unsigned int ssa) const
{
   return tree_helper::is_ssa_name(HLS->AppM->get_tree_manager(), ssa) &&
         !tree_helper::is_virtual(HLS->AppM->get_tree_manager(), ssa) && /// virtual ssa_name is not considered
         !HLS->HLS_C->Rmem->has_base_address(ssa) && /// ssa_name allocated in memory
         !tree_helper::is_parameter(HLS->AppM->get_tree_manager(), ssa); /// parameters have been already stored in a register by the calling function
}
