/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file liveness_computer.hpp
 * @brief Class specification for algorithms that compute the liveness of variables
 *
 * This class is abstract and it has to be specialized in order to implement the desider algorithm
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/

#ifndef LIVENESS_COMPUTER_HPP
#define LIVENESS_COMPUTER_HPP

#include "HLS_step.hpp"

#include "refcount.hpp"
#include "config_HAVE_EXPERIMENTAL.hpp"

/**
 * @name Forward declarations.
 */
//@{
REF_FORWARD_DECL(liveness_computer);
REF_FORWARD_DECL(hls);
REF_FORWARD_DECL(Parameter);
class xml_element;
//@}

class liveness_computer : public HLS_step
{
   public:

      typedef enum
      {
         FSM_NI_SSA_LIVENESS = 0
#if HAVE_EXPERIMENTAL
         ,
         EPDG_DATAFLOW,
         DUMMY_LIVENESS
#endif
   } liveness_algorithm;

      /**
       * Constructor
       * @param Param is the refcount to the class containing all the parameters
       */
      liveness_computer(const ParameterRef Param, const hlsRef HLS);

      /**
       * Destructor
       */
      virtual ~liveness_computer();
    
      /**
       * Factory method.
       */
      static
      liveness_computerRef create_solver(liveness_algorithm algorithm, const ParameterRef Param, const hlsRef HLS);

      /**
       * Factory method from XML node
       */
      static
      liveness_computerRef xload(const xml_element* node, const ParameterRef Param, const hlsRef HLS);

      /**
       * is the ssa_name allocable on a register
       */
      bool is_register_compatible(unsigned int ssa) const;

};

//refcount definition for class
typedef refcount<liveness_computer> liveness_computerRef;

#endif

