/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file memory_allocation.hpp
 * @brief Base class to allocate memories in high-level synthesis
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/

#ifndef _MEMORY_ALLOCATION_HPP_
#define _MEMORY_ALLOCATION_HPP_

#include "refcount.hpp"
/**
 * @name forward declarations
*/
//@{
REF_FORWARD_DECL(application_manager);
REF_FORWARD_DECL(HLS_target);
REF_FORWARD_DECL(HLS_constraints);
REF_FORWARD_DECL(Parameter);
//@}

#include "utility.hpp"

/**
 * Allocation memory class
 */
class memory_allocation
{
   public:

      /// Memory allocation algorithms
      typedef enum
      {
         GSS = 0,    /// all global variables, static variables and strings are allocated on BRAMs
         ALL_BRAM, /// all objects that need to be stored in memory are allocated on BRAMs
         NO_BRAM /// all objects that need to be stored in memory are allocated on an external memory
      } type_t;

   protected:

      /// class containing all the parameters
      const ParameterRef Param;

      /// debugging level of the class
      int debug_level;

      /// verbosity level of the class
      int output_level;

      /// application manager
      const application_managerRef AppM;

      /// HLS constraints
      const HLS_constraintsRef HLS_C;

      /// this is the implemented algorithm
      type_t allocation_algorithm;

   public:

      /**
       * Constructor
       */
      memory_allocation(const ParameterRef Param,
                        const application_managerRef AppM, const HLS_constraintsRef HLS_C);

      /**
       * Destructor
       */
      ~memory_allocation();

      /**
       * Executes the memory allocation and create the memory datastructure
       */
      void exec();
};
///refcount definition of the class
typedef refcount<memory_allocation> memory_allocationRef;

#endif
