/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file moduleGenerator.hpp
 * @brief
 *
 *
 *
 * @author Alessandro Nacci <alenacci@gmail.com>
 * @author Gianluca Durelli <durellinux@gmail.com>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/

#include "HLS_step.hpp"

#include "refcount.hpp"
#include <vector>
#include <tr1/tuple>

#include "function_behavior.hpp"
#include "structural_objects.hpp"
#include "structural_manager.hpp"

/**
 * @name forward declarations
 */
//@{
REF_FORWARD_DECL(structural_object);
REF_FORWARD_DECL(application_manager);
REF_FORWARD_DECL(Parameter);
//@}


class moduleGenerator
{
   protected:

      ///class containing all the parameters
      const ParameterRef Param;

      ///debugging level of the class
      unsigned int debug_level;

      ///class containing the specification of the application
      const application_managerRef AppM;

   public:

      /**
       * Constructor.
       */
      moduleGenerator(const ParameterRef Param, const application_managerRef AppM);

      /**
       * Destructor.
       */
      virtual ~moduleGenerator();
      
      structural_type_descriptorRef getDataType(unsigned int variable, const function_behaviorRef FB);

      void add_port_parameters(structural_objectRef generated_port,structural_objectRef currentPort);

      std::string generate_verilog(std::string verilog_template, std::vector<std::tr1::tuple<unsigned int,unsigned int> >& required_variables, const function_behaviorRef FB, std::string path_dynamic_generators);

      std::string get_specialized_name(std::vector<std::tr1::tuple<unsigned int,unsigned int> >& required_variables, const function_behaviorRef FB);

      void specialize_fu(std::string fuName, vertex ve, std::string libraryId, const technology_managerRef TM, const function_behaviorRef FB, std::string new_fu_name, std::map<std::string,technology_nodeRef> & new_fu);

      /**
       * Nothing to do
       */
      void exec();

      /**
       * Returns the name of the step
       */
      std::string get_kind_text() const;

};

///Refcount type definition of the class
typedef refcount<moduleGenerator> moduleGeneratorRef;
