/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file priority.cpp
 * @brief set of classes used to define different priority schemes.
 *
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 * $Locker:  $
 * $State: Exp $
 *
*/

#include "priority.hpp"
#include "ASLAP.hpp"

#include "schedule.hpp"

priority_static_mobility::priority_static_mobility(const ASLAPRef &aslap)
{
   const schedule & asap = aslap->get_ASAP();
   const schedule & alap = aslap->get_ALAP();
   
   const graphRef data = aslap->get_graph();
   VertexIterator vIt, vEnd;
   for(tie(vIt, vEnd) = boost::vertices(*data); vIt != vEnd; vIt++)
   {
      if (!asap.is_scheduled(*vIt)) continue;
      operator[](*vIt) = static_cast<int>(- alap.get_cstep(*vIt) + asap.get_cstep(*vIt)); ///Note that usually high priority in list based means low mobility.
   }
}

priority_dynamic_mobility::priority_dynamic_mobility(const ASLAPRef &aslap, const std::set<vertex> &_ready_nodes) : ready_nodes(_ready_nodes)
{
   const schedule & asap = aslap->get_ASAP();
   const schedule & alap = aslap->get_ALAP();
   
   const graphRef data = aslap->get_graph();
   VertexIterator vIt, vEnd;
   for(tie(vIt, vEnd) = boost::vertices(*data); vIt != vEnd; vIt++)
   {
      if (!asap.is_scheduled(*vIt)) continue;
      operator[](*vIt) = static_cast<int>(- alap.get_cstep(*vIt) + asap.get_cstep(*vIt)); ///Note that usually high priority in list based means low mobility.
   }
}

bool priority_dynamic_mobility::update()
{
   std::set<vertex>::const_iterator it_end = ready_nodes.end();
   std::set<vertex>::const_iterator it = ready_nodes.begin();
   if(it != it_end)
   {
      while(it != it_end)
      {
         operator[](*it)++; ///increase priority
         it++;
      }
      return true;
   }
   else
      return false;
}

priority_fixed::priority_fixed(const std::tr1::unordered_map<vertex,int>& priority_value)
{
   std::tr1::unordered_map<vertex,int>::const_iterator it_end = priority_value.end();
   for(std::tr1::unordered_map<vertex,int>::const_iterator it = priority_value.begin(); it != it_end; it++)
      operator[](it->first) = it->second;
}
