/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file HLStestbench.hpp
 * @brief Class to compute testbenches for high-level synthesis
 *
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10609 $
 * $Date: 2012-03-12 11:40:21 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: pilato $
 *
 */

#ifndef HLS_TESTBENCH_HPP
#define HLS_TESTBENCH_HPP

#define SIMULATION_DEFINE "ONLY_SIMULATION"

#include "refcount.hpp"

/**
 * @name forward declarations
 */
//@{
REF_FORWARD_DECL(Parameter);
REF_FORWARD_DECL(application_manager);
CONSTREF_FORWARD_DECL(tree_manager);
REF_FORWARD_DECL(tree_manager);
REF_FORWARD_DECL(technology_manager);
REF_FORWARD_DECL(HLS_constraints);
REF_FORWARD_DECL(behavioral_helper);
REF_FORWARD_DECL(structural_object);
REF_FORWARD_DECL(memory);
REF_FORWARD_DECL(language_writer);
//@}

#include <vector>
#include <string>
#include <map>

class HLStestbench
{
      ///class containing all the parameters
      const ParameterRef Param;

      ///debug level of the class
      int debug_level;

      ///output directory
      std::string output_directory;

      unsigned int parse_test_vectors(const application_managerRef AppM, const std::string& tb_input_xml, std::vector<std::map<std::string, std::string> >& test_vectors);

      std::string creating_c_testbench(const application_managerRef AppM, const HLS_constraintsRef& HLS_C, std::map<std::string, std::string>& test_vectors, const memoryRef& Rmem, const std::string& input_file);

      void generate_input_file(const std::string& fileName);

      std::string convert_in_binary(const behavioral_helperRef& BH, unsigned int base_type, const std::string &C_value, unsigned int precision);

      std::string print_var_init(const behavioral_helperRef& BH, const tree_managerRef& TreeM, unsigned int var, const memoryRef& mem);

      bool is_all_8zeros(std::string & str);

      // store the address value of the parameters
      std::map<unsigned int, unsigned int> param_address;

      void write_testbench(std::ostream& hdl_file, language_writerRef writer, const structural_objectRef &cir, double target_period, std::string simulation_values_path, bool generate_vcd_output, bool xilinx_isim, const tree_managerConstRef TreeM) const;

   public:

      /**
       * Constructor.
       */
      HLStestbench(const ParameterRef& Param);

      /**
       * Destructor.
       */
      virtual ~HLStestbench();

      /**
       * Generates all the files required for the testbench
       */
      void generate_testbench(const application_managerRef AppM, const technology_managerRef& TM, const HLS_constraintsRef& HLS_C, const memoryRef& mem);

      /**
       * Creates the testbench file associated with the given component
       */
      std::string create_testbench(const std::string & filename_bench, const structural_objectRef &cir, const double target_period, bool xilinx_isim, const tree_managerConstRef TreeM) const;

};

///Refcount type definition of the class
typedef refcount<HLStestbench> HLStestbenchRef;

#endif
