/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file STG_creator.hpp
 * @brief Base class for all the STG creation algorithms.
 *
 * This class is a pure virtual one, that has to be specilized in order to implement a particular algorithm to create
 * the state transition graph.
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/

#ifndef _STG_CREATOR_HPP_
#define _STG_CREATOR_HPP_

#include "HLS_step.hpp"

#include "refcount.hpp"
/**
 * @name forward declarations
*/
//@{
REF_FORWARD_DECL(hls);
REF_FORWARD_DECL(STG_creator);
REF_FORWARD_DECL(Parameter);
class xml_element;
//@}

/**
 * Generic class managing all the stg creation algorithms.
 */
class STG_creator : public HLS_step
{
   public:

      ///state transition graph creation algorithms implemented
      enum stg_type
      {
         BB_BASED = 0
      };


   protected:

      ///current type of the algorithm adopted for the creation of the STG
      stg_type type;

   public:

      /**
       * Constructor.
       * @param Param is the class containing all the parameters
       * @param HLS is the data structure storing all the intermediate HLS results.
       */
      STG_creator(const ParameterRef Param, const hlsRef HLS);

      /**
       * Destructor.
       */
      virtual ~STG_creator();

      /**
       * Creates the proper algorithm based on the specified option
       * @param type is the desidered stg creation algorithm
       * @param Param is the class containing all the parameters
       * @param HLS is the data structure storing all the intermediate HLS results.
       * @return the reference to stg creator specialization
       */
      static
      STG_creatorRef create_creator(stg_type type, const ParameterRef Param, const hlsRef HLS);

      /**
       * Creates the proper algorithm starting from the XML node
       * @param node is the the XML node
       * @param Param is the class containing all the parameters
       * @param HLS is the data structure storing all the intermediate HLS results.
       * @return the reference to stg creator specialization
       */
      static
      STG_creatorRef xload(const xml_element* node, const ParameterRef Param, const hlsRef HLS);

};
///refcount definition of the class
typedef refcount<STG_creator> STG_creatorRef;

#endif
