/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file hls_instruction_writer.cpp
 * @brief Simple class to print single instruction
 *
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/

///Header include
#include "hls_instruction_writer.hpp"

///Backend include
#include "c_writer.hpp"

///Behavior include
#include "behavioral_helper.hpp"
#include "function_behavior.hpp"

///Parameter include
#include "Parameter.hpp"

///Utility include
#include "utility.hpp"

HLSInstructionWriter::HLSInstructionWriter(const ParameterConstRef _parameters) :
   InstructionWriter(_parameters)
{
   debug_level = _parameters->get_class_debug_level(GET_CLASS(*this));
}

HLSInstructionWriter::~HLSInstructionWriter()
{}

void HLSInstructionWriter::declareFunction(std::ostringstream& stream, simple_indent &, const var_pp_functorConstRef varFunctor, const routineFunctorConstRef codeFunc)
{
   //All I have to do is to change main in _main
   const behavioral_helperConstRef BH = codeFunc->FB->get_helper();
   unsigned int funId = BH->get_function_index();
   std::string stringTemp = BH->print_type(funId, false, true, false, 0, varFunctor);
   std::string name = BH->get_function_name();
   if(name == "main")
   {
      boost::replace_all(stringTemp, " main(", " _main("); /// the assumption is strong but the code that prints the name of the function is under our control ;-)
   }
   stream << stringTemp;
}
