/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file instruction_writer.cpp
 * @brief Simple class to print single instruction
 *
 * @author Luca Fossati <fossati@elet.polimi.it>
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/

///Header include
#include "instruction_writer.hpp"

///Backend include
#include "c_writer.hpp"
#include "prettyPrintVertex.hpp"
#include "simple_indent.hpp"

///Behavior include
#include "function_behavior.hpp"
#include "behavioral_helper.hpp"
#include "basic_block.hpp"
#include "loop.hpp"
#include "loops.hpp"

///Graph include
#include "graph.hpp"

///Parameter include
#include "Parameter.hpp"

///STD include
#include <fstream>
#include <iostream>
#include <ostream>
#include <math.h>

///Utility include
#include "refcount.hpp"

InstructionWriter::InstructionWriter(const ParameterConstRef _parameters) :
   parameters(_parameters)
{
   debug_level = parameters->get_class_debug_level(GET_CLASS(*this));
}

InstructionWriter::~InstructionWriter()
{}

void InstructionWriter::write(std::ostringstream& stream, const vertex v, simple_indent &indenter, const var_pp_functorConstRef varFunctor, const routineFunctorConstRef codeFunc, const bool)
{
   const behavioral_helperRef& BH = codeFunc->FB->get_helper();
   const graph * inGraph = codeFunc->FB->get_graph(function_behavior::CFG);
   BH->print_vertex(stream, inGraph, v, varFunctor, indenter);
}

void InstructionWriter::declareFunction(std::ostringstream& stream, simple_indent &indenter, const var_pp_functorConstRef varFunctor, const routineFunctorConstRef codeFunc)
{
   const behavioral_helperRef& BH = codeFunc->FB->get_helper();
   unsigned int funId = BH->get_function_index();
   prettyPrintVertex::print_prototype_declaration(stream, funId, indenter, varFunctor, BH);
}

void InstructionWriter::printReturn(std::ostringstream& , simple_indent &, const var_pp_functorConstRef , const routineFunctorConstRef )
{
   /*indenter(stream, "return ");
   unsigned int retVal = BH->get_function_return();
   stream << retVal;
   prettyPrintVertex::print_var_name(stream, retVal, *prototypeFunc, indenter);
   indenter(stream, ";\n");*/
}

void InstructionWriter::write_declarations(std::ostringstream &, simple_indent &)
{
}
