/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file instruction_writer.hpp
 * @brief Simple class to print single instruction
 *
 * @author Luca Fossati <fossati@elet.polimi.it>
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/

#ifndef INSTRUCTION_WRITER_HPP
#define INSTRUCTION_WRITER_HPP

///Graph include
#include "graph.hpp"

///STD include
#include <ostream>
#include <fstream>
#include <iostream>

///Utility include
#include "refcount.hpp"

CONSTREF_FORWARD_DECL(Parameter);
CONSTREF_FORWARD_DECL(var_pp_functor);
CONSTREF_FORWARD_DECL(behavioral_helper);
CONSTREF_FORWARD_DECL(routineFunctor);
class simple_indent;

class InstructionWriter
{
   protected:
      ///The set of input parameters
      const ParameterConstRef parameters;

      ///The debug level
      int debug_level;

   public:
      /**
       * Constructor
       * @param parameters is the set of input parameters
       */
      InstructionWriter(const ParameterConstRef parameters);

      /**
       * Destructor
       */
      virtual ~InstructionWriter();

      /**
       * Write an operation
       * @param stream is the stream where operation is printed
       * @param v is the vertex to be printed
       * @param simple_indent is the indenter
       * @param varFunctor is the variable functor
       * @param codeFunc is the function functor
       * @param isLast tells if the operation is the last one
       */
      virtual void write(std::ostringstream& stream, const vertex v, simple_indent &indenter, const var_pp_functorConstRef varFunctor, const routineFunctorConstRef codeFunc, const bool isLast = false);

      virtual void declareFunction(std::ostringstream& stream, simple_indent &indenter, const var_pp_functorConstRef varFunctor, const routineFunctorConstRef codeFunc);

      virtual void printReturn(std::ostringstream& stream, simple_indent &indenter, const var_pp_functorConstRef prototypeFunc, const routineFunctorConstRef codeFunc);

      /**
       * Write code needed for declaration/initialization of auxiliary variables
       * @param stream is the stream where code is printed
       * @param indenter is the indenter
       */
      virtual void write_declarations(std::ostringstream & stream, simple_indent &indenter);
};
typedef refcount<InstructionWriter> InstructionWriterRef;
#endif
