/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file call_graph_helper.hpp
 * @brief Helper class working on the call graph
 *
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef CALL_GRAPH_HELPER_HPP
#define CALL_GRAPH_HELPER_HPP

#include "application_manager.hpp"
#include "function_behavior.hpp"
#include "behavioral_helper.hpp"

///STL include
#include <list>

///Utility include
#include "refcount.hpp"

#include <boost/graph/graph_traits.hpp>
#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/depth_first_search.hpp>

/**
 * Visitor to identify the list of called functions
 */
struct called_functions_visitor : public boost::default_dfs_visitor
{
   private:

      std::list<unsigned int>& functions;

      const application_managerRef AppM;

   public:
   /**
    * Constructor
    *
    */
   called_functions_visitor(std::list<unsigned int>& functions_, const application_managerRef AppM_) :
      functions(functions_),
      AppM(AppM_)
   {}

   template <typename Edge, typename Graph>
   void back_edge(const Edge&, Graph&)
   {
      THROW_ERROR("recursive functions not yet supported");
   }
   /**
    * Function called when a vertex has been finished
    * @param u is the vertex
    * @param g is the graph
    */
   template <typename Vertex, typename Graph>
   void finish_vertex(const Vertex& u, Graph&)
   {
      unsigned int function_id = AppM->get_call_graph()->get_function(u);
      if(AppM->get_function_behavior(function_id)->get_helper()->has_implementation())
         functions.push_back(function_id);
   }
};

#endif // CALL_GRAPH_HELPER_HPP
