/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file op_edge_info.hpp
 * @brief Data stuctures used to represent an edge in operations graph.
 *
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef OP_EDGE_INFO_HPP
#define OP_EDGE_INFO_HPP

///Superclass include
#include "cdfg_edge_info.hpp"

/**
 * @name Constants identifying the type of the edges
*/
//@{
/// Transitive reducted edge selector
#define TRED_SELECTOR 1

/// Control flow graph edge selector
#define CFG_SELECTOR 2
/// Feedback control flow edge selector
#define FB_CFG_SELECTOR 4
/// Control flow graph with feedback edges
#define FCFG_SELECTOR (CFG_SELECTOR | FB_CFG_SELECTOR)

/// Control dependence edge selector
#define CDG_SELECTOR 8
/// Feedback control dependence edge selector
#define FB_CDG_SELECTOR 16
/// Control dependence graph selector with feedback edges
#define FCDG_SELECTOR (CDG_SELECTOR | FB_CDG_SELECTOR)


/// Data flow graph edge selector between computed on ssa
#define DFG_SSA_SELECTOR 32
/// Data flow graph edge selector between not computed on ssa
#define DFG_SER_SELECTOR 64
/// Data flow graph edge selector
#define DFG_SELECTOR (DFG_SSA_SELECTOR | DFG_SER_SELECTOR)
/// Feedback Data flow graph edge selector
#define FB_DFG_SELECTOR 128
/// Data flow graph with feedback edges
#define FDFG_SELECTOR (DFG_SELECTOR | FB_DFG_SELECTOR)

/// Anti-dependence graph edge selector
#define ADG_SELECTOR 256
///Feedback anti-dependence graph edge selector
#define FB_ADG_SELECTOR 512
/// Anti-dependence graph selector with feedback edges
#define FADG_SELECTOR (ADG_SELECTOR | FB_ADG_SELECTOR)

/// Output-dependence graph edge selector
#define ODG_SELECTOR 1024
///Feedback output-dependence graph edge selector
#define FB_ODG_SELECTOR 2048
/// Output-dependence graph selector with feedback edges
#define FODG_SELECTOR 4096

/// Control and Data dependence graph edge selector
#define SDG_SELECTOR (CDG_SELECTOR | DFG_SELECTOR)
/// Control and Data dependence graph edge selector with feedback edges
#define FSDG_SELECTOR (CDG_SELECTOR | DFG_SELECTOR | FB_CDG_SELECTOR | FB_DFG_SELECTOR)
/// Reducted sdg with feedback edges
#define RSDG_SELECTOR (TRED_SELECTOR)
/// Reducted sdg with feedback edges
#define RFSDG_SELECTOR (TRED_SELECTOR | FB_CDG_SELECTOR| FB_FG_SELECTOR)

/// Control and Data dependence and antidependence graph edge selector
#define SADG_SELECTOR (CDG_SELECTOR | DFG_SELECTOR | ADG_SELECTOR)
/// Control and Data dependence and antidependence graph edge selector with feedback edges
#define FSADG_SELECTOR (CDG_SELECTOR | DFG_SELECTOR | ADG_SELECTOR | FB_CDG_SELECTOR | FB_DFG_SELECTOR | FB_ADG_SELECTOR)
/// Control and Data dependence and antidependence graph edge selector
#define SAODG_SELECTOR (CDG_SELECTOR | DFG_SELECTOR | ADG_SELECTOR | ODG_SELECTOR)
/// Control and Data dependence, antidependence and output graph edge selector with feedback edges
#define FSAODG_SELECTOR (CDG_SELECTOR | DFG_SELECTOR | ADG_SELECTOR | ODG_SELECTOR | FB_CDG_SELECTOR | FB_DFG_SELECTOR | FB_ADG_SELECTOR | FB_ODG_SELECTOR)
/// data dependence antidependence and feedback graph edge selector
#define FADFG_SELECTOR (DFG_SELECTOR | ADG_SELECTOR)

/// Control edge in a speculation graph
#define CSG_SELECTOR 8192
/// Help graph
#define HG_SELECTOR 16384
/// Speculation graph
#define SG_SELECTOR (CSG_SELECTOR | DFG_SELECTOR | ADG_SELECTOR | ODG_SELECTOR | FLG_SELECTOR)

/// Flow edge selector
#define FLG_SELECTOR 32768

/// Feedback Flow edge selector*/
#define FB_FG_SELECTOR 65536

/// Path Profiling Basic Block edge selector
#define PP_SELECTOR 131072

/// Reduced PDG edge selector
#define RPDG_SELECTOR 262144

/**
 * The info associated with an edge of operation graph
 */
class OpEdgeInfo : public CdfgEdgeInfo
{
   public:
      /**
       * Constructor
       */
      OpEdgeInfo();

      /**
       * Destructor
       */
      ~OpEdgeInfo();
};
#endif
