/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file op_node_info.cpp
 * @brief Specialization of class node_info for operation graphs
 *
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/

///Header include
#include "op_node_info.hpp"

OpNodeInfo::OpNodeInfo() :
   TypedNodeInfo(),
   bb_index(0),
   cer(0)
{}

void OpNodeInfo::add_parameter(unsigned int _NodeID)
{
   parameters.push_back(_NodeID);
}

void OpNodeInfo::add_return(unsigned int _NodeID)
{
   returns.push_back(_NodeID);
}

void OpNodeInfo::add_ssa_written(unsigned int _NodeID)
{
   ssa_written.insert(_NodeID);
}

void OpNodeInfo::add_ssa_read(unsigned int _NodeID)
{
   ssa_read.push_back(_NodeID);
}

void OpNodeInfo::add_ref_var(unsigned int _NodeID)
{
   referenced_var.insert(_NodeID);
}

void OpNodeInfo::add_redefined_var(unsigned int _NodeID)
{
   vdef_ops.insert(_NodeID);
}

void OpNodeInfo::add_store_var(unsigned int _NodeID)
{
   stores.insert(_NodeID);
}

void OpNodeInfo::add_load_var(unsigned int _NodeID)
{
   loads.insert(_NodeID);
}

void OpNodeInfo::add_NodeID(unsigned int nodeid)
{
   if(std::find(NodeID.begin(), NodeID.end(), nodeid) == NodeID.end())
      NodeID.push_back(nodeid);
}

