/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file constant_strings.hpp
 * @brief constant strings
 *
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
 */
#ifndef CONSTANT_OPTIONS_HPP
#define CONSTANT_OPTIONS_HPP

///frontend statistics
#define STR_OPT_frontend_statistics "frontend_statistics"
///Name of the current benchmark
#define STR_OPT_benchmark_name "benchmark_name"
///Parameter of the current benchmark for table results (this value is not used for profiling)
#define STR_OPT_benchmark_fake_parameters "benchmark_fake_parameters"
///hand mapping
#define STR_OPT_hand_mapping "hand_mapping"
///check diopsis
#define STR_OPT_check_diopsis "check_diopsis"



///The default fork cost
#define NUM_CST_fork_cost 10

///Maximum distance among the label added by instrumentation and what we are looking for in objdump
#define NUM_CST_instrumentation_frame 70

///Digits in the address on sparc architecture
#define NUM_CST_SPARC_address_digit_size 8

///The number of surviving benchmarks
#define NUM_CST_surviving_benchmarks 300

///The default threshold
#define NUM_CST_task_threshold 4000



/**
 * Identifier of architecture elements
 */
///The string identifing a ARM component
#define STR_CST_ARM "ARM"
///The string identifing a BUS
#define STR_CST_BUS "BUS"
///The string identifing a DSP component
#define STR_CST_DSP "DSP"
///The string identifing a FPGA component
#define STR_CST_FPGA "FPGA"
///The string identifing a GPP component
#define STR_CST_GPP "GPP"




///The string used to separate breakpoint
#define STR_CST_breakpoint_delimiter "__breakpoint_delimiter__"
///The name of the default driving component
#define STR_CST_driving_component_name "<driving-component>"
///The gcc output
#define STR_CST_gcc_output "__gcc_output"
///The file where output of grep of grmon is saved
#define STR_CST_grep_grmon_output "__grep_grmon_output"
///The file where output of grep of grmon is saved
#define STR_CST_grep_objdump_output "__grep_objdump_output"
///The file where output of grmon is saved
#define STR_CST_grmon_output "__grmon_output"
///The output file of hartes configuration
#define STR_CST_hartes_configuration_output_file_name "__hartes_configuration_output"
///The string used to define the size of the array storing profiling information
#define STR_CST_instrumentation_counter "INSTRUMENTATION_COUNTER"
///The named of the array used for collect profiling information in instrumented source code file
#define STR_CST_instrumentation_counter_name "__instrumentation_counter_name"
///The type used for collect profiling information in instrumented source code file
#define STR_CST_instrumentation_counter_type "__instrumentation_counter_type"
///The first header used in instrumentation
#define STR_CST_instrumentation_header1 "__instumentation_header1.h"
///The second header used in instrumentation
#define STR_CST_instrumentation_header2 "__instumentation_header2.h"
///The instrumented source code file
#define STR_CST_instrumented_main "instrumented.main_original.c"
///The source file instrumented to collect data about single components
#define STR_CST_instrumented_main_component "instrumented.main_component.c"
///The source file instrumented to collect data about single functions
#define STR_CST_instrumented_main_function "instrumented.main_function.c"
///The source file instrumented to collect data about pragma speed-up
#define STR_CST_instrumented_main_pragma "instrumented.main_pragma.c"
///The source file instrumented to collect data about parallel speed-up
#define STR_CST_instrumented_main_parallel "instrumented.main_parallel.c"
///The file used to store profiling information about maximum number of loop iteration
#define STR_CST_loop_data "__loop_data.dat"
///The makefile
#define STR_CST_makefile "makefile"
///The objdump output
#define STR_CST_objdump_output "__objdump_output"

///The string used to replace sizeof keyword in the original source code (first step)
#define STR_CST_panda_sizeof "__panda_sizeof"
///The file where profiling data are written by instrumented executable
#define STR_CST_profile_data "profile.dat"
///The file where profiling overhead data are saved
#define STR_CST_profiling_overhead "profiling_overhead.xml"
///The file where output of instrumented executable is saved
#define STR_CST_profiling_output "profiling_exe_output"
///The xml file containing profiling data
#define STR_CST_profiling_xml_data "profiling_data.xml"
///The string used to identify the code regions belonging to source functions
#define STR_CST_source_region "source_region"
///The output of ssh command
#define STR_CST_ssh_output "__ssh_output"
///The string used to replace sizeof keyword in the original source code (second step)i
#define STR_CST_string_sizeof "__string_sizeof"
///The string used to identify the code regions belonging to system functions
#define STR_CST_system_region "system_region"
///The name of the file where statistics about source code are saved
#define STR_CST_source_code_statistics_file_name "source_code_statistics.xml"
///The compiled file for running on target architecture
#define STR_CST_target_run "__target_run"
///The temporary directory pattern
#define STR_CST_temporary_directory "panda-temp"
///The file containing the execution trace generated by Simit
#define STR_CST_trace_xml "trace.xml"




/**
 * a3
 */
///The directory where temporary files created by a3 will be put
#define STR_CST_a3_batch_directory "__a3_batch_file.apd"

///The batch file for absint a3
#define STR_CST_a3_batch_file "__a3_batch_file.apx"

///The executable file for absint a3
#define STR_CST_a3_exec "a3"

///The results file for absint a3
#define STR_CST_a3_results "__a3_results.xml"

///The suffix of the file containing detailed information about WCET
#define STR_CST_a3_WCET_table_suffix "wcet-table-aiT.xml"

///The report file for absint a3
#define STR_CST_a3_xml_report "__a3_report.xml"



/**
 * Compare measure regions
 */
///The name of the examined execution
#define STR_CST_compare_measure_regions_examined_execution "compare_measure_regions_examined_execution"

///The source code for the examined file
#define STR_CST_compare_measure_regions_examined_file "compare_measure_regions_examined_file.c"

///The name of the golden execution
#define STR_CST_compare_measure_regions_golden_execution "compare_measure_regions_golden_execution"

///The source code for the golden file
#define STR_CST_compare_measure_regions_golden_file "compare_measure_regions_golden_file.c"

///The file containing the results of the comparison 
#define STR_CST_compare_measure_regions_results "divergent_measure_regions.xml"



/**
 * Compare models
 */
///The name of the results file
#define STR_CST_compare_model_results "compare_models_results.xml"



/**
 * Instrumentation
 */
///The name of the file instrumented for measuring loop performances
#define STR_CST_instrumented_main_loop "instrumented_loop_main.c"

///Instrumentation prefix
#define STR_CST_instrumentation_prefix "PandA_instrumentation_"

///The name of not operating system region
#define STR_CST_not_os_region "not_os_region"

///The name of the source code region
#define STR_CST_not_source_code_region "not_source_code_region"

///The name of operating system region
#define STR_CST_os_region "os_region"

///The name of the source code region
#define STR_CST_source_code_region "source_code_region"

///The name of the whole application region
#define STR_CST_whole_application_region "whole_application_region"



/**
 * Metrics
 */
///The file where time report is stored
#define STR_CST_time_report_file "time_report.xml"



/**
 * Output files
 */
///The file used to store shell output
#define STR_CST_shell_output_file "shell_output"

///The file used to store the script shell
#define STR_CST_shell_script "shell_script"

///The source code default output file
#define STR_CST_source_code_output_file "output.c"



/**
 * Parameters
 */
///The string representing all classes
#define STR_CST_debug_all "all"



/**
 * Performance estimation
 */
///The number of "main" operations
#define NUM_CST_main_operations 1000


///The average case estimation technique
#define STR_CST_average_case "average_case"

///The path based estimation technique
#define STR_CST_path_based "path_based"

///The worst case estimation technique
#define STR_CST_worst_case "worst_case"



/**
 * Profiling analysis
 */
///The output file for rtl operation counters
#define STR_CST_rtl_operation_counters_file "rtl_operation_counters.xml"

///The output file for tree operation counters
#define STR_CST_tree_operation_counters_file "tree_operation_counters.xml"



/**
 * Pragmas
 */
///The prefix common to all functions replacing pragma
#define STR_CST_pragma_prefix "__pragma"

///The function replacing an ending pragma
#define STR_CST_pragma_function_end STR_CST_pragma_prefix"_end__"

///The function replacin a generic pragma
#define STR_CST_pragma_function_generic STR_CST_pragma_prefix"_generic__"

///The function replacing a single line pragma with single argument
#define STR_CST_pragma_function_single_line_one_argument STR_CST_pragma_prefix"_single_line_one_argument__"

///The function replacing a single line pragma with two argument
#define STR_CST_pragma_function_single_line_two_arguments STR_CST_pragma_prefix "_single_line_two_arguments__"

///The function replacing an opening pragma
#define STR_CST_pragma_function_start STR_CST_pragma_prefix"_start__"

///The call_point_hw pragma keyword
#define STR_CST_pragma_keyword_call_hw "call_hw"

///The call_point_hw pragma keyword
#define STR_CST_pragma_keyword_call_point_hw "call_point_hw"

///The keyword 'map' which identifies mapping pragmas
#define STR_CST_pragma_keyword_map "map"

///The keyword 'omp' which identifies openmp pragmas
#define STR_CST_pragma_keyword_omp "omp"

///The keyword 'omp' which identifies openmp recursive
#define STR_CST_pragma_keyword_recursive "recursive"



/**
 * Parameter
 */
#define STR_CST_string_separator std::string(1,31)



/**
 * Probability distribution
 */
#define NUM_CST_xml_value_precision 6



/**
 * Profiling file
 */
///The compiled file for running on host architecture
#define STR_CST_host_run "host.run"

///The instrumend file for loop profiling
#define STR_CST_loops_profiling "LP_profiling.c"



/**
 * R
 */
///Debug file where R commands are saved
#define STR_CST_R_input "R_commands"



/**
 * Regression estimation
 */
///The default for prediction interval value (0.95 is the default also in R)
#define NUM_CST_default_prediction_interval_value 0.95

///The maximum distance between a line and the origin such that two points can be considered as linearly dependent
#define NUM_CST_linearity_approximation 5

///The minimum number of normalized cycles which a benchmark can have
#define NUM_CST_min_normalized_cycles 0.5

///The maximum number of normalized cycles which a benchmark can have
#define NUM_CST_max_normalized_cycles 100
///The label of the id benchmark column
#define STR_CST_benchmark "Benchmark"

///The label of the cycles column
#define STR_CST_cycles "Cycles"

///The prefix used in regression file data
#define STR_CST_regression_matrix_file_testing "input_matrix_testing"

///The prefix used in regression file data
#define STR_CST_regression_matrix_file_validation "input_matrix_validation"



/**
 * Super class
 */
///String identifying null class (i.e., features belonging to this class must be ignored)
#define STR_CST_super_class_null "null"



/**
 * Target architecture file
 */
///The beginning of row containing profiling information
#define STR_CST_target_architecture_row_identifier "#__#"



/**
 * Translator
 */
///The maximum width of a latex column
#define NUM_CST_latex_max_column_width 10

///The default precision of the number
#define NUM_CST_latex_number_precision 3



/**
 * Variable estimation
 */
///Size of the column in output printing
#define NUM_CST_variable_estimation_column_size 25

///Precision of printed number
#define NUM_CST_variable_estimation_precision 3



/**
 * XML Nodes: a3 files
 */
///The root node of report file
#define STR_XML_a3_a3 "a3"

///The node containing analyses performed by a3
#define STR_XML_a3_analyses "analyses"

///The node containing an analysis performed by a3
#define STR_XML_a3_analysis "analysis"

///The node containing cycles computed by a3
#define STR_XML_a3_cycles "cycles"

///The decode information about an object file
#define STR_XML_a3_decode "decode"

///The executable in a3 batch file
#define STR_XML_a3_executables "executables"

///The files section in a3 batch file
#define STR_XML_a3_files "files"

///The root node of batch file
#define STR_XML_a3_project "project"

///A node containing the result of an a3 analysis
#define STR_XML_a3_result "result"

///The root of the a3 result files
#define STR_XML_a3_results "results"

///Information about routine
#define STR_XML_a3_routine "routine"

///Information about routines
#define STR_XML_a3_routines "routines"

///The a3 configuration file node containing name of result file
#define STR_XML_a3_results_file "xml_results"

///The node containing information about a routine (a routine is a function or a loop)
#define STR_XML_a3_routine "routine"

///The a3 table containing detailed results
#define STR_XML_a3_table "wcettable"

///The cycles of a routine
#define STR_XML_a3_wce_cycles "wce_cycles"

///The cycles of a routine
#define STR_XML_a3_wce_cycles "wce_cycles"

///Node containing report of a wcet_analysis
#define STR_XML_a3_wcet_analysis_task "wcet_analysis_task"

///Root of wcet table
#define STR_XML_a3_wcettable "wcettable"

///The report file
#define STR_XML_a3_xml_report "xml_report"



/**
 * XML Nodes: architecture file
 */
///The node containing information about class of  a component
#define STR_XML_architecture_class "CLASS"

///The node containing information about execution of a function on a component
#define STR_XML_architecture_component "COMPONENT"

///The node containing information about implementation of a function on a compnent
#define STR_XML_architecture_implementation "IMPLEMENTATION"

///The node containing information about a name (of a function or of a component)
#define STR_XML_architecture_name "NAME"

///The node containing information about operation execution time
#define STR_XML_architecture_operation "OPERATION"

///The node containing information about operations execution
#define STR_XML_architecture_operations "OPERATIONS"

///The node containing information about type of component
#define STR_XML_architecture_type "TYPE"

///The node containing information about application
#define STR_XML_application "application"

///The node containing information about solutions
#define STR_XML_solution "solution"

///The node containing information about performance solution
#define STR_XML_performance "performance"

///The node containing information about task mapping
#define STR_XML_mapping_task "mapping_task"

///The node containing information about tasks
#define STR_XML_task "task"

///The node containing information about data transfers
#define STR_XML_transfer "transfer"

///The node containing information about implementations
#define STR_XML_implementation "implementation"

///The node containing information about haerware solution
#define STR_XML_hardwareFile "hardwareFile"

///The node containing information about software solution
#define STR_XML_softwareFile "srcFile"



/**
 * XML Nodes: compare measure regions
 */
///The node containing information about error of a divergent measure region
#define STR_XML_compare_measure_regions_error "error"

///The node containing information about examined time of a divergent measure region
#define STR_XML_compare_measure_regions_examined_time "examined_time"

///The attribute containing the name of a measure region
#define STR_XML_compare_measure_regions_name "name"

///The node containing information about golden time of a divergent measure region
#define STR_XML_compare_measure_regions_golden_time "golden_time"

///The node containing information about a divergent Measure Region
#define STR_XML_compare_measure_regions_measure_region "measure_region"

///The root node
#define STR_XML_compare_measure_regions_root "compare_measure_regions_results"

///The attribute containing the value of a node
#define STR_XML_compare_measure_regions_value "value"



/**
 * XML Nodes: gcc configuration
 */
///Node containing information about defines
#define STR_XML_gcc_defines "defines"

///Node containing information about includes
#define STR_XML_gcc_includes "includes"

///Node containing information about libraries
#define STR_XML_gcc_libraries "libraries"

///Node containing information about library_directories
#define STR_XML_gcc_library_directories "library_directories"

///Attribute containing information about name of optimization/parameter
#define STR_XML_gcc_name "name"

///Node containing information about optimizations
#define STR_XML_gcc_optimizations "optimizations"

///Node containing information about an optimization flag
#define STR_XML_gcc_optimization_flag "optimization_flag"

///Node containing information about optimization flags
#define STR_XML_gcc_optimization_flags "optimization_flags"

///Node containing information about an optimization value
#define STR_XML_gcc_optimization_value "optmization_value"

///Node containing information about optimization values
#define STR_XML_gcc_optimization_values "optmization_values"

///Node containing information about a parameter value
#define STR_XML_gcc_parameter_value "parameter_value"

///Node containing information about parameter values
#define STR_XML_gcc_parameter_values "parameter_values"

///The root node of a gcc configuration
#define STR_XML_gcc_root "gcc_configuration"

///Node containing information about standard
#define STR_XML_gcc_standard "standard"

///Node containing information about undefines
#define STR_XML_gcc_undefines "undefines"

///Attribute containing information about value stored in a node
#define STR_XML_gcc_value "value"

///Node containing information about warnings
#define STR_XML_gcc_warnings "warnings"




/**
 * XML Nodes: Metrics
 */
///The root node of the execution time report
#define STR_XML_Metrics "Metrics"

///The attribute storing the benchmark name
#define STR_XML_Metrics_Benchmark_Name "bechmark_name"

///The attribute containing which distribution probability has been used during performance estimation
#define STR_XML_Metrics_Distribution "distribution"

///The node containing information about measured execution times
#define STR_XML_Metrics_Dynamic "Dynamic"

///The node containing information about final parallel execution
#define STR_XML_Metrics_Final_Parallel_Execution "Final_Parellel_Execution"

///The node containing information about initial parallel execution
#define STR_XML_Metrics_Initial_Parallel_Execution "Initial_Parellel_Execution"

///The node containing information about original execution
#define STR_XML_Metrics_Original_Execution "Original_Execution"

///The node containing information about estimated execution times
#define STR_XML_Metrics_Static "Static"

///The node containing information about sequential estimation
#define STR_XML_Metrics_Sequential_Estimation "Sequential_Estimation"

///The node containing information about sequential execution
#define STR_XML_Metrics_Sequential_Execution "Sequential_Execution"

///The attribute containing the execution time of a simulation/run
#define STR_XML_Metrics_value "value"



/**
 * XML Nodes: Probability Distribution
 */
///Node containing data about a constant distribution
#define STR_XML_probability_distribution_constant_distribution "constant_distribution"

///Node containing data about the value of a constant distribution
#define STR_XML_probability_distribution_constant_distribution_value "value"

///Counter of a single stochastic variable
#define STR_XML_probability_distribution_counter "counter"

///Node containing data about normal distribution
#define STR_XML_probability_distribution_normal_distribution "normal_distribution"

///Attribute containing the mean of a normal distribution
#define STR_XML_probability_distribution_normal_distribution_mean "mean"

///Attribute containing the variance of a normal distribution
#define STR_XML_probability_distribution_normal_distribution_variance "variance"

///Distribution probability of type stochastic
#define STR_XML_probability_distribution_stochastic "stochastic"

///Node containing a single stochastic variable of a stochastic distribution
#define STR_XML_probability_distribution_stochastic_variable "stochastic_variable"

///Distribution probability of type symbolic
#define STR_XML_probability_distribution_symbolic "symbolic"

///Node containing data about a symbolic distribution"
#define STR_XML_probability_distribution_symbolic_distribution "symbolic_distribution"

///The attribute containig the name of a symbolic distribution
#define STR_XML_probability_distribution_symbolic_distribution_name "name"

///The attribute containing the type of distribution
#define STR_XML_probability_distribution_type "distribution"



/**
 * XML NOdes: profiling analysis
 */
///Information about number of absolute iterations of a loop
#define STR_XML_profiling_analysis_ABS_iterations "absolute_iterations"

///Arguments with which zebu has been invoked
#define STR_XML_profiling_analysis_args "args"

///Accesses to variable of array type
#define STR_XML_profiling_analysis_array_accesses "array_accesses"

///Information about number of average iterations of a loop
#define STR_XML_profiling_analysis_AVG_iterations "absolute_iterations"

///Node containing information about a single component
#define STR_XML_profiling_analysis_component "component"

///Attribute containing information about name of a component
#define STR_XML_profiling_analysis_component_name "name"

///Node containing information about tree node of a given type with a fixed number of constant operands
#define STR_XML_profiling_analysis_constant_operands "constant_operands_"

///Node containing information about cycles on target processing elemnt
#define STR_XML_profiling_analysis_cycles "cycles"

///Node containing information about dynamic operations
#define STR_XML_profiling_analysis_dynamic "dynamic_operations"

///Node containing information about memory footprint of a loop
#define STR_XML_profiling_analysis_footprint_loop_size "footprint_loop_size"

///Node contating information about a single function
#define STR_XML_profiling_analysis_function "function"

///Attribute containing name of a function
#define STR_XML_profiling_analysis_function_name "name"

///Name containing information about id of a loop
#define STR_XML_profiling_analysis_id "id"

///Node containing information about instruction size of a loop
#define STR_XML_profiling_analysis_instruction_loop_size "loop_instruction_size"

///Accesses to variable of integer type
#define STR_XML_profiling_analysis_integer_accesses "integer_accesses"

///Level at which analysis has been executed
#define STR_XML_profiling_analysis_level "level"

///Node containing information about characteristics of a loop
#define STR_XML_profiling_analysis_loop "loop"

///Node containing information about characteristics of loops
#define STR_XML_profiling_analysis_loops "loops"

///The node containing information about counter of single rtl_node of a given mode
#define STR_XML_profiling_analysis_rtl_mode "rtl_mode"

///Name of the benchmark
#define STR_XML_profiling_analysis_name "name"

///Node containing information about nesting of a loop
#define STR_XML_profiling_analysis_nesting "nesting"

///Path where zebu has been executed
#define STR_XML_profiling_analysis_path "path"

///Accesses to variable of pointer type
#define STR_XML_profiling_analysis_pointer_accesses "pointer_accesses"

///Accesses to variable of real type
#define STR_XML_profiling_analysis_real_accesses "real_accesses"

///The root node of the rtl operation counters
#define STR_XML_profiling_analysis_rtl_counter "rtl_operation_counters"

///The node containing information about counter of single rtl_node
#define STR_XML_profiling_analysis_rtl_node "rtl_node"

///Node containing information about static operations
#define STR_XML_profiling_analysis_static "static_operations"

///Attribute containing information about target considered during target profiling
#define STR_XML_profiling_analysis_target "target"

///Time when zebu has been executed
#define STR_XML_profiling_analysis_time "date"

///The root node of the tree operation counters
#define STR_XML_profiling_analysis_tree_counter "tree_operation_counters"

///The node containing information about counter of single tree_node
#define STR_XML_profiling_analysis_tree_node "tree_node"

///The node containing information abount counter of single tree_node of a given type
#define STR_XML_profiling_analysis_type "type"

///The value of a node
#define STR_XML_profiling_analysis_value "value"

///Zebu revision
#define STR_XML_profiling_analysis_zebu_revision "revision"



/**
 * XML Nodes: latex table format
 */
///The node containing type of alignment
#define STR_XML_latex_table_alignment "alignment"

///The node containing information about single column
#define STR_XML_latex_table_column "column"

///The node containing information about the name of the column
#define STR_XML_latex_table_column_name "name"

///The node containing information about column comparison
#define STR_XML_latex_table_comparison "compare"

///The attribute containing information about type of comparison
#define STR_XML_latex_table_operator "operator"

///The node containg information about the precision of the number
#define STR_XML_latex_table_precision "precision"

///The node containing type of text to be printed
#define STR_XML_latex_table_text_format "text_format"

///The root node
#define STR_XML_latex_table_root "latex_table_format"

///The node containing the source name of this column
#define STR_XML_latex_table_source_name "source_name"

///The attribute containing the value of the field
#define STR_XML_latex_table_value "value"



/**
 * XML Nodes: profiling overhead
 */
///The node containing the EPP overhead time
#define STR_XML_profiling_overhead_EPP_time "EPP_time"

///The node containing the HPP overhead time
#define STR_XML_profiling_overhead_HPP_time "HPP_time"

///The root of the profiling overhead time file
#define STR_XML_profiling_overhead_time "profiling_overhead_time"

///The attribute containing the value of a node
#define STR_XML_profiling_overhead_value "value"



/**
 * XML Nodes: skip rows
 */
///The root node of the xml containing rows to be skipped
#define STR_XML_skip_rows_root "skip_rows"
///The attribute containing the name of the row to be removed
#define STR_XML_skip_rows_row_name "name"



/**
 * XML Nodes: source code statistics
 */
///The number of conditional construct in the source code
#define STR_XML_source_code_conditional_construct "conditional_construct"

///The number of function with body in the source code
#define STR_XML_source_code_function_with_body "functions_with_body"

///The number of lines of source code
#define STR_XML_source_code_lines "source_code_lines"

///The number of loop in the source code
#define STR_XML_source_code_loop_number "loops_number"

///The maximum depth of loop
#define STR_XML_source_code_maximum_depth "loop_maximum_depth"

///The benchmark name attribute
#define STR_XML_source_code_name "benchmark_name"

///The root node of the xml containing statistics about source code
#define STR_XML_source_code_statistics "source_code_statistics"

///The attribute containing the value of a node
#define STR_XML_source_code_value "value"



/**
 * XML Nodes: weight models
 */
///The attribute containing the cycles of an operations
#define STR_XML_weights_cycles "cycles"

///The node containing the execution time of a function
#define STR_XML_weights_function "function"

///The node containing information about gimple based model
#define STR_XML_weights_gimple "tree_weights"

///The node containing information about rtl mode
#define STR_XML_weights_mode "mode"

///The attribute containing the name of the node/type/etc
#define STR_XML_weights_name "name"

///The attribute containing the processing element of the model
#define STR_XML_weights_processing_element "processing_element"

///The root of a weights model
#define STR_XML_weights_root "model_weights"

///The node containing information about gimple based model
#define STR_XML_weights_rtl "rtl_weights"

///The node containing information about rtl node
#define STR_XML_weights_rtx "rtx"

///The attribute storing the length of the sequences
#define STR_XML_weights_sequences_length "sequence_length"

///The attribute containing the name of a symbolic stochastic variable 
#define STR_XML_weights_variable "variable"

///The attribute containing the variance of distribution probability of cycles of an operation
#define STR_XML_weights_variance "variance"

#endif
