/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file BambuFrontendFlow.cpp
 * @brief This file contains the implementation of some methods of the flow from the raw to
 *        the intermediate representation used by the PandA framework for Bambu tool.
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/

///Autoheader include
#include "config_HAVE_EXPERIMENTAL.hpp"

///Header include
#include "BambuFrontendFlow.hpp"

///Frontend include
#include "BaseAnalysis.hpp"
#if HAVE_EXPERIMENTAL
#include "speculation_edges_computation.hpp"
#endif
#include "determine_memory_accesses.hpp"
//#include "verification_operation.hpp"
//#include "unrolling_degree.hpp"
#include "NI_SSA_liveness.hpp"

#if HAVE_EXPERIMENTAL
#endif
BambuFrontendFlow::BambuFrontendFlow (const ParameterRef _Param, const application_managerRef _AppM, const HLS_targetRef _HLS_T, const HLS_constraintsRef _HLS_C, const std::tr1::unordered_map<FrontendAnalysisType, unsigned int>& transformations) :
   FrontendFlow (transformations, _AppM, _Param),
   HLS_T(_HLS_T),
   HLS_C(_HLS_C)
{
}

BambuFrontendFlow::~BambuFrontendFlow ()
{

}

BaseAnalysisRef BambuFrontendFlow::create_tool_analysis(const FrontendAnalysisType type, const unsigned int fun_id)
{
   switch(type)
   {
#if 0
      case VERIFICATION_OPERATION:
      {
         return BaseAnalysisRef(new verification_operation(Param, AppM, fun_id));
      }
      case UNROLLING_DEGREE:
      {
         return BaseAnalysisRef(new unrolling_degree(TM, HLS_C, Param, AppM, fun_id));
      }
#endif
#if HAVE_EXPERIMENTAL
      case SPECULATION_EDGES_COMPUTATION:
      {
         return BaseAnalysisRef(new speculation_edges_computation(Param, AppM, fun_id));
      }
#endif
      case DETERMINE_MEMORY_ACCESSES:
      {
         return BaseAnalysisRef(new determine_memory_accesses(Param, AppM, fun_id));
      }
      case NI_SSA_LIVENESS:
      {
         return BaseAnalysisRef(new NI_SSA_liveness(Param, AppM, fun_id));
      }
      default:
         THROW_ERROR("Analysis step " + boost::lexical_cast<std::string>(type) + " not yet supported");
   }
   return BaseAnalysisRef();
}
