/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file BambuFrontendFlow.hpp
 * @brief This class defines the flow from the raw to the intermediate representation used by the PandA framework for the Bambu tool.
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/

#ifndef BAMBUFRONTENDFLOW_HPP
#define BAMBUFRONTENDFLOW_HPP

///Superclass include
#include "FrontendFlow.hpp"

///STL include
#include <tr1/unordered_map>

///Utility include
#include "refcount.hpp"
/**
 * @name Forward declarations
 */
//@{
/// Forward declaration
REF_FORWARD_DECL(HLS_target);
REF_FORWARD_DECL(HLS_constraints);
//@}

class BambuFrontendFlow : public FrontendFlow
{
      ///reference to the information about the target device
      HLS_targetRef HLS_T;

      ///reference to the information about the synthesis constraints
      HLS_constraintsRef HLS_C;

      /**
       * Creates the analysis step in according to requirements
       * @param type is the identifier of the required analysis step
       * @param fun_id is the identifier of the function where the analysis has to work on (if the value is 0, it means that it
       *               is a global analysis
       * @return a reference to the implementation of the proper analysis step
       */
      BaseAnalysisRef create_tool_analysis(const FrontendAnalysisType, const unsigned int fun_id);

   public:
      /**
       * Constructor
       * @param Param is the set of parameters
       * @param AppM is the reference to the behavioral specification
       * @param HLS_T is the reference to the structure containing information about the target device
       * @param HLS_C is the reference to the datastructure containing information about the synthesis constraints
       * @param transformations is the map the represent the optional transformations to be executed or not
       */
      BambuFrontendFlow
      (
         const ParameterRef Param,
         const application_managerRef _AppM,
         const HLS_targetRef HLS_T,
         const HLS_constraintsRef HLS_C,
         const std::tr1::unordered_map<FrontendAnalysisType, unsigned int>& transformations
      );

      /**
       * Destructor
       */
      ~BambuFrontendFlow ();

};
#endif
