/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file BaseAnalysis.cpp
 * @brief 
 *
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/

///Header include
#include "BaseAnalysis.hpp"

///Frontend include
#include "Parameter.hpp"

///Utility include
#include <iostream>
#include "utility.hpp"

BaseAnalysis::BaseAnalysis(const ParameterConstRef _Param, const application_managerRef _AppM, const FrontendAnalysisType _analysis_type, unsigned int _funID) :
   AppM(_AppM),
   analysis_type(_analysis_type),
   funID(_funID),
   loop_irriducible_breaks(true),
   Param(_Param),
   output_level(_Param->getOption<int>(OPT_output_level)),
   debug_level(Param->get_class_debug_level(GET_CLASS(*this), DEBUG_LEVEL_NONE))
{
}

BaseAnalysis::~BaseAnalysis()
{

}

const std::set< std::pair<FrontendAnalysisRelationship, FrontendAnalysisType> > & BaseAnalysis::GetPreConds() const
{
   return PreConds;
}

void BaseAnalysis::SetPreCond(const FrontendAnalysisRelationship _funID, const FrontendAnalysisType _BaseAnalysis)
{
   this->PreConds.insert(std::make_pair(_funID, _BaseAnalysis));
}

void BaseAnalysis::SetPostCond(const FrontendAnalysisRelationship relationship, const FrontendAnalysisType type)
{
   this->PostConds.insert(std::make_pair(relationship, type));
}

void BaseAnalysis::print(std::ostream& os) const
{
   os << "Analysis: " << get_kind_text() << std::endl;
   if (funID)
     os << "Function Identifier: " << funID << std::endl;
   os << "Pre Conditions: " << std::endl;
   for(std::set< std::pair<FrontendAnalysisRelationship, FrontendAnalysisType> >::const_iterator p = PreConds.begin(); p != PreConds.end(); p++)
   {
      os << "   - " << boost::lexical_cast<std::string>(p->first) << " * Analysis: " << boost::lexical_cast<std::string>(p->second) << std::endl;
   }
   os << "Post Conditions: " << std::endl;
   for(std::set<std::pair<FrontendAnalysisRelationship, FrontendAnalysisType> >::const_iterator p = PostConds.begin(); p != PostConds.end(); p++)
   {
      os << "   - " << boost::lexical_cast<std::string>(p->first) << " * Analysis: " << boost::lexical_cast<std::string>(p->second) << std::endl;
   }
}

FrontendAnalysisType BaseAnalysis::get_type() const
{
   return analysis_type;
}

unsigned int BaseAnalysis::get_funID() const
{
   return funID;
}

const std::set<std::pair<FrontendAnalysisRelationship, FrontendAnalysisType> > & BaseAnalysis::GetPostConds() const
{
   return PostConds;
}
