/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file NI_SSA_liveness.hpp
 * @brief Non-Iterative liveness analysis for SSA based gimple descriptions.
 *
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef NI_SSA_LIVENESS_HPP
#define NI_SSA_LIVENESS_HPP

#include "refcount.hpp"
#include "BaseAnalysis.hpp"

/**
 * @name forward declarations
 */
//@{
REF_FORWARD_DECL(NI_SSA_liveness);
REF_FORWARD_DECL(application_manager);
CONSTREF_FORWARD_DECL(Parameter);
class statement_list;
REF_FORWARD_DECL(bloc);
REF_FORWARD_DECL(tree_node);
//@}

/**
 * 
 */
class NI_SSA_liveness : public BaseAnalysis
{
   private:

      /// Algorithm 5: Explore all paths from a variable’s use to its definition.
      void Up_and_Mark(blocRef B, tree_nodeRef v, statement_list * sl);

   public:

      /**
       * Constructor
       * @param Param is the set of the parameters
       * @param function_id is the index of the function
       */
      NI_SSA_liveness(const ParameterConstRef _Param, const application_managerRef _AppM, unsigned int function_id);

      /**
       * Destructor
       */
      virtual ~NI_SSA_liveness();

      /**
       * Performes the liveness analysis.
       */
      void exec();

      /**
       * Returns the name of the analysis step
       */
      std::string get_kind_text() const;
};

#endif
