/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file basic_blocks_cfg_computation.hpp
 * @brief Build basic block control flow graph data structure starting from the tree_manager.
 *
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef BASIC_BLOCK_CFG_COMPUTATION_HPP
#define BASIC_BLOCK_CFG_COMPUTATION_HPP

#include "BaseAnalysis.hpp"

/**
 * Build call graph structures starting from the tree_manager.
 */
class BasicBlocksCfgComputation : public BaseAnalysis
{
   public:

      /**
       * Constructor
       * @param Param is the set of the parameters
       * @param fun_id is the function for which basic block control flow graph has to be built
       */
      BasicBlocksCfgComputation(const ParameterConstRef _Param, const application_managerRef _AppM, unsigned int fun_id);

      /**
       * Destructor
       */
      ~BasicBlocksCfgComputation();

      /**
       * Computes the basic block graph structure
       */
      void exec();

      /**
       * Returns the name of the analysis step
       */
      std::string get_kind_text() const;

};

#endif
