/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file block_fix.hpp
 * @brief Analysis step that modifies the control flow graph of the tree to make it more compliant
 *
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef BLOCK_FIX_HPP
#define BLOCK_FIX_HPP

///Superclass include
#include "BaseAnalysis.hpp"

///Utility include
#include "refcount.hpp"

/**
 * @name forward declarations
 */
//@{
REF_FORWARD_DECL(call_graph);
REF_FORWARD_DECL(tree_manager);
REF_FORWARD_DECL(tree_node);
class statement_list;
//@}


/**
 * Restructure the tree control flow graph
 */
class block_fix : public BaseAnalysis
{
   private:

      /// flag to check if initial tree has been dumped
      static bool tree_dumped;

   public:
      /**
       * Constructor.
       * @param Param is the set of the parameters
       * @param function_id is the identifier of the function
       */
      block_fix(const ParameterConstRef _Param, const application_managerRef _AppM, unsigned int function_id);

      /**
       *  Destructor
       */
      ~block_fix();

      /**
       * Updates the tree to have a more compliant CFG
       */
      void exec();

      /**
       * Returns the name of the analysis step
       */
      std::string get_kind_text() const;
};

#endif
