/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file call_graph_computation.hpp
 * @brief Build call_graph data structure starting from the tree_manager.
 *
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef CALL_GRAPH_COMPUTATION_HPP
#define CALL_GRAPH_COMPUTATION_HPP

#include "BaseAnalysis.hpp"

#include "refcount.hpp"
#include "utility.hpp"
#include "BaseAnalysis.hpp"

#include <map>

/**
 * @name forward declarations
 */
//@{
REF_FORWARD_DECL(expand);
REF_FORWARD_DECL(tree_manager);
REF_FORWARD_DECL(call_graph);
REF_FORWARD_DECL(tree_node);
REF_FORWARD_DECL(call_graph_computation);
REF_FORWARD_DECL(behavioral_helper);
//@}

/**
 * Build call graph structures starting from the tree_manager.
*/
class call_graph_computation : public BaseAnalysis
{
   private:
      ///Index of current function
      unsigned int current;

      ///set of already examined addr_expr used to avoid circular recursion
      std::set<tree_nodeRef> already_visited;

      /// Functor used to check if the analysis should consider the body of a function
      expandRef exp;

      std::map<unsigned int, std::set<unsigned int> > type_to_declaration;
      std::map<std::pair<unsigned int, unsigned int>, unsigned int> pointercall_to_type;

      /**
       * call graph computation for functions called indirectly.
       * @param tm is the tree manager.
       * @param CG is call graph data structure.
       * @param function_id is the node id of the inner function.
      */
      void exec_internal(call_graphRef &CG, const tree_managerRef &tm, unsigned int function_id, unsigned int node_stmt = 0);

      /**
       * Recursive analysis of the tree nodes looking for call expressions.
       * @param tm is the tree manager.
       * @param CG is call graph data structure.
       * @param tn is current tree node.
      */
      void call_graph_computation_recursive(const tree_managerRef &tm, call_graphRef &CG, const tree_nodeRef &tn, unsigned int node_stmt = 0);

      void funpointer_analysis(const tree_nodeRef &tn, unsigned int node_stmt);

   public:

      /**
       * Constructor.
       * @param Param is the set of the parameters
       * @param _exp is the functor used to check if the analysis should consider the body of the function.
       */
      call_graph_computation(const ParameterConstRef _Param, const application_managerRef _AppM, expandRef &_exp);

      /**
       *  Destructor
       */
      ~call_graph_computation();

      /**
       * Computes the call graph data structure.
       */
      void exec();

      /**
       * Returns the name of the analysis step
       */
      std::string get_kind_text() const { return "call_graph_computation"; }

};

#endif
