/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file data_flow_cleanIng.hpp
 * @brief Analysis step removing fake data dependences
 *
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef DATA_FLOW_CLEANING_HPP
#define DATA_FLOW_CLEANING_HPP

#include "refcount.hpp"
#include "BaseAnalysis.hpp"

/**
 * @name forward declarations
 */
//@{
// REF_FORWARD_DECL(operations_cfg_computation);
// REF_FORWARD_DECL(operations_graph_constructor);
REF_FORWARD_DECL(call_graph);
REF_FORWARD_DECL(behavioral_helper);
REF_FORWARD_DECL(tree_manager);
// REF_FORWARD_DECL(tree_node);

//@}

#include <map>
#include <list>
#include <string>
#include <set>

/**
 * Clean fake data dependences
 */
class data_flow_cleaning : public BaseAnalysis
{
   private:

      /// the tree manager
      tree_managerRef TM;

      /// helper associated with the function
      behavioral_helperRef helper;

      /**
       * Return pointer used in the subtree with root nodeId
       * @param NodeID is the index of the root of the subtree
       * @param candidate_pointers is the set of pointer whose dependences are fake
       * @param not_candidate_pointers is the set of pointer whose dependences are not fake
       */
      void get_candidate_pointers(unsigned int NodeID, std::set<unsigned int> & candidate_pointers, std::set<unsigned int> & not_candidate_pointers);

   public:

      /**
       * Constructor.
       * @param Param is the set of the parameters
       * @param function_id is the identifier of the function
       */
      data_flow_cleaning(const ParameterConstRef _Param, const application_managerRef _AppM, unsigned int function_id);

      /**
       *  Destructor
       */
      ~data_flow_cleaning();

      /**
       * Cleans the fake data dependences
       */
      void exec();

      /**
       * Returns the name of the analysis step
       */
      std::string get_kind_text() const;
};

#endif
