/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file determine_memory_accesses.hpp
 * @brief Determine variables to be stored in memory
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $ $
 * Last modified by $Author: ferrandi $
 *
*/

#ifndef DETERMINE_MEMORY_ACCESSES_HPP
#define DETERMINE_MEMORY_ACCESSES_HPP

#include "BaseAnalysis.hpp"
#include "refcount.hpp"

REF_FORWARD_DECL(tree_manager);
REF_FORWARD_DECL(behavioral_helper);

#include <list>
#include <map>


class determine_memory_accesses : public BaseAnalysis
{
   private:

      /**
       * Analyze the given node ID to determine which variables have to be referred in memory
       */
      void analyze_node(unsigned int node_id, const tree_managerRef& TM, const behavioral_helperRef& BH, bool left_p);

   public:
      /**
       * Constructor.
       * @param function_id is the node id of the function analyzed.
       * @param _debug_level is the debug level.
       * @param alloc is the reference to allocation class
       * @param constr is the high-level synthesis constraints.
       */
      determine_memory_accesses(const ParameterConstRef _Param, const application_managerRef _AppM, unsigned int function_id);

      /**
       *  Destructor
       */
      ~determine_memory_accesses();

      /**
       * Determines the variables that require a memory access
       */
      void exec();

      /**
       * Returns the name of the analysis step
       */
      std::string get_kind_text() const;

};


#endif

