/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file loop_regions_computation.hpp
 * @brief Analysis step creating the regions for loops. See PDG definition for details.
 * 
 * To easy the parallelization task each header of a loop has to have a unique starting point and all basic blocks of a loop having an
 * edge going towards the header block has to have an unique ending point. This can be obtained quite easily by adding to the tree
 * intermediate representation, a gimple_label working as unique starting point and some gimple_goto working as unique ending point.
 * No modification is required in case 
 * - the last statement is a gimple_goto or a gimple_cond or a gimple_switch
 * - the first statement is a gimple_label.
 *
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef LOOP_REGIONS_COMPUTATION_HPP
#define LOOP_REGIONS_COMPUTATION_HPP

#include "refcount.hpp"
#include "BaseAnalysis.hpp"

/**
 * @name forward declarations
 */
//@{
REF_FORWARD_DECL(loop_regions_computation);
REF_FORWARD_DECL(call_graph);
REF_FORWARD_DECL(tree_manager);
//@}

/**
 * Compute the regions of a loop.
 */
class loop_regions_computation : public BaseAnalysis
{
   private:

      ///flag to check if the initial tree has been dumped
      static bool tree_dumped;

   public:

      /**
       * Constructor.
       * @param Param is the set of the parameters
       * @param function_id is the identifier of the function
       */
      loop_regions_computation(const ParameterConstRef _Param, const application_managerRef _AppM, unsigned int function_id);

      /**
       *  Destructor
       */
      virtual ~loop_regions_computation();

      /**
       * Computes the regions of a loop.
       */
      void exec();

      /**
       * Returns the name of the analysis step
       */
      std::string get_kind_text() const { return "loop_regions_computation"; }

};

#endif
