/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file split_phi_nodes.hpp
 * @brief 
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef SPLITPHINODES_HPP
#define SPLITPHINODES_HPP

#include "refcount.hpp"
#include "utility.hpp"
#include "BaseAnalysis.hpp"

#include "graph.hpp"

/**
 * @name forward declarations
 */
//@{
REF_FORWARD_DECL(split_phi_nodes);
REF_FORWARD_DECL(call_graph);
REF_FORWARD_DECL(tree_manager);
REF_FORWARD_DECL(tree_node);
REF_FORWARD_DECL(bloc);
REF_FORWARD_DECL(tree_manager);
//@}

/**
 * 
 */
class split_phi_nodes : public BaseAnalysis
{
   private:

      /// flag to check if initial tree has been dumped
      static bool tree_dumped;

      ///Basic block introduced after entry
      blocRef next_entry;

      ///Basic block splitting information: if replace[source, target] exists, then between source and target replace[source, target] has been inserted
      std::map<std::pair<unsigned int, unsigned int>, unsigned int> replace;

      /**
       * split a particular phi in two or more assignments
       * @param phi is the phi
       * @param bb_block is the current basic block
       * @param list_of_bloc is basic block of the current function
       * @param TM is the tree manager
       */
      bool split_phi(tree_nodeRef phi, blocRef& bb_block, std::map<unsigned int, blocRef>& list_of_bloc, const tree_managerRef TM);

   public:

      /**
       * Constructor
       * @param Param is the set of the parameters
       * @param function_id is the index of the function
       */
      split_phi_nodes(const ParameterConstRef _Param, const application_managerRef _AppM, unsigned int function_id);

      /**
       * Destructor
       */
      virtual ~split_phi_nodes();

      /**
       * Performes the splitting of the phi-nodes.
       */
      void exec();

      /**
       * Returns the name of the analysis step
       */
      std::string get_kind_text() const;
};

#endif
