/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file tree_manager_fix.hpp
 * @brief Fix tree manager removing duplicated nodes
 *
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/

///Header include 
#include "tree_manager_fix.hpp"

///Frontend include
#include "application_manager.hpp"

#include "Parameter.hpp"

///Tree include
#include "tree_manager.hpp"

///Utility include
#include <fstream>

tree_manager_fix::tree_manager_fix(const ParameterConstRef _Param, const application_managerRef _AppM) :
   BaseAnalysis(_Param, _AppM, TREE_MANAGER_FIX, 0)
{
   debug_level = Param->get_class_debug_level("tree_manager_fix", DEBUG_LEVEL_NONE);
   //no pre and post condition.
}

tree_manager_fix::~tree_manager_fix()
{
}

void tree_manager_fix::exec()
{
   const tree_managerRef old_TM = AppM->get_tree_manager();
   if(debug_level >= DEBUG_LEVEL_PEDANTIC)
   {
      INDENT_DBG_MEX(DEBUG_LEVEL_PEDANTIC, debug_level, "-->Dumping Tree-Manager for debug purpose");
      std::string raw_file_name = Param->getOption<std::string>(OPT_output_temporary_directory) + "before_tree_manager_fix.raw";
      std::ofstream raw_file(raw_file_name.c_str());
      raw_file << old_TM;
      raw_file.close();
      INDENT_DBG_MEX(DEBUG_LEVEL_PEDANTIC, debug_level, "<--Dumped Tree-Manager for debug purpose");
   }
   tree_managerRef TM(new tree_manager(Param));
   TM->merge_tree_managers(old_TM);
   AppM->set_tree_manager(TM);
   if(debug_level >= DEBUG_LEVEL_PEDANTIC)
   {
      std::string raw_file_name = Param->getOption<std::string>(OPT_output_temporary_directory) + "after_tree_manager_fix.raw";
      std::ofstream raw_file(raw_file_name.c_str());
      PRINT_DBG_MEX(DEBUG_LEVEL_PEDANTIC, debug_level, "Tree-Manager dumped for debug purpose");
      raw_file << TM;
      raw_file.close();
   }
}

std::string tree_manager_fix::get_kind_text() const
{
   return "tree_manager_fix";
}
