/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file tree_manager_fix.hpp
 * @brief Fix tree manager removing duplicated nodes
 *
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef TREE_MANAGER_FIX_HPP
#define TREE_MANAGER_FIX_HPP

///Superclass include
#include "BaseAnalysis.hpp"

///Utility inlcude
#include "refcount.hpp"
#include "utility.hpp"


/**
 * @name forward declarations
 */
//@{
REF_FORWARD_DECL(expand);
REF_FORWARD_DECL(tree_manager);
REF_FORWARD_DECL(call_graph);
REF_FORWARD_DECL(tree_node);
REF_FORWARD_DECL(call_graph_computation);
REF_FORWARD_DECL(behavioral_helper);
//@}

/**
 * Fix tree manager removing duplicated nodes
*/
class tree_manager_fix : public BaseAnalysis
{
   public:
      /**
       * Constructor.
       * @param Param is the set of the parameters
       */
      tree_manager_fix(const ParameterConstRef _Param, const application_managerRef _AppM);

      /**
       *  Destructor
       */
      ~tree_manager_fix();

      /**
       * Removes duplicated nodes
       */
      void exec();

      /**
       * Returns the name of the analysis step
       */
      std::string get_kind_text() const;
};
#endif
