/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file var_computation.hpp
 * @brief Analyzes operations and creates the sets of read and written variables
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef VAR_COMPUTATION_HPP
#define VAR_COMPUTATION_HPP

#include "refcount.hpp"
#include "utility.hpp"
#include "BaseAnalysis.hpp"

#include "graph.hpp"

/**
 * @name forward declarations
 */
//@{
REF_FORWARD_DECL(var_computation);
REF_FORWARD_DECL(call_graph);
REF_FORWARD_DECL(tree_manager);
REF_FORWARD_DECL(vops);
//@}

/**
 * 
 */
class var_computation : public BaseAnalysis
{
   private:

      ///flag to check if this pass has been already executed
      bool executed;

      /**
       * Analyzes a node id associated to a statement (i.e. a vertex in the control flow graph) to obtain information about read and written
       * variables
       * @param Name this is the name associated to the vertex
       * @param NodeID is the node id associated to the vertex
       */
      void AnalyzeNodeID(const vertex &Ver, unsigned int NodeID);

      void Analyze_lvalue(const vertex &Ver, unsigned int NodeID, bool onlyReference = false);
      void Analyze_rvalue(const vertex &Ver, unsigned int NodeID);
      void add_vops(const vertex &Ver, const vopsRef& virtual_operands);
      std::set<unsigned int> ref_vars;
      std::set<unsigned int> vops;
      bool isRef;

      void report();

   public:

      /**
       * Constructor.
       * @param Param is the set of the parameters
       * @param AppM is the reference to the application manager
       * @param function_id is the index of the function
       */
      var_computation(const ParameterConstRef _Param, const application_managerRef _AppM, unsigned int function_id);

      /**
       * Destructor
       */
      ~var_computation();

      /**
       * Computes the set of read and written variables.
       */
      void exec();

      /**
       * Returns the name of the analysis step
       */
      std::string get_kind_text() const { return "var_computation"; }

};

#endif
