/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file var_decl_fix.hpp
 * @brief Pre-analysis step fixing var_decl duplications.
 *
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef VAR_DECL_FIX_HPP
#define VAR_DECL_FIX_HPP

///STL include
#include "tr1/unordered_set"
#include <set>

///Utility include
#include "refcount.hpp"
#include "BaseAnalysis.hpp"

/**
 * @name forward declarations
 */
//@{
REF_FORWARD_DECL(var_decl_fix);
REF_FORWARD_DECL(call_graph);
REF_FORWARD_DECL(tree_manager);
REF_FORWARD_DECL(tree_node);
//@}

/**
 * Pre-analysis step. It transforms the raw intermediate representation removing 
 * var_decl duplication: two var_decls with the same variable name in the same function.
 */
class var_decl_fix : public BaseAnalysis
{
   private:

      /// flag to check if initial tree has been dumped
      static bool tree_dumped;

      ///Already considered decl_node
      std::tr1::unordered_set<unsigned int> already_examinated_decls;

      ///Already found variable and parameter names
      std::tr1::unordered_set<std::string> already_examinated_names;

      ///Already found type names
      std::tr1::unordered_set<std::string> already_examinated_type_names;

      ///Already visited address expression (used to avoid infite recursion)
      std::tr1::unordered_set<unsigned int> already_visited_ae;

      /**
      * Recursive examinate tree node
      */
      void recursive_examinate(const tree_nodeRef & tn);

   public:
      /**
       * Constructor.
       * @param Param is the set of the parameters
       * @param fun_id is the function index
       */
      var_decl_fix(const ParameterConstRef _Param, const application_managerRef _AppM, unsigned int fun_id);

      /**
       * Destructor
       */
      ~var_decl_fix();

      /**
       * Fixes the var_decl duplication.
       */
      void exec();

      /**
       * Returns the name of the analysis step
       */
      std::string get_kind_text() const;
};

#endif

