/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file loop_regions_flow_computation.hpp
 * @brief Analysis step performing edge flow computation for loop regions.
 *
 * This analysis step performs two computations:
 * - add to the vertices of type TYPE_LABEL flow edges from 
 *   the label vertex to all vertices belonging to the same basic block.
 * - add to the vertices of type TYPE_GOTO flow edges from 
 *   all vertices belonging to the same basic block to the goto vertex.
 * 
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef LOOP_REGIONS_FLOW_COMPUTATION_HPP
#define LOOP_REGIONS_FLOW_COMPUTATION_HPP


#include "BaseAnalysis.hpp"

#include "refcount.hpp"

/**
 * @name forward declarations
 */
//@{
REF_FORWARD_DECL(loop_regions_flow_computation);
REF_FORWARD_DECL(call_graph);
REF_FORWARD_DECL(tree_manager);
//@}


/**
 * loops computation analysis step
*/
class loop_regions_flow_computation : public BaseAnalysis
{
   public:

      /**
       * Constructor.
       * @param Param is the set of the parameters
       * @param function_id is the node id of the function analyzed.
       */
      loop_regions_flow_computation(const ParameterConstRef _Param, const application_managerRef _AppM, unsigned int function_id);

      /**
       *  Destructor
       */
      ~loop_regions_flow_computation();

      /**
       * Computes the edge flow for the loop regions.
       */
      void exec();

      /**
       * Returns the name of the analysis step
       */
      std::string get_kind_text() const { return "loop_regions_flow_computation"; }

};

#endif
