/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file loops_computation.cpp
 * @brief Analysis step performing loops computation.
 *
 *
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * $Revision: 10609 $
 * $Date: 2012-03-12 11:40:21 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: pilato $
 *
*/
///Header include
#include "loops_computation.hpp"

#include "application_manager.hpp"
#include "graph.hpp"
#include "function_behavior.hpp"
#include "behavioral_helper.hpp"
#include "loops.hpp"
#include "loop.hpp"
#include "basic_block.hpp"
#include "tree_basic_block.hpp"
#include "Parameter.hpp"

#include <iostream>

loops_computation::loops_computation(const ParameterConstRef _Param, const application_managerRef _AppM, unsigned int function_id) :
   BaseAnalysis(_Param, _AppM, LOOPS_IDENTIFICATION, function_id)
{
   this->SetPreCond(ID, BASIC_BLOCKS_CFG_COMPUTATION);
   this->SetPreCond(ID, DOM_POST_DOM_COMPUTATION);
}


loops_computation::~loops_computation()
{
}

void loops_computation::exec()
{
   const function_behaviorRef FB = AppM->get_function_behavior(funID);
   bb_graph * fbb = FB->fbb;
   const behavioral_helperRef helper = FB->get_helper();

   THROW_ASSERT(FB->dominators, "Dominators has to be computed!");
   FB->loops = LoopsConstRef(new Loops(FB, Param));
   if(output_level>= OUTPUT_LEVEL_VERY_PEDANTIC || Param->getOption<bool>(OPT_print_dot))
      FB->GetLoops()->WriteDot((helper->get_function_name() + "_" + boost::lexical_cast<std::string>(funID) + "_LF.dot").c_str());
   std::list<LoopConstRef> loops = FB->GetLoops()->GetList();
   std::list<LoopConstRef>::const_iterator loop_end = loops.end();
   for(std::list<LoopConstRef>::const_iterator loop = loops.begin(); loop != loop_end; loop++)
   {
      ///FIXME: zero loop
      if((*loop)->GetId() == 0)
         continue;
      if ((*loop)->IsReducible())
      {
         PRINT_DBG_MEX(DEBUG_LEVEL_VERY_PEDANTIC, debug_level, "Found a reducibile loop composed by:");
      }
      else
      {
         if(loop_irriducible_breaks)
         {
            THROW_ERROR_CODE(IRREDUCIBLE_LOOPS_EC,"Irreducible loops not yet supported");
            THROW_ERROR("Irreducible loops not yet supported");
         }
         else
            PRINT_DBG_MEX(DEBUG_LEVEL_VERY_PEDANTIC, debug_level,  "Found an irreducible loop");
      }
      const std::tr1::unordered_set<vertex> blocks = (*loop)->get_blocks();
      std::tr1::unordered_set<vertex>::const_iterator bb_it, bb_it_end = blocks.end();
      for(bb_it = blocks.begin(); bb_it != bb_it_end; bb_it++)
      {
         bb_node_info * operations = GET_BB_NODE_INFO(fbb, *bb_it);
         operations->loop_id = (*loop)->GetId();
         PRINT_DBG_MEX(DEBUG_LEVEL_VERY_PEDANTIC, debug_level, "  Basic block " + boost::lexical_cast<std::string>(operations->block->number));
      }
   }
   PRINT_DBG_MEX(DEBUG_LEVEL_VERY_PEDANTIC, debug_level,  "Number of reducible loops: " + boost::lexical_cast<std::string>(FB->GetLoops()->NumLoops()));
}
