/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file reachability_computation.hpp
 * @brief Analysis step computing reachability between operations
 *
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $
 *
*/
#ifndef MUTUAL_EXCLUSION_COMPUTATION_HPP
#define MUTUAL_EXCLUSION_COMPUTATION_HPP

#include "BaseAnalysis.hpp"

/**
 * Computes reachability between operations according to the extended control flow graph
 * An operation inside a loop reaches all the operation after the loop
 * An operation inside a loop does not reach all the operation inside the loop but which are not reachable without crossing a feedback edge
 */
class reachability_computation: public BaseAnalysis
{
   public:

      /**
       * Constructor.
       * @param Param is the set of the parameters
       * @param function_id is the identifier of the function
       */
      reachability_computation(const ParameterConstRef _Param, const application_managerRef _AppM, unsigned int function_id);

      /**
       *  Destructor
       */
      ~reachability_computation();

      /**
       * Computes the reachability between operations according to the control flow graph WITHOUT feedback
       */
      void exec();

      /**
       * Returns the name of the analysis step
       */
      std::string get_kind_text() const;
};

#endif
