/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file global_enums.hpp
 * @brief global enums used which are widely used in framework
 *
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
 */
#ifndef GLOBAL_ENUMS_HPP
#define GLOBAL_ENUMS_HPP

///Autoheader include
#include "config_HAVE_A3.hpp"
#include "config_HAVE_BAMBU_BUILT.hpp"
#include "config_HAVE_DESIGN_ANALYSIS_BUILT.hpp"
#include "config_HAVE_DIOPSIS.hpp"
#include "config_HAVE_FROM_C_BUILT.hpp"
#include "config_HAVE_FROM_LIBERTY.hpp"
#include "config_HAVE_LEON3.hpp"
#include "config_HAVE_REGRESSORS_BUILT.hpp"
#include "config_HAVE_SIMIT.hpp"
#include "config_HAVE_TSIM.hpp"
#include "config_HAVE_TUCANO_BUILT.hpp"
#include "config_HAVE_ZEBU_BUILT.hpp"


///STD include
#include <string>

///STL include
#include <tr1/unordered_map>

/**
 * Profiling analysis level (at which level data are aggregated
 */
typedef enum
{
   AL_NONE = 0,     /**< No analysis is performed */
   AL_ALL,          /**< Analysis performed at application level */
   AL_OS,           /**< Analysis performed at application level (system function excluded)*/
   AL_SOURCE,       /**< Analysis performed at application level (system and library function excluded)*/
   AL_COMPONENT,    /**< Analysis performed at component level (data of functions mapped on the same component are aggregated)*/
   AL_FUNCTION,     /**< Analysis performed at function level*/
   AL_LOOP          /**< Analysis performed at loop level*/
} AnalysisLevel;

/**
 * target of the compiler
 */
typedef enum
{
   CT_NO_GCC = 0,
   CT_I386_GCC = 1,
   CT_ARM_GCC = 2,
   CT_SPARC_GCC = 4,
   CT_SPARC_ELF_GCC = 8,
} CompilerTarget;

#if HAVE_DIOPSIS
/**
 * Instrumentation type used to profile diopsis
 */
typedef enum
{
   DI_GNAM,           /**< instrumentation based on gettimeofday function */
   DI_CPU_TIME,       /**< instrumentation based on gnam_linux_hw library */ 
   DI_HGET            /**< instrumentation based on hget_timer function */
} DiopsisInstrumentation;
#endif

/**
 * File formats
 */
typedef enum
{
   FF_UNKNOWN = 0,             /**< UNKNOWN */
   FF_CSV,                     /**< (Input) comma separated value */
   FF_CSV_RTL,                 /**< (Output) comma separated value rtl sequences */
   FF_CSV_TRE,                 /**< (Output) comma seperated value tree sequences */
#if HAVE_FROM_LIBERTY
   FF_LIB,                     /**< (Input) Liberty file */
#endif
   FF_LOG,                     /**< (Input) log file */
   FF_PA,                      /**< (Input) Profiling analysis */
   FF_TEX,                     /**< (Output) Latex table */
   FF_VERILOG,                 /**< (Input) verilog */
   FF_VHDL,                    /**< (Input) vhdl */
   FF_XML,                     /**< (Input/Output) XML */
#if HAVE_REGRESSORS_BUILT
   FF_XML_AGG,                 /**< (Input) XML aggregated features */
#endif
#ifdef HAVE_FROM_LIBERTY
   FF_XML_CELLS,     /**< (Input) XML describing list of cells */
#endif
#if HAVE_DESIGN_ANALYSIS_BUILT
   FF_XML_DESIGN_HIERARCHY,    /**< (Input) XML storing hierarchy of a design */
#endif
   FF_XML_IP_XACT_COMPONENT,   /**< (Input) XML storing IP-XACT component */
   FF_XML_IP_XACT_DESIGN,      /**< (Input) XML storing IP-XACT design */
   FF_XML_IP_XACT_GENERATOR,   /**< (Input) XML storing IP-XACT generator chain */
   FF_XML_IP_XACT_CONFIG,      /**< (Input) XML storing IP-XACT design configuration */
   FF_XML_SKIP_ROW,            /**< (Input) XML benchhmarks to be execluded from training set */
   FF_XML_STAT,                /**< (Input) XML statistics about source code files */
   FF_XML_SYM_SIM,             /**< (Input) XML storing symbolic symulation results */
   FF_XML_TEX_TABLE,           /**< (Input) XML storing format of latex table to be produced */
   FF_XML_WGT_GM,              /**< (Output) XML weights of single operations computed */ 
} FileFormat;

#if HAVE_DESIGN_ANALYSIS_BUILT
/**
 * High level design flow steps
 */
typedef enum
{
   HLD_VERILOG_STRUCTURAL=1,            /**< Generate structural verilog */
   HLD_VERILOG_SPLIT=2,                 /**< Split HDL verilog in multiple files */
   HLD_ANALYZE_DC_LOG=4,                /**< Compute instances */
   HLD_IPXACT_INSTANTATION=8,           /**< Generate IP-XACT architecture instance */
   HLD_IPXACT_NORMALIZATION=16,         /**< Normalize name in IP-XACT xmls */
   HLD_IPXACT_CHARACTERIZATION=32,      /**< Characterize components */
   HLD_IPXACT_CANDIDATE_EXPLORATION=64, /**< Candidate exploration and identification */
   HLD_IPXACT_DESIGN_CONFIGURATION=128, /**< Generate design configuration */
   HLD_SCRIPT_GENERATION=256,           /**< Generate script for synthesis tools */
   HLD_IR_BUILDING=512,                 /**< Build graph based IR */
} DesignAnalysisStep;
#endif

#if HAVE_FROM_C_BUILT
///Possible optimization sets
typedef enum
{
   OS_O0,      /**< -O0 */
   OS_O1,      /**< -O1 */
   OS_O2,      /**< -O2 */
   OS_O3,      /**< -O3 */
   OS_Os,      /**< -Os */
#if HAVE_BAMBU_BUILT
   OS_OBAMBU,  /**< Bambu optimizations + OPT_gcc_opt_level */
#endif
#if HAVE_TUCANO_BUILT
   OS_OTUCANO, /**< Tucano optimizations + OPT_gcc_opt_level */
#endif
#if HAVE_ZEBU_BUILT
   OS_OZEBU,   /**< Zebu optimizations + OPT_gcc_opt_level */
#endif
} OptimizationSet;
#endif

/**
 * Algorithm used to estimate the performance of applications and tasks
 */
typedef enum
{
   PE_NONE = 0,        /** No algorithm selected */
   PE_WORST_CASE = 2,  /** Worst case execution time for each task */
   PE_PATH_BASED = 4,  /** Path based */
   PE_AVERAGE_CASE = 8 /** Average case execution time for each task */
} PerformanceEstimationAlgorithm;

/**
 * The name of the performance estimation algorithms
 */
static
std::string performance_estimation_algorithm_names[] = {"None", "", "Worst_Case", "", "Path_Based", "", "", "", "Average_Case"};

/**
 * Definition of hash function for PerformanceEstimationAlgorithm
 */
namespace std
{
   namespace tr1
   {
      template <>
      struct hash<PerformanceEstimationAlgorithm> : public unary_function<PerformanceEstimationAlgorithm, size_t>
      {
         size_t operator()(PerformanceEstimationAlgorithm algorithm) const
         {
            hash<int> hasher;
            return hasher(static_cast<int>(algorithm));
         }
      };
   }
}

/// Different profiling method
typedef enum
{
   PM_NONE                =  0, /**< None profiling method selected */
   PM_HPP                 =  1, /**< Hierarchical Path Profiling */
   PM_TP                  =  2, /**< Tree based Path Profiling */
   PM_MAX_LOOP_ITERATIONS =  4, /**< Maximum number of iteration profiling */
   PM_PATH_PROBABILITY    =  8, /**< Probability based path */
   PM_XML_FILE            = 16  /**< Data read from XML file */
} ProfilingMethod;

/**
 * Target architectures
 */
typedef enum
{
#if HAVE_A3
   TA_A3,                 /**< absint a3 */
#endif
#if HAVE_DIOPSIS
   TA_ARM_DIOPSIS,        /**< arm of diopsis board */
#endif
#if HAVE_SIMIT
   TA_SIMIT,              /**< SimIt simulator */
#endif
#if HAVE_LEON3
   TA_LEON3,              /**< Leon3 board */
#endif
#if HAVE_TSIM
   TA_TSIM_BREAK,         /**< TSIM simulator based on brakpoint */
   TA_TSIM_CLOCK,         /**< TSIM simulator based on clock */
   TA_TSIM_PROF,          /**< TSIM simulator based on prof */
   TA_TSIM_SINGLE_PROF,   /**< TSIM simulator based on prof with sampling period fixed to 1*/
   TA_TSIM_TRACE,         /**< TSIM simulator based on trace */
#endif
   TA_UNKNOWN,            /**< Unknown */
   TA_LAST
} TargetArchitectureKind;
#endif
