/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file graph_info.hpp
 * @brief Base class for graph property.
 *
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef GRAPH_INFO_HPP
#define GRAPH_INFO_HPP

#include <ostream>
#include "refcount.hpp"
/**
* @name forward declarations
*/
//@{
REF_FORWARD_DECL(graph_info);
//@}


/**
 * Base class storing user data information to the whole graph.
 * This class is associated with the graph data structure.
*/
struct graph_info
{
   /// Constructor
   graph_info(){}

   /// Destructor
   virtual ~graph_info(){}

   /**
    * Print the information associated with the whole graph.
    * @param os is the output stream.
   */
   virtual void print(std::ostream& ) const
   {}

   /**
    * Friend definition of the << operator.
    * @param os is the output stream.
    * @param s is the graph info to print.
   */
   friend std::ostream& operator<<(std::ostream& os, const graph_info& s)
   {
      s.print(os);
      return os;
   }
   /**
    * Friend definition of the << operator. Pointer version.
    * @param os is the output stream.
    * @param s is the graph info to print.
   */
   friend std::ostream& operator<<(std::ostream& os, const graph_infoRef &s)
   {
      if (s)
         s->print(os);
      return os;
   }
};

/**
 * RefCount type definition of the graph_info class structure
*/
typedef refcount<graph_info> graph_infoRef;

#endif

