/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file RTL_characterization.hpp
 * @brief Class for performing RTL characterization
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * $Revision: 10622 $
 * $Date: 2012-03-14 09:23:58 +0100 (Wed, 14 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef RTL_CHARACTERIZATION_HPP
#define RTL_CHARACTERIZATION_HPP

#include "refcount.hpp"

/**
 * @name forward declarations
 */
//@{
//parameters
CONSTREF_FORWARD_DECL(Parameter);
REF_FORWARD_DECL(target_device);
REF_FORWARD_DECL(target_technology);
REF_FORWARD_DECL(technology_manager);
REF_FORWARD_DECL(technology_node);
REF_FORWARD_DECL(library_manager);
REF_FORWARD_DECL(language_writer);
REF_FORWARD_DECL(structural_object);
class xml_element;
//@}

#include <vector>
#include <set>

class RTL_characterization
{
   private:

      ///reference to the datastructure where all the parameters and options have been stored
      const ParameterConstRef Param;

      ///debug level of the class
      int debug_level;

      ///debug level of the class
      int output_level;

      /**
       * Characterize the specific library with respect to the target device
       */
      void characterize_library(const technology_managerRef& TM, const library_managerRef& LM, const target_deviceRef& device);

      /**
       * Characterize the given functional unit with respect to the target device
       */
      void characterize_fu(const technology_nodeRef& functional_unit, const technology_managerRef& TM, const library_managerRef& LM, const target_deviceRef& device);

      /**
       * Create a template instance to be specialized
       */
      technology_nodeRef create_template_instance(const technology_nodeRef& fu_template, std::string& name, const target_deviceRef& device, unsigned int prec);

      /**
       * Performing the specialization of the given object
       */
      void specialize_fu(const structural_objectRef& obj, unsigned int prec);

      /**
       * Generate the output file
       */
      void xwrite_device_file(const technology_managerRef& TM, const target_deviceRef& device);

      /**
       * Add the characterization to the output file
       */
      void xwrite_characterization(const technology_managerRef& TM, const target_deviceRef& device, xml_element* nodeRoot);

      ///set of units completed with success
      std::set<std::string> completed;

   public:

      /**
       * Constructor
       */
      RTL_characterization(const ParameterConstRef Param);

      /**
       * Destructor
       */
      ~RTL_characterization();

      /**
       * Perform RTL characterization of the modules with respect to the target device
       */
      void exec(const technology_managerRef& TM, const target_deviceRef& target);
};

/// RefCount definition of the class
typedef refcount<RTL_characterization> RTL_characterizationRef;

#endif
