/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file core_generation.hpp
 * @brief Class for coverting an IP core in XML description
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef _CORE_GENERATION_HPP
#define _CORE_GENERATION_HPP

#include "refcount.hpp"

/**
 * @name forward declarations
 */
//@{
CONSTREF_FORWARD_DECL(Parameter);
REF_FORWARD_DECL(technology_manager);
//@}

class core_generation
{
   private:

      ///class containing all the parameters
      const ParameterConstRef Param;

      ///debug level of the class
      int debug_level;

      /**
       * Coverts a VHDL description into an XML based one
       * @param vhdl_module is the path to the VHDL description to be converted
       * @param output_file is the name of the file to be generated
       */
      void convert_vhdl_to_XML(const std::string& vhdl_module, const std::string& output_file);

   public:

      /**
       * Constructor
       * @param _Param is the reference to the class containing all the parameters
       */
      core_generation(const ParameterConstRef Param);

      /**
       * Destructor
       */
      ~core_generation();

      /**
       * Converts the given module into an XML-based description
       * @param hdl_module is the string containing the path to the HDL file describing the module
       */
      void convert_to_XML(const std::string& hdl_module);

      /**
       * Exports the HDL description of the given module
       * @param TM is the class containing all the information about the library resources
       * @param module_name is a string representing the name of the module to be exported
       */
      void export_core(const technology_managerRef TM, const std::string& module_name);

};
/// RefCount definition of the class
typedef refcount<core_generation> core_generationRef;

#endif
