/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file clb_model.hpp
 * @brief Specification of the class for representing FPGA area/resource models
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef _CLB_MODEL_HPP_
#define _CLB_MODEL_HPP_

#include "area_model.hpp"

#include <vector>
#include <map>

///Utility include
#include "refcount.hpp"

CONSTREF_FORWARD_DECL(Parameter);
REF_FORWARD_DECL(target_technology);

class clb_model : public area_model
{
   public:

      ///type of resources
      typedef enum
      {
         SLICE_REGISTERS,
         SLICE_LUTS,
         LUT_FF_PAIRS,
         BONDED_IOBS
      } value_t;

   protected:

      ///a double value representing the area of the component
      double area;

      ///resources required for the component
      std::map<value_t, double> used_resources;

   public:

      /**
       * Constructor
       */
      clb_model(const ParameterConstRef Param, const target_technologyRef target);

      /**
       * Constructor with specified area value
       */
      clb_model(const ParameterConstRef Param, const target_technologyRef target, const double& _area_);

      /**
       * Destructor
       */
      virtual ~clb_model();

      /**
       * Prints the used resources into an output stream
       */
      void print(std::ostream& os) const;

      /**
       * Sets the nominal value for the area of the component
       */
      void set_area_value(const double& _area_);

      /**
       * Returns the nominal value for the area of the component
       */
      double get_area_value() const;

      /**
       * Checks if there is a characterization for the given type
       */
      bool is_characterization(unsigned int characterization_type) const;

      /**
       * Sets the number of resources used for the specified type
       */
      void set_resource_value(value_t val, double num);

      /**
       * Returns true if the specified resource is used, false otherwise
       */
      bool is_used_resource(value_t val) const;

      /**
       * Returns the number of resources used for the given type
       */
      double get_resource_value(value_t val) const;
};

typedef refcount<clb_model> clb_modelRef;

#endif
