/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/

/**
 * @file FPGA_device.hpp
 * @brief This class represents an FPGA as target device for the synthesis process
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 */
#ifndef _FPGA_DEVICE_HPP_
#define _FPGA_DEVICE_HPP_

#include "target_device.hpp"
#include <map>

class FPGA_device : public target_device
{

      ///map between the default device string and the corresponding configuration stream
      std::map<std::string, std::string> default_device_data;

   public:

      /**
       * Constructor
       * @param Param is the reference to the class that contains all the parameters
       * @param TM is the reference to the class containing all the technology libraries
       */
      FPGA_device(const ParameterConstRef Param, const technology_managerRef TM);

      /**
       * Destructor
       */
      virtual ~FPGA_device();

      /**
       * Initializes the target device based on the given parameters
       */
      virtual void initialize();

      /**
       * Method to write an XML node
       * @param node is the node for writing the information
       */
      virtual void xwrite(xml_element* node);

};

#endif


