/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file target_device.cpp
 * @brief 
 *
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 */
#include "target_device.hpp"

///Autoheader include
#include "config_HAVE_CMOS_BUILT.hpp"
#if HAVE_CMOS_BUILT
#include "IC_device.hpp"
#endif
#include "FPGA_device.hpp"

#include "technology_manager.hpp"
#include "target_technology.hpp"

#include "exceptions.hpp"

///XML includes
#include "polixml.hpp"
#include "xml_dom_parser.hpp"
#include "fileIO.hpp"
#include "xml_helper.hpp"
///parameters' includes
#include "Parameter.hpp"
#include "constant_strings.hpp"
///boost includes for file manipulations
#include <boost/filesystem.hpp>

target_device::target_device(const ParameterConstRef _Param, const technology_managerRef _TM) :
      Param(_Param),
      TM(_TM),
      core_height(0),
      core_width(0)
{

}

target_device::~target_device( )
{

}

target_deviceRef target_device::create_device(const type_t type, const ParameterConstRef param, const technology_managerRef TM)
{
   switch(type)
   {
#if HAVE_CMOS_BUILT
      case IC:
         return target_deviceRef(new IC_device(param, TM));
#endif
      case FPGA:
         return target_deviceRef(new FPGA_device(param, TM));
      default:
         THROW_ERROR("Not supported device type");
   }
   ///this point should never be reached
   return target_deviceRef();
}

void target_device::xload(const std::string& file_name, const xml_element* node)
{
   const xml_node::node_list c_list = node->get_children();
   for(xml_node::node_list::const_iterator n = c_list.begin(); n != c_list.end(); n++)
   {
      if ((*n)->get_name() == "device")
      {
         const xml_element* dev_xml = GetPointer<const xml_element>(*n);
         xload_device_parameters(dev_xml);
      }
   }

   unsigned int output_level = Param->getOption<unsigned int>(OPT_output_level);
   PRINT_OUT_MEX(OUTPUT_LEVEL_MINIMUM, output_level, "Target technology = " + target->get_string_type());

   for(xml_node::node_list::const_iterator n = c_list.begin(); n != c_list.end(); n++)
   {
      if ((*n)->get_name() == "technology")
      {
         const xml_element* tech_xml = GetPointer<const xml_element>(*n);
         technology_manager::xload(file_name, tech_xml, TM, Param, target);
      }
   }
}

void target_device::xload_device_parameters(const xml_element* dev_xml)
{
   const xml_node::node_list t_list = dev_xml->get_children();
   for(xml_node::node_list::const_iterator t = t_list.begin(); t != t_list.end(); t++)
   {
      const xml_element* t_elem = GetPointer<const xml_element>(*t);
      if (!t_elem) continue;

      std::string value;
      LOAD_XVM(value, t_elem);

      parameters[t_elem->get_name()] = value;
   }
}

double target_device::get_core_height() const
{
   return core_height;
}

double target_device::get_core_width() const
{
   return core_width;
}

target_technologyRef target_device::get_target_technology() const
{
   return target;
}

technology_managerRef target_device::get_technology_manager() const
{
   return TM;
}
