/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file ext_tree_node.cpp
 * @brief Class implementation of the tree_node structures not present in gcc intermediate representation
 *
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $
 *
*/

#include "ext_tree_node.hpp"
#include "tree_reindex.hpp"

void null_node::visit(tree_node_visitor * const v) const
{
   unsigned int mask=ALL_VISIT;
   (*v)(this, mask);
   VISIT_SC(mask,tree_node,visit(v));
}

void pragma_node::visit(tree_node_visitor * const v) const
{
   unsigned int mask=ALL_VISIT;
   (*v)(this, mask);
   VISIT_SC(mask,srcp,visit(v));
   VISIT_SC(mask,tree_node,visit(v));
   VISIT_MEMBER(mask, scope, visit(v));
   VISIT_MEMBER(mask, directive, visit(v));
}

void call_hw_pragma::visit(tree_node_visitor * const v) const
{
   unsigned int mask=ALL_VISIT;
   (*v)(this, mask);
   VISIT_SC(mask, map_pragma, visit(v));
}

void call_point_hw_pragma::visit(tree_node_visitor * const v) const
{
   unsigned int mask=ALL_VISIT;
   (*v)(this, mask);
   VISIT_SC(mask, map_pragma, visit(v));
}

void profiling_pragma::visit(tree_node_visitor * const v) const
{
   unsigned int mask=ALL_VISIT;
   (*v)(this, mask);
   VISIT_SC(mask,tree_node,visit(v));
}

void statistical_profiling::visit(tree_node_visitor * const v) const
{
   unsigned int mask=ALL_VISIT;
   (*v)(this, mask);
   VISIT_SC(mask,profiling_pragma,visit(v));
}

void omp_parallel_pragma::visit(tree_node_visitor * const v) const
{
   unsigned int mask=ALL_VISIT;
   (*v)(this, mask);
   VISIT_SC(mask,omp_pragma,visit(v));
}

void omp_sections_pragma::visit(tree_node_visitor * const v) const
{
   unsigned int mask=ALL_VISIT;
   (*v)(this, mask);
   VISIT_SC(mask,omp_pragma,visit(v));
}

void omp_parallel_sections_pragma::visit(tree_node_visitor * const v) const
{
   unsigned int mask=ALL_VISIT;
   (*v)(this, mask);
   VISIT_SC(mask,omp_pragma,visit(v));
   VISIT_MEMBER(mask,op0,visit(v));
   VISIT_MEMBER(mask,op1,visit(v));
}

void omp_section_pragma::visit(tree_node_visitor * const v) const
{
   unsigned int mask=ALL_VISIT;
   (*v)(this, mask);
   VISIT_SC(mask,omp_pragma,visit(v));
}

void omp_for_pragma::visit(tree_node_visitor * const v) const
{
   unsigned int mask=ALL_VISIT;
   (*v)(this, mask);
   VISIT_SC(mask,omp_pragma,visit(v));
}

void map_pragma::visit(tree_node_visitor * const v) const
{
   unsigned int mask=ALL_VISIT;
   (*v)(this, mask);
   VISIT_SC(mask,tree_node,visit(v));
}

void issue_pragma::visit(tree_node_visitor * const v) const
{
   unsigned int mask=ALL_VISIT;
   (*v)(this, mask);
   VISIT_SC(mask,tree_node,visit(v));
}

void blackbox_pragma::visit(tree_node_visitor * const v) const
{
   unsigned int mask=ALL_VISIT;
   (*v)(this, mask);
   VISIT_SC(mask,issue_pragma,visit(v));
}

void omp_pragma::visit(tree_node_visitor * const v) const
{
   unsigned int mask=ALL_VISIT;
   (*v)(this, mask);
   VISIT_SC(mask,tree_node,visit(v));
}

void while_expr::visit(tree_node_visitor * const v) const
{
   unsigned int mask=ALL_VISIT;
   (*v)(this, mask);
   VISIT_SC(mask,expr_node_vops,visit(v));
   VISIT_MEMBER(mask,op0,visit(v));
}

void for_expr::visit(tree_node_visitor * const v) const
{
   unsigned int mask=ALL_VISIT;
   (*v)(this, mask);
   VISIT_SC(mask,while_expr,visit(v));
   VISIT_MEMBER(mask,op1,visit(v));
   VISIT_MEMBER(mask,op2,visit(v));
}
