/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file tree_common.hpp
 * @brief This C++ header file contains common macros for the tree structure
 *
 * This C++ header file define some macros useful during the tree structure hierarchy definition and manipulation.
 *
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 *
*/
#ifndef TREE_COMMON_HPP
#define TREE_COMMON_HPP

///STD include
#include <string>

///Utility include
#include <boost/preprocessor/cat.hpp>
#include <boost/preprocessor/seq/for_each.hpp>
#include "refcount.hpp"
#include "utility.hpp"

#define TREE_NODE_NAME(r, data, elem) #elem,

#define TREE_NODE_KIND(r, data, elem) elem,

#define TREE_NODE_LIST (tree_reindex_K) (function_decl_K) (identifier_node_K) (void_type_K) (boolean_type_K) (lang_type_K) (complex_type_K) (vector_type_K) (CharType_K) (offset_type_K) (set_type_K) (qual_union_type_K) (real_type_K) (integer_type_K) (enumeral_type_K) (function_type_K) (pointer_type_K) (reference_type_K) (array_type_K) (record_type_K) (union_type_K) (method_type_K) (type_decl_K) (translation_unit_decl_K) (label_decl_K) (const_decl_K) (namespace_decl_K) (result_decl_K) (field_decl_K) (parm_decl_K) (var_decl_K) (exc_ptr_expr_K) (filter_expr_K) (gimple_return_K) (gimple_goto_K) (gimple_label_K) (unsave_expr_K) (gimple_resx_K) (va_arg_expr_K) (exit_expr_K) (loop_expr_K) (nop_expr_K) (negate_expr_K) (truth_not_expr_K) (convert_expr_K) (addr_expr_K) (reference_expr_K) (cleanup_point_expr_K) (fix_trunc_expr_K) (fix_ceil_expr_K) (fix_floor_expr_K) (fix_round_expr_K) (float_expr_K) (abs_expr_K) (conj_expr_K) (realpart_expr_K) (imagpart_expr_K) (card_expr_K) (non_lvalue_expr_K) (view_convert_expr_K) (indirect_ref_K) (modify_expr_K) (gimple_assign_K) (gimple_nop_K) (gt_expr_K) (plus_expr_K) (minus_expr_K) (mult_expr_K) (trunc_div_expr_K) (trunc_mod_expr_K) (le_expr_K) (truth_or_expr_K) (ne_expr_K) (eq_expr_K) (compound_expr_K) (init_expr_K) (ceil_div_expr_K) (floor_div_expr_K) (round_div_expr_K) (ceil_mod_expr_K) (floor_mod_expr_K) (round_mod_expr_K) (rdiv_expr_K) (exact_div_expr_K) (min_expr_K) (max_expr_K) (lshift_expr_K) (rshift_expr_K) (lrotate_expr_K) (rrotate_expr_K) (bit_ior_expr_K) (bit_xor_expr_K) (bit_and_expr_K) (bit_not_expr_K) (truth_andif_expr_K) (truth_orif_expr_K) (truth_and_expr_K) (truth_xor_expr_K) (lt_expr_K) (ge_expr_K) (unordered_expr_K) (ordered_expr_K) (unlt_expr_K) (unle_expr_K) (ungt_expr_K) (unge_expr_K) (uneq_expr_K) (ltgt_expr_K) (in_expr_K) (set_le_expr_K) (range_expr_K) (complex_expr_K) (predecrement_expr_K) (preincrement_expr_K) (postdecrement_expr_K) (postincrement_expr_K) (component_ref_K) (array_ref_K) (fdesc_expr_K) (try_catch_expr_K) (try_finally_K) (catch_expr_K) (eh_filter_expr_K) (gimple_switch_K) (save_expr_K) (cond_expr_K) (gimple_cond_K) (bit_field_ref_K) (vtable_ref_K) (with_cleanup_expr_K) (placeholder_expr_K) (goto_subroutine_K) (buffer_ref_K) (array_range_ref_K) (case_label_expr_K) (gimple_bind_K) (call_expr_K) (gimple_call_K) (target_expr_K) (labeled_block_expr_K) (exit_block_expr_K) (gimple_asm_K) (vector_cst_K) (string_cst_K) (complex_cst_K) (integer_cst_K) (real_cst_K) (tree_list_K) (statement_list_K) (ssa_name_K) (tree_vec_K) (error_mark_K) (block_K) (constructor_K) (gimple_phi_K) (binfo_K) (template_decl_K) (template_type_parm_K) (typename_type_K) (cast_expr_K) (static_cast_expr_K) (sizeof_expr_K) (scope_ref_K) (ctor_initializer_K) (arrow_expr_K) (decl_stmt_K) (expr_stmt_K) (return_stmt_K) (modop_expr_K) (new_expr_K) (vec_new_expr_K) (overload_K) (reinterpret_cast_expr_K) (template_id_expr_K) (throw_expr_K) (trait_expr_K) (try_block_K) (compound_stmt_K) (handler_K) (baselink_K) (template_parm_index_K) (with_size_expr_K) (obj_type_ref_K) (struct_field_tag_K) (name_memory_tag_K) (symbol_memory_tag_K) (memory_partition_tag_K) (mem_ref_K) (pointer_plus_expr_K) (target_mem_ref_K) (target_mem_ref461_K) (while_expr_K) (for_expr_K) (do_stmt_K) (for_stmt_K) (if_stmt_K) (while_stmt_K) (pragma_node_K) (omp_pragma_K) (omp_for_pragma_K) (omp_parallel_pragma_K) (omp_sections_pragma_K) (omp_parallel_sections_pragma_K) (omp_section_pragma_K) (map_pragma_K) (call_hw_pragma_K) (call_point_hw_pragma_K) (issue_pragma_K) (blackbox_pragma_K) (profiling_pragma_K) (statistical_profiling_K) (null_node_K) (gimple_predict_K) (widen_sum_expr_K) (widen_mult_expr_K) (none_K) (last_tree_K)

enum kind {
   BOOST_PP_SEQ_FOR_EACH(TREE_NODE_KIND, BOOST_PP_EMPTY, TREE_NODE_LIST)
};

const std::string tree_node_names[] = {
   BOOST_PP_SEQ_FOR_EACH(TREE_NODE_NAME, BOOST_PP_EMPTY, TREE_NODE_LIST)
};

/**
* Macro which define a function that return the parameter as a enum kind.
*/
#define GET_KIND(meth) enum kind get_kind() const {return (meth##_K);}

#endif
