/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file tree_node.hpp
 * @brief Classes specification of the tree_node data structures.
 *
 * Classes used to described the tree nodes imported from the raw file.
 * The first version of this code is due to Fabrizio Ferrandi, Katia Turati, Giacomo Galbiati
 *
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef TREE_NODE_HPP
#define TREE_NODE_HPP

///Autoheader include
#include "config_HAVE_CODE_ESTIMATION_BUILT.hpp"

///STL include
#include <list>
#include <map>
#include <set>
#include <tr1/unordered_map>
#include <vector>

///Tree include
#include "token_interface.hpp"
#include "tree_common.hpp"

///Utility include
#include "exceptions.hpp"

/**
 * @name forward declarations
 */
//@{
REF_FORWARD_DECL(bloc);
REF_FORWARD_DECL(tree_manager);
REF_FORWARD_DECL(tree_node);
#if HAVE_CODE_ESTIMATION_BUILT
REF_FORWARD_DECL(WeightInformation);
#endif
//@}

#define NON_LEAF_TREE_NODES (tree_node) (WeightedNode) (attr) (srcp) (vuse) (vdef) (vops) (decl_node) (expr_node) (expr_node_vops) (unary_expr) (binary_expr) (ternary_expr) (quaternary_expr) (type_node) (memory_tag) (cst_node)

#define UNARY_EXPRESSION_TREE_NODES (abs_expr)\
   (addr_expr)\
   (arrow_expr)\
   (bit_not_expr)\
   (buffer_ref)\
   (card_expr)\
   (cast_expr)\
   (cleanup_point_expr)\
   (conj_expr)\
   (convert_expr)\
   (exit_expr)\
   (fix_ceil_expr)\
   (fix_floor_expr)\
   (fix_round_expr)\
   (fix_trunc_expr)\
   (float_expr)\
   (imagpart_expr)\
   (indirect_ref)\
   (loop_expr)\
   (negate_expr)\
   (non_lvalue_expr)\
   (nop_expr)\
   (realpart_expr)\
   (reference_expr)\
   (reinterpret_cast_expr)\
   (gimple_resx)\
   (sizeof_expr)\
   (static_cast_expr)\
   (throw_expr)\
   (truth_not_expr)\
   (unsave_expr)\
   (va_arg_expr)\
   (view_convert_expr) \
   (gimple_predict)

#define BINARY_EXPRESSION_TREE_NODES \
   (bit_and_expr)\
   (bit_ior_expr)\
   (bit_xor_expr)\
   (catch_expr)\
   (ceil_div_expr)\
   (ceil_mod_expr)\
   (complex_expr)\
   (compound_expr)\
   (eh_filter_expr)\
   (eq_expr)\
   (exact_div_expr)\
   (fdesc_expr)\
   (floor_div_expr)\
   (floor_mod_expr)\
   (ge_expr)\
   (gt_expr)\
   (goto_subroutine)\
   (in_expr)\
   (init_expr)\
   (le_expr)\
   (lrotate_expr)\
   (lshift_expr)\
   (lt_expr)\
   (max_expr)\
   (mem_ref)\
   (min_expr)\
   (minus_expr)\
   (mult_expr)\
   (ne_expr)\
   (ordered_expr)\
   (plus_expr)\
   (pointer_plus_expr)\
   (postdecrement_expr)\
   (postincrement_expr)\
   (predecrement_expr)\
   (preincrement_expr)\
   (range_expr)\
   (rdiv_expr)\
   (round_div_expr)\
   (round_mod_expr)\
   (rrotate_expr)\
   (rshift_expr)\
   (set_le_expr)\
   (trunc_div_expr)\
   (trunc_mod_expr)\
   (truth_and_expr)\
   (truth_andif_expr)\
   (truth_or_expr)\
   (truth_orif_expr)\
   (truth_xor_expr)\
   (try_catch_expr)\
   (try_finally)\
   (uneq_expr)\
   (ltgt_expr)\
   (unge_expr)\
   (ungt_expr)\
   (unle_expr)\
   (unlt_expr)\
   (unordered_expr)\
   (widen_sum_expr)\
   (widen_mult_expr)\
   (with_size_expr)

#define TERNARY_EXPRESSION_TREE_NODES \
   (cond_expr)\
   (component_ref)\
   (bit_field_ref)\
   (vtable_ref)\
   (with_cleanup_expr)\
   (obj_type_ref)\
   (save_expr)

#define QUATERNARY_EXPRESSION_TREE_NODES \
   (array_range_ref)\
   (array_ref)

#define MISCELLANEOUS_EXPR_TREE_NODES \
   (gimple_asm)\
   (gimple_bind)\
   (gimple_call)\
   (call_expr)\
   (case_label_expr)\
   (gimple_cond)\
   (exit_block_expr)\
   (filter_expr)\
   (gimple_goto)\
   (labeled_block_expr)\
   (gimple_label)\
   (modop_expr)\
   (new_expr)\
   (placeholder_expr)\
   (gimple_return)\
   (scope_ref)\
   (gimple_switch)\
   (target_expr)\
   (template_id_expr)\
   (vec_new_expr)\
   

#define MISCELLANEOUS_OBJ_TREE_NODES \
   (baselink)\
   (binfo)\
   (block)\
   (compound_stmt)\
   (constructor)\
   (ctor_initializer)\
   (decl_stmt)\
   (error_mark)\
   (expr_stmt)\
   (gimple_assign)\
   (gimple_nop)\
   (handler)\
   (identifier_node)\
   (overload)\
   (gimple_phi)\
   (return_stmt)\
   (ssa_name)\
   (statement_list)\
   (target_mem_ref)\
   (target_mem_ref461)\
   (template_parm_index)\
   (trait_expr)\
   (tree_list)\
   (tree_vec)\
   (try_block)\
   

#define TYPE_NODE_TREE_NODES \
   (array_type)\
   (boolean_type)\
   (CharType)\
   (complex_type)\
   (enumeral_type)\
   (function_type)\
   (integer_type)\
   (lang_type)\
   (method_type)\
   (offset_type)\
   (pointer_type)\
   (qual_union_type)\
   (real_type)\
   (record_type)\
   (reference_type)\
   (set_type)\
   (template_type_parm)\
   (typename_type)\
   (union_type)\
   (vector_type)\
   (void_type)

#define CONST_OBJ_TREE_NODES \
   (complex_cst)\
   (integer_cst)\
   (real_cst)\
   (string_cst)\
   (vector_cst)

#define DECL_NODE_TREE_NODES \
   (const_decl)\
   (field_decl)\
   (function_decl)\
   (label_decl)\
   (namespace_decl)\
   (parm_decl)\
   (result_decl)\
   (template_decl)\
   (translation_unit_decl)\
   (type_decl)\
   (var_decl)

/// sequence of memory_tag tree nodes
#define MEMORY_TAG_TREE_NODES \
   (memory_partition_tag)\
   (name_memory_tag)\
   (struct_field_tag)\
   (symbol_memory_tag)

/// basic block tree_node
#define BASIC_BLOCK_TREE_NODES (bloc)

/// pragma tree_node
#define PANDA_PRAGMA_TREE_NODES (pragma_node) (issue_pragma) (blackbox_pragma) (profiling_pragma) (statistical_profiling) \
                   (map_pragma) (call_hw_pragma) (call_point_hw_pragma) (omp_pragma) (null_node) \
                   (omp_for_pragma) (omp_parallel_pragma) (omp_sections_pragma) (omp_parallel_sections_pragma) (omp_section_pragma)

/// sequence of tree_node extensions
#define PANDA_EXTENSION_TREE_NODES PANDA_PRAGMA_TREE_NODES (while_expr) (for_expr) (tree_reindex)

/// sequence of all objects
#define VISITED_OBJ_SEQ NON_LEAF_TREE_NODES UNARY_EXPRESSION_TREE_NODES BINARY_EXPRESSION_TREE_NODES TERNARY_EXPRESSION_TREE_NODES QUATERNARY_EXPRESSION_TREE_NODES MISCELLANEOUS_EXPR_TREE_NODES MISCELLANEOUS_OBJ_TREE_NODES TYPE_NODE_TREE_NODES CONST_OBJ_TREE_NODES DECL_NODE_TREE_NODES MEMORY_TAG_TREE_NODES BASIC_BLOCK_TREE_NODES PANDA_EXTENSION_TREE_NODES

/// sequence of obj that have to be specialized
#define OBJ_SPECIALIZED_SEQ (tree_node) (WeightedNode) (attr) (srcp) (vuse) (vdef) (vops) (decl_node)\
         (expr_node) (expr_node_vops) (unary_expr) (binary_expr) (ternary_expr) (quaternary_expr) (type_node) (memory_tag) (cst_node)\
   (array_type) (gimple_asm) (baselink) (gimple_bind) (binfo) (block) (call_expr) (gimple_call) (case_label_expr) (complex_cst)\
         (complex_type) (compound_stmt) (gimple_cond) (const_decl) (constructor) (decl_stmt) (enumeral_type) (exit_block_expr)\
         (expr_stmt) (field_decl) (function_decl) (function_type) (gimple_assign) (gimple_goto) (handler) (identifier_node)\
         (integer_cst) (integer_type) (label_decl) (labeled_block_expr) (gimple_label) (method_type) (namespace_decl) (overload) (parm_decl) (gimple_phi)\
         (pointer_type) (real_cst) (real_type) (record_type) (reference_type) (result_decl) (gimple_return) (return_stmt) (scope_ref)\
         (ssa_name) (statement_list) (string_cst) (gimple_switch) (target_expr) (template_decl) (tree_list) (tree_vec) (try_block)\
         (type_decl) (union_type) (var_decl) (vector_cst) (vector_type) (target_mem_ref) (target_mem_ref461)\
         (bloc) (null_node) (pragma_node) (issue_pragma) (blackbox_pragma) (profiling_pragma) (statistical_profiling) \
         (map_pragma) (call_hw_pragma) (call_point_hw_pragma) (omp_pragma) \
         (omp_for_pragma) (omp_parallel_pragma) (omp_sections_pragma) (omp_parallel_sections_pragma) (omp_section_pragma) (while_expr) (for_expr) (tree_reindex)

#include "visitor.hpp"

typedef object_visitor tree_node_visitor;

/**
 * Abstract pure class for the tree structure. This node and in particular its refCount type will be used to describe all
 * nodes readed from the tree gcc raw structure.
*/
class tree_node
{
   private:
      /// Map string to corresponding enum
      static
      std::map<std::string, enum kind> string_to_kind;

   public:
      /**
       * Represent the index readed from the raw file and the index-1 of the vector
       * of tree_node associated to the functions vector present in the tree_manager.
      */
      const unsigned int index;

      /**
       * Constructor
       */
      tree_node(unsigned int i) : index(i){}

      /// Destructor
      virtual ~tree_node() {}

      /**
      * Virtual function returning the type of the actual class
      */
      virtual enum kind get_kind() const = 0;

      /**
       * Virtual function returning the name of the actual class.
      */
      virtual std::string get_kind_text() const = 0;

      /**
       * virtual function used to traverse the tree_node data structure.
       * @param v is a reference to the tree_node visitor class
       */
      virtual void visit(tree_node_visitor * const v) const;

      /**
       * Friend definition of the << operator.
       * @param os is the output stream
       * @param tn is the tree_node to be printed
       */
      friend std::ostream& operator<<(std::ostream& os, const tree_node * tn);

      /**
       * Friend definition of the << operator.
       * @param os is the output stream
       * @param tn is the tree_node to be printed
       */
      friend std::ostream& operator<<(std::ostream& os, const tree_nodeRef tn);

      /**
       * Print this node as string in gimple format
       * @return the string containing the gimple representation of this node
       */
      std::string ToString() const;

      /**
      * Given a string, return the corresponding kind
      * @param name is the string
      * @return the corresponding kind
      */
      static
      enum kind get_kind(const std::string name);

};


/**
 * RefCount type definition of the tree_node class structure
*/
typedef refcount<tree_node> tree_nodeRef;
typedef refcount<const tree_node> tree_nodeConstRef;


/**
 * Macro used to hide implementation details when accesing a tree_node from another tree_node
 * @param t is the tree_nodeRef to access
 * @return the pointer to t 
*/
#ifndef NDEBUG
#define GET_NODE(t) (t ? ( GetPointer<tree_reindex>(t) ? (GetPointer<tree_reindex>(t))->actual_tree_node : throw_error(t, #t, __PRETTY_FUNCTION__, __FILE__, __LINE__)) : throw_error(t, #t, __PRETTY_FUNCTION__, __FILE__, __LINE__))
#define GET_CONST_NODE(t) (t ? ( GetPointer<const tree_reindex>(t) ? (GetPointer<const tree_reindex>(t))->actual_tree_node : throw_error(t, #t, __PRETTY_FUNCTION__, __FILE__, __LINE__)) : throw_error(t, #t, __PRETTY_FUNCTION__, __FILE__, __LINE__))
#else
#define GET_NODE(t) (GetPointer<tree_reindex>(t))->actual_tree_node  
#define GET_CONST_NODE(t) (GetPointer<const tree_reindex>(t))->actual_tree_node  
#endif

/**
 * Macro used to hide implementation details when accesing a tree_node from another tree_node
 * @param t is the tree_nodeRef to access
 * @return the index of t in tree_manager
*/
#define GET_INDEX_NODE(t) (GET_NODE(t))->index


/**
 * This macro collects all case labels for unary_expr objects.
 * Its use it is quite simple: just add the following line in the switch statement
 * case CASE_UNARY_EXPRESSION:
 */
#define CASE_UNARY_EXPRESSION \
   abs_expr_K:\
   case addr_expr_K:\
   case arrow_expr_K:\
   case bit_not_expr_K:\
   case buffer_ref_K:\
   case card_expr_K:\
   case cast_expr_K:\
   case cleanup_point_expr_K:\
   case conj_expr_K:\
   case convert_expr_K:\
   case exit_expr_K:\
   case fix_ceil_expr_K:\
   case fix_floor_expr_K:\
   case fix_round_expr_K:\
   case fix_trunc_expr_K:\
   case float_expr_K:\
   case imagpart_expr_K:\
   case indirect_ref_K:\
   case loop_expr_K:\
   case negate_expr_K:\
   case non_lvalue_expr_K:\
   case nop_expr_K:\
   case realpart_expr_K:\
   case reference_expr_K:\
   case reinterpret_cast_expr_K:\
   case gimple_resx_K:\
   case sizeof_expr_K:\
   case static_cast_expr_K:\
   case throw_expr_K:\
   case truth_not_expr_K:\
   case unsave_expr_K:\
   case va_arg_expr_K:\
   case view_convert_expr_K:\
   case gimple_predict_K

/**
 * This macro collects all case labels for binary_expr objects.
 */
#define CASE_BINARY_EXPRESSION \
   bit_and_expr_K:\
   case bit_ior_expr_K:\
   case bit_xor_expr_K:\
   case catch_expr_K:\
   case ceil_div_expr_K:\
   case ceil_mod_expr_K:\
   case complex_expr_K:\
   case compound_expr_K:\
   case eh_filter_expr_K:\
   case eq_expr_K:\
   case exact_div_expr_K:\
   case fdesc_expr_K:\
   case floor_div_expr_K:\
   case floor_mod_expr_K:\
   case ge_expr_K:\
   case gt_expr_K:\
   case goto_subroutine_K:\
   case in_expr_K:\
   case init_expr_K:\
   case le_expr_K:\
   case lrotate_expr_K:\
   case lshift_expr_K:\
   case lt_expr_K:\
   case max_expr_K:\
   case mem_ref_K:\
   case min_expr_K:\
   case minus_expr_K:\
   case mult_expr_K:\
   case ne_expr_K:\
   case ordered_expr_K:\
   case plus_expr_K:\
   case pointer_plus_expr_K:\
   case postdecrement_expr_K:\
   case postincrement_expr_K:\
   case predecrement_expr_K:\
   case preincrement_expr_K:\
   case range_expr_K:\
   case rdiv_expr_K:\
   case round_div_expr_K:\
   case round_mod_expr_K:\
   case rrotate_expr_K:\
   case rshift_expr_K:\
   case set_le_expr_K:\
   case trunc_div_expr_K:\
   case trunc_mod_expr_K:\
   case truth_and_expr_K:\
   case truth_andif_expr_K:\
   case truth_or_expr_K:\
   case truth_orif_expr_K:\
   case truth_xor_expr_K:\
   case try_catch_expr_K:\
   case try_finally_K:\
   case uneq_expr_K:\
   case ltgt_expr_K:\
   case unge_expr_K:\
   case ungt_expr_K:\
   case unle_expr_K:\
   case unlt_expr_K:\
   case unordered_expr_K:\
   case widen_sum_expr_K:\
   case widen_mult_expr_K:\
   case with_size_expr_K

/**
 * This macro collects all case labels for ternary_expr objects.
 */
#define CASE_TERNARY_EXPRESSION \
   component_ref_K:\
   case bit_field_ref_K:\
   case vtable_ref_K:\
   case with_cleanup_expr_K:\
   case obj_type_ref_K:\
   case save_expr_K:\
   case cond_expr_K

/**
 * This macro collects all case labels for quaternary_expr objects.
 */
#define CASE_QUATERNARY_EXPRESSION \
   array_range_ref_K:\
   case array_ref_K

/// macro to create simple tree classes
#define CREATE_TREE_NODE_CLASS(class_name,superclass) \
struct class_name : public superclass                \
{                                                     \
   GET_KIND_TEXT(class_name);                         \
   GET_KIND(class_name);                              \
   virtual void visit(tree_node_visitor * const v) const;\
   enum {GETID(superclass)=0};                       \
   class_name(unsigned int i) : superclass(i) {}\
}

/**
 * struct definition of common part of WeightedNode (gimple_assign, expr_node)
 */
struct WeightedNode : public tree_node
{
#if HAVE_CODE_ESTIMATION_BUILT
   WeightInformationRef weight_information;
#endif
   /// visitor enum
   enum {GETID(tree_node)=0, GETID(virtual_operands)};

   /**
    * Constructor
    */
   WeightedNode(unsigned int i);

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;
};


/**
 * struct definition of the field attr on function_decl, field_decl, var_decl tree node.
 * The tree walker structure of this field is:
 * (TOK_NEW | TOK_DELETE | TOK_ASSIGN | TOK_MEMBER | TOK_PUBLIC | TOK_PRIVATE | TOK_PROTECTED | TOK_NORETURN | TOK_VOLATILE | TOK_NOINLINE | TOK_ALWAYS_INLINE | TOK_USED |
 *  TOK_UNUSED | TOK_CONST | TOK_TRANSPARENT_UNION | TOK_CONSTRUCTOR | TOK_DESTRUCTOR |
 *  TOK_MODE | TOK_SECTION | TOK_ALIGNED | TOK_WEAK | TOK_ALIAS | TOK_NO_INSTRUMENT_FUNCTION | TOK_MALLOC | TOK_NO_STACK_LIMIT |
 *  TOK_PURE | TOK_DEPRECATED | TOK_VECTOR_SIZE | TOK_VISIBILITY | TOK_TLS_MODEL | TOK_NONNULL | TOK_NOTHROW | TOK_MAY_ALIAS |
 *  TOK_WARN_UNUSED_RES | TOK_FORMAT | TOK_FORMAT_ARG | TOK_NULL |
 *  TOK_GLOBAL_INIT | TOK_GLOBAL_FINI | TOK_CONVERSION | TOK_VIRTUAL | TOK_LSHIFT | TOK_PSEUDO | TOK_TEMPL | TOK_MUTABLE
 *  TOK_VECNEW | TOK_VECDELETE | TOK_POS | TOK_NEG | TOK_ADDR | TOK_DEREF | TOK_NOT | TOK_LNOT | TOK_PREINC | TOK_PREDEC | TOK_PLUSASSIGN | TOK_PLUS |
 *  TOK_MINUSASSIGN | TOK_MINUS | TOK_MULTASSIGN | TOK_MULT | TOK_DIVASSIGN | TOK_DIV | TOK_MODASSIGN | TOK_MOD | TOK_ANDASSIGN | TOK_AND |
 *  TOK_ORASSIGN |TOK_OR | TOK_XORASSIGN | TOK_XOR | TOK_LSHIFTASSIGN | TOK_RSHIFTASSIGN | TOK_RSHIFT | TOK_EQ | TOK_NE | TOK_LT | TOK_GT |
 *  TOK_LE | TOK_GE | TOK_LAND | TOK_LOR | TOK_COMPOUND | TOK_MEMREF | TOK_REF | TOK_SUBS | TOK_POSTINC | TOK_POSTDEC | TOK_CALL |
 *  TOK_THUNK | TOK_THIS_ADJUSTING | TOK_RESULT_ADJUSTING )*
 */
struct attr
{
   /// list of TOKEN, represented as int, associated to the tree_node
   std::set<unsigned int> list_attr;

   /**
    * Destructor
    */
   virtual ~attr();

   /**
    * Add an attribute to list of attribute.
    * @param a is the token number of the token attribute.
   */
   void add(const unsigned int a)
   {
      list_attr.insert(a);
   }

   /// Return true if there is TOK_CONSTRUCTOR in the list of attributes
   bool is_constructor();

   /// Return true if there is TOK_DESTRUCTOR in the list of attributes
   bool is_destructor();

   /// Return true if there is TOK_MEMBER in the list of attributes
   bool is_member();

   /// returns true if there is a TOK_CALL in the list of attributes
   bool is_call();

   /// returns true if there is a TOK_NEW in the list of attributes
   bool is_new();

   /// returns true if there is a TOK_PUBLIC in the list of attributes
   bool is_public();

   /// returns true if there is a TOK_PRIVATE in the list of attributes
   bool is_private();

   /// returns true if there is a TOK_PROTECTED in the list of attributes
   bool is_protected();

   /// returns true if there is a TOK_BITFIELD in the list of attributes
   bool is_bitfield();

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

};


/**
 * struct definition of the source position.
 * The tree walker structure of this node is:
 * #(TOK_SRCP TOK_STRING TOK_NUMBER TOK_NUMBER);
*/
struct srcp
{
   /**
    * include_name is a filename string, 
    * this can be the location of a reference, if no definition has been seen.
    * (macro DECL_SOURCE_FILE)
   */
   std::string include_name;

   /**
    * line_number holds a line number.
    * (macro DECL_SOURCE_LINE)
   */
   unsigned int line_number;  

   /**
    * column_number holds the column number.
   */
   unsigned int column_number;

   /**
    * constructor
    */
   srcp(): line_number(0), column_number(0){}

   /**
    * Destructor
    */
   virtual ~srcp();

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;
};

/**
 * virtual operand use data structure
 * This is the set of all the virtual variables used by S.
 */
struct vuse
{
   /// list of vuse operands
   std::vector<tree_nodeRef> list_of_vuse_ops;

   /// loads in a vuse operand
   std::vector<tree_nodeRef> list_of_loads;

   /**
    * Destructor
    */
   virtual ~vuse();

   /**
    * Add a vuse_op to list of virtual operands.
    * @param a is a NODE_ID.
   */
   void add_vuse_op(const tree_nodeRef a)
   {
      list_of_vuse_ops.push_back(a);
   }

   /**
    * Add a load to list of loads.
    * @param a is a NODE_ID.
    */
   void add_loads(const tree_nodeRef a)
   {
      list_of_loads.push_back(a);
   }

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(list_of_vuse_ops)=0, GETID(list_of_loads)};
};

/// refcount type for vuse object
typedef refcount<vuse> vuseRef;

/**
 * virtual operand definition data structure
 * This is the set of all the virtual variables defined or stored by the statement S. VDEF operators represent non-killing definitions because
 * they may or may not occur at run time. A VDEF operator is of the form V = VDEF <V>, which means that a new value for V is created from V's old
 * value 
 */
struct vdef
{
   tree_nodeRef vdefinition;

   /// list of vdef operands
   std::vector<tree_nodeRef> list_of_vdef_ops;

   /// stores in a vdef operand
   std::vector<tree_nodeRef> list_of_stores;

   /**
    * Destructor
    */
   virtual ~vdef();

   /**
    * Add a vdef_op to list of virtual operands.
    * @param a is a NODE_ID.
    */
   void add_vdef_op(const tree_nodeRef a)
   {
      list_of_vdef_ops.push_back(a);
   }

   /**
    * Add a stor to list of stores.
    * @param a is a NODE_ID.
    */
   void add_stores(const tree_nodeRef a)
   {
      list_of_stores.push_back(a);
   }

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   
   /// visitor enum
   enum { GETID(vdefinition)=0, GETID(list_of_vdef_ops), GETID(list_of_stores)};
};

/// refcount type for vdef object
typedef refcount<vdef> vdefRef;

/**
 * virtual operators data structure
 * (TOK_VUSE (TOK_VUSE_OP vuse_op )* TOK_LOADS loads)* (TOK_VDEF (vdefinition)? (TOK_VDEF_OP vdef_op )* TOK_STORES stores)*
 *
 * Tree SSA implements two types of operands: real and virtual. Real operands are those that represent a single, non-aliased, memory location
 * which is atomically read or modified by the statement (i.e., variables of non-aggregate types whose address is not taken). Virtual operands
 * represent either partial or aliased references (i.e., structures, unions, pointer dereferences and aliased variables). Since the SSA form uses
 * a versioning scheme on variable names, in principle it would not be possible to assign version numbers to virtual operands. So, the compiler
 * associates a symbol name to the operand and provides SSA versioning for that symbol. Symbols for virtual operands are either created or
 * derived from the original operand.
 *
 * The SSA form is not suited for handling non-scalar variable types. For instance, given an array M[100][100], not only would the compiler need
 * to keep track of 10,000 different version numbers for M, but it may also be impossible to determine whether two references M[i][j] and M[k][l]
 * are the same variable or not. Structures, unions and aliased variables present similar problems.
 * One alternative to handling non-scalar types would be to simply ignore them. After all, if the operands are not converted into SSA form,
 * they would not be considered for optimization. However, that would also mean that statements referencing nothing but non-scalars would appear
 * dead to the optimizers. Also, situations like scalar variables aliased by a structure field would also be missed.
 * To address this problem, operands referencing non-scalar variables are considered references to the base object for that variable. For
 * instance, references to M[i][j] and M[k][l] in the previous example would be considered references to M. Since these operands need to
 * be treated separately by the optimizers, they are known as virtual operands, as opposed to the real operands for scalar variables. Therefore,
 * every GIMPLE statement S contains four distinct sets of operands:
 *   - DEF(S): If S is an assignment statement, this is the variable at its left-hand side.
 *   - USES(S) is the set of all the variables used (or loaded) by S.
 *   - VDEFS(S) is the set of all the virtual variables defined (or stored) by S. VDEF operators represent non-killing deﬁnitions because
 *     they may or may not occur at run time. A VDEF operator is of the form V = VDEF<V>, which means that a new value for V is created from 
 *     V’s old value.
 *   - VUSES(S) is the set of all the virtual variables sused by S.
 * Virtual operands are also used to handle situations where the program is altering variables in ways that are difficult or impossible to 
 * determine statically. In these cases, the data flow framework needs to gather enough information to prevent the optimizers from missing a
 * potential data dependency. In all these cases, virtual operands are used. Some of the more common situations are:
 *   - Aliasing. If two variables a and b may alias each other, then the compiler selects one of them to serve as the representative
 *     for all the aliased references. Every reference to either variable is then considered a virtual operand using the alias representative.
 *   - Call clobbering. Function calls may modify addressable local variables and globals in unknown ways. This is handled using a similar
 *     approach. Variables that may be call clobbered are considered alias of an artificial variable called <i>.global_var</i>. This variable is
 *     considered modified by function calls and by assignments to any of the variables associated with it.
 *   - Inline assembly. Much like function calls, inline assembly may modify local variables in ways that the optimizers do not understand.
 *     variables listed in the outputs or clobbers list of GCC’s asm statement, are considered VDEF operands.
 */
struct vops
{
   /// list of virtual operands uses
   std::vector<vuseRef> list_of_vuses;

   /// list of virtual operands definitions
   std::vector<vdefRef> list_of_vdefs;

   /**
    * Destructor
    */
   virtual ~vops();

   /**
    * Add a vuse to the list of vuses.
    * @param a is a vuseRef.
   */
   void add_vuse(const vuseRef a)
   {
      list_of_vuses.push_back(a);
   }

   /**
    * Add a vdef to the list of vdefs.
    * @param a is a vdefRef.
    */
   void add_vdef(const vdefRef a)
   {
      list_of_vdefs.push_back(a);
   }

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum { GETID(list_of_vuses)=0, GETID(list_of_vdefs)};
};

/// refcount type for vops object
typedef refcount<vops> vopsRef;

/**
 * struct definition of the declaration node structures.
 * struct definition of the field decla.
 * The tree walker structure of this field is:
 * (name)? (mngl)? (type)? (scpe)? (srcp)? (TOK_ARTIFICIAL)? (chan)?
*/
struct decl_node : public srcp, public tree_node
{
   /**
    * name field contains an identifier_node used to represent a name.
    * (tree-dump.c use the macro DECL_NAME)
   */
   tree_nodeRef name;

   /**
    * mngl field contains the name of the object as the assembler will see it.
    * Often this is the same as DECL_NAME
    * It is an identifier_node
   */
   tree_nodeRef mngl;

   /**
    * For any sort of a ..._DECL node, this points to the original (abstract)
    * decl node which this decl is an instance of, or else it is NULL indicating
    * that this decl is not an instance of some other decl.  For example,
    * in a nested declaration of an inline function, this points back to the
    * definition.
    */
   tree_nodeRef orig;

   /**
    * type field holds the data type of the object, when relevant.
    * label_decl have no data type.
    * for type_decl type field contents are the type whose name is being declared.
    * (tree-dump.c use the macro TREE_TYPE)
   */
   tree_nodeRef type;

   ///scope declaration
   tree_nodeRef scpe;

   tree_nodeRef attributes;

   /**
    * artificial_flag field is used to indicate that this decl_node represents a compiler-generated entity.
    * (tree-dump.c use the macro DECL_ARTIFICIAL)
   */
   bool artificial_flag;

   /// operating system flag: it's true when this is a variable of operating system library
   bool operating_system_flag;

   /// library system flag: it's true when this is a variable of a standard library (e.g libmath)
   bool library_system_flag;

   /**
    * chan field: the decls in one binding context are chained through this field.
    * (tree-dump.c use the macro TREE_CHAIN)
   */
   tree_nodeRef chan;

   /// is true when the declaration has the C attribute
   bool C_flag;

   /// The uid
   unsigned int uid;

   /**
    * Constructor
    */
   decl_node(unsigned int i);

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum { GETID(srcp)=0, GETID(tree_node), GETID(name), GETID(mngl), GETID(orig), GETID(type), GETID(scpe), GETID(chan), GETID(attributes)};
};

/**
 * struct definition of the common part of an expression
*/
struct expr_node : public srcp, public WeightedNode
{
   /// constructor
   expr_node(unsigned int i) : WeightedNode(i) {}

   /// type of the expression
   tree_nodeRef type;

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum { GETID(srcp)=0, GETID(WeightedNode), GETID(type)};

};

/**
 * struct definition of the common part of a gimple  with virtual operands
 */
struct expr_node_vops : public srcp, public WeightedNode
{
   /// constructor
   expr_node_vops(unsigned int i) : WeightedNode(i) {}

   /// virtual operands
   vopsRef virtual_operands;

   /// list of pragmas associated to the function
   std::vector<tree_nodeRef> pragmas;

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum { GETID(srcp)=0, GETID(WeightedNode), GETID(virtual_operands), GETID(pragmas)};
};


/**
 * struct definition of the unary node structures.
 * The tree walker structure is for example:
 * node_expr: #(TOK_NODE_EXPR type op)
*/
struct unary_expr : public expr_node
{
   /// constructor
   unary_expr(unsigned int i) : expr_node(i) {}

   /// op field is the operand of the unary expression
   tree_nodeRef op;

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum { GETID(expr_node)=0, GETID(op)};
};

/**
 * struct definition of the binary node structures.
 * The tree walker structure is for example:
 * node_expr: #(TOK_NODE_EXPR (type)? op op)
*/
struct binary_expr : public expr_node
{
   /// constructor
   binary_expr(unsigned int i) : expr_node(i) {}

   /// The first operand of the binary expression
   tree_nodeRef op0;

   /// The second operand of the binary expression
   tree_nodeRef op1;

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum { GETID(expr_node)=0, GETID(op0), GETID(op1)};
};

/**
 * struct definition of the ternary node structures.
 * The tree walker structure is for example:
 * node_expr: #(TOK_NODE_EXPR type op op op)
*/
struct ternary_expr : public expr_node
{
   /// constructor
   ternary_expr(unsigned int i) : expr_node(i) {}

   /// The first operand of the ternary expression
   tree_nodeRef op0;

   /// The second operand of the ternary expression
   tree_nodeRef op1;

   /// The third operand of the ternary expression
   tree_nodeRef op2;

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum { GETID(expr_node)=0, GETID(op0), GETID(op1), GETID(op2)};
};

/**
 * struct definition of the quaternary node structures.
 * The tree walker structure is for example:
 * node_expr: #(TOK_NODE_EXPR type op op op op)
*/
struct quaternary_expr : public expr_node
{
   /// constructor
   quaternary_expr(unsigned int i) : expr_node(i) {}

   /// The first operand of the quaternary expression
   tree_nodeRef op0;

   /// The second operand of the quaternary expression
   tree_nodeRef op1;

   /// The third operand of the quaternary expression
   tree_nodeRef op2;

   /// The fourth operand of the quaternary expression
   tree_nodeRef op3;

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum { GETID(expr_node)=0, GETID(op0), GETID(op1), GETID(op2), GETID(op3)};
};

/**
 * struct definition of the type node structures.
 * struct definition of the field type_node.
 * The tree walker structure of this field is:
 * (qual)? (name)? (unql)? (size)? (scpe)? (algn)?
*/
struct type_node : public tree_node
{
   /// constructor
   type_node(unsigned int i) :
      tree_node(i),
      qual(0),
      algn(0),
      system_flag(false)
   {}


   /**
    * qual is the set of type qualifiers for this type.
    * here is represented as token type: (TOK_QUAL_R | TOK_QUAL_V | TOK_QUAL_VR | TOK_QUAL_C | TOK_QUAL_CR | TOK_QUAL_CV | TOK_QUAL_CVR )
    * (macro: TYPE_QUAL_CONST, TYPE_QUAL_VOLATILE, TYPE_QUAL_RESTRICT)
   */
   unsigned int qual;

   /**
   * name field contains info on the name used in the program
   * for this type.It is either a TYPE_DECL node, for types that are typedefs, or an IDENTIFIER_NODE
   * in the case of structs, unions or enums that are known with a tag,
   * or zero for types that have no special name.
   * (macro TYPE_NAME)
   */
   tree_nodeRef name;

   /**
    * unql field, in any member of such a chain, points to the start of the chain.
    * (macro TYPE_MAIN_VARIANT)
   */
   tree_nodeRef unql;

   /**
    * size field contains a tree that is an expression for the size in bits.
    * (macro TYPE_SIZE)
   */
   tree_nodeRef size;

   /**
    * context/scope of the type object.
    * (macro TYPE_CONTEXT)
       */
   tree_nodeRef scpe;

   /**
    * algn field is the alignment necessary for objects of this type.
    * The value is an int, measured in bits.
    * (macro TYPE_ALIGN)
   */
   unsigned int algn;

   /// system flag: it's true when this is a system variable
   bool system_flag;

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum { GETID(tree_node)=0, GETID(name), GETID(unql), GETID(size), GETID(scpe)};
};


/**
 * Memory tags used in tree-ssa to represent memory locations in virtual SSA
 */
struct memory_tag : public decl_node
{
   /// constructor
   memory_tag(unsigned int i) : decl_node(i) {}

   /**
    * list of aliases associated with the memory tag.
    */
   std::vector<tree_nodeRef> list_of_aliases;

   /**
    * Add an alias to the list of aliases.
    * @param a is a NODE_ID.
    */
   void add_alias(const tree_nodeRef a)
   {
      list_of_aliases.push_back(a);
   }

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum { GETID(decl_node)=0, GETID(list_of_aliases)};
};

/**
 * This struct specifies super class for constant nodes
 */
struct cst_node : tree_node
{
   /// constructor
   cst_node(unsigned int i) : tree_node(i) {}

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(tree_node)=0};
};



/**
 * This struct specifies the abs_expr node.
 * Represents the absolute value of the operand.
 * An ABS_EXPR must have either an INTEGER_TYPE or a REAL_TYPE.  The operand of the ABS_EXPR must have the same type.
*/
CREATE_TREE_NODE_CLASS(abs_expr,unary_expr);

/**
 * This struct specifies the addr_expr node.
 * & in C.  Value is the address at which the operand's value resides.
 * Operand may have any mode.  Result mode is Pmode.
*/
CREATE_TREE_NODE_CLASS(addr_expr,unary_expr);

/**
 struct describing an array range. Likewise an array, except that the result is a range ("slice") of the array.  The
   starting index of the resulting array is taken from operand 1 and the size
   of the range is taken from the type of the expression.  
 */
CREATE_TREE_NODE_CLASS(array_range_ref,quaternary_expr);

/** Array indexing.
   Operand 0 is the array; operand 1 is a (single) array index.
   Operand 2, if present, is a copy of TYPE_MIN_VALUE of the index.
   Operand 3, if present, is the element size, measured in units of
   the alignment of the element type.  */
CREATE_TREE_NODE_CLASS(array_ref,quaternary_expr);

/**
 * struct definition of the array_type tree node.
 * The tree walker structure of this node is:
 * #(TOK_ARRAY_TYPE type_node elts domn)
*/
struct array_type : public type_node
{
   /// constructor
   array_type(unsigned int i) : type_node(i) {}

   /*type_node fields are in the parent class: type_node*/

   /// field elts is the type of an array element (tree-dump.c use the macro TREE_TYPE)
   tree_nodeRef elts;

   /// field domn is the type to index by (tree-dump.c use the macro TYPE_DOMAIN). Its range of values specifies the array length.
   tree_nodeRef domn;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(array_type)

   /// Redefinition of get_kind.
   GET_KIND(array_type)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum { GETID(type_node)=0, GETID(elts), GETID(domn)};
};


/**
* This struct reperesent arrow_expr
*/
CREATE_TREE_NODE_CLASS(arrow_expr,unary_expr);

/**
 * This struct specifies the gimple_asm node.
 * Used to represent an inline assembly statement.  ASM_STRING returns a STRING_CST for the instruction (e.g., "mov x, y"). 
 * ASM_OUTPUTS, ASM_INPUTS, and ASM_CLOBBERS represent the outputs, inputs, and clobbers for the statement. 
 * The tree walker structure of this node is:
 * #(TOK_GIMPLE_ASM type (TOK_VOLATILE)? str (out)? in (clob)?)
*/
struct gimple_asm : public expr_node_vops
{
   /// constructor
   gimple_asm(unsigned int i) : expr_node_vops(i), volatile_flag(false) {}

   /// volatile_flag is true if the node is public: it means that the name is accessible from outside.
   bool volatile_flag;

   /// str is the operand 0: ASM_STRING. ASM_STRING returns a STRING_CST for the instruction (e.g., "mov x, y").
   std::string str;

   /// out is the operand 1: ASM_OUTPUTS, this represents the outputs for the statement.
   tree_nodeRef out;

   /// in is the operand 2: ASM_INPUTS, this represents the inputs for the statement.
   tree_nodeRef in;

   /// clob is the operand 3: ASM_CLOBBERS, this represents the clobbers for the statement.
   tree_nodeRef clob;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(gimple_asm)

   /// Redefinition of get_kind.
   GET_KIND(gimple_asm)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum { GETID(expr_node_vops)=0, GETID(out), GETID(in), GETID(clob)};
};


/**
 * This struct represents a reference to a member function or member
 * functions from a base class
 */
struct baselink : public tree_node
{
   /// constructor
   baselink(unsigned int i) : tree_node(i) {}

   /// is the type of the baselink
   tree_nodeRef type;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(baselink)

   /// Redefinition of get_kind.
   GET_KIND(baselink)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum { GETID(tree_node)=0, GETID(type)};
};

/**
 * GIMPLE_BIND <VARS, BLOCK, BODY> represents a lexical scope.
 * VARS is the set of variables declared in that scope.
 * BLOCK is the symbol binding block used for debug information.
 * BODY is the sequence of statements in the scope.
 * #(TOK_GIMPLE_BIND (vars)? body)
 */
struct gimple_bind : public expr_node
{
   /// constructor
   gimple_bind(unsigned int i) : expr_node(i) {}

   /// vars is the operand 0 (GIMPLE_BIND_VARS), this is a chain of VAR_DECL nodes for the variables.
   std::vector<tree_nodeRef> list_of_vars;

   /// body is the operand 1 (GIMPLE_BIND_BODY), this is the body, the expression to be computed using the variables. The value of operand 1 becomes that of the GIMPLE_BIND.
   tree_nodeRef body;

   /**
    * Add a var to list of vars.
    * @param a is a NODE_ID.
   */
   void add_vars(const tree_nodeRef a)
   {
      list_of_vars.push_back(a);
   }

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(gimple_bind)

   /// Redefinition of get_kind.
   GET_KIND(gimple_bind)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(expr_node)=0, GETID(list_of_vars), GETID(body)};
};

/**
 * This struct specifies the binfo node.
 * The tree walker structure of this node is:
 * #(TOK_BINFO (type)? (TOK_VIRT)? (baseinfo)*
*/
struct binfo : public tree_node
{
   /// constructor
   binfo(unsigned int i) : tree_node(i), virt_flag(false) {}

   /// type field is the actual data type node being inherited in this basetype.(BINFO_TYPE)
   tree_nodeRef type;

   /// virt_flag is true if the node is a virtual declaration (macro TREE_VIA_VIRTUAL)
   bool virt_flag;

   /// is the list of pair access binf: baseinfo vector.
   std::vector<std::pair<treeVocabularyTokenTypes::token_enum, tree_nodeRef> > list_of_access_binf;

   /**
    * return the i-th element of baseinfo
    * @param i is the index of baseinfo vector
   */
   tree_nodeRef get_base(size_t i)
   {
      return list_of_access_binf[i].second;
   }

   /// return the size of baseinfo vector
   size_t get_baseinfo_size()
   {
      return list_of_access_binf.size();
   }

   /**
    * Add a pair <access, binf> to the list of access binf.
    * @param binf is the binf.
    * @param access is a token between TOK_PUB TOK_PROT TOK_PRIV.
   */
   void add_access_binf(const tree_nodeRef binf, treeVocabularyTokenTypes::token_enum access);

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(binfo)

   /// Redefinition of get_kind.
   GET_KIND(binfo)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(tree_node)=0, GETID(type), GETID(list_of_access_binf)};
};


/**
 * This struct specifies the bit_and_expr node.
 * Bitwise operation.  Operands have same mode as result.
*/
CREATE_TREE_NODE_CLASS(bit_and_expr,binary_expr);

/**
 * This struct specifies the bit_field_ref node.
 * Reference to a group of bits within an object.  Similar to COMPONENT_REF except the position is given explicitly rather than via a FIELD_DECL.
 * Operand 0 is the structure or union expression;
 * operand 1 is a tree giving the number of bits being referenced;
 * operand 2 is a tree giving the position of the first referenced bit.
 * The field can be either a signed or unsigned field; BIT_FIELD_REF_UNSIGNED says which.
*/
CREATE_TREE_NODE_CLASS(bit_field_ref,ternary_expr);

/**
 * This struct specifies the bit_ior_expr node.
 * Bitwise operation.  Operands have same mode as result.
*/
CREATE_TREE_NODE_CLASS(bit_ior_expr,binary_expr);

/**
 * This struct specifies the bit_not_expr node.
 * Bitwise operation.  Operands have same mode as result.
*/
CREATE_TREE_NODE_CLASS(bit_not_expr,unary_expr);

/**
 * This struct specifies the bit_xor_expr node.
 * Bitwise operation.  Operands have same mode as result.
*/
CREATE_TREE_NODE_CLASS(bit_xor_expr,binary_expr);

/**
 * This struct specifies the block node.
 * A symbol binding block.  These are arranged in a tree, where the BLOCK_SUBBLOCKS field contains a chain of subblocks chained through the BLOCK_CHAIN field.
 * The tree walker structure of this node is:
 * #(TOK_BLOCK)
*/
struct block : public tree_node
{
   /// constructor
   block(unsigned int i) : tree_node(i), bl_flag(false) {}

   /// used to know if bl string is set
   bool bl_flag;

   /// May contain the string "block"
   std::string bl;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(block)

   /// Redefinition of get_kind.
   GET_KIND(block)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(tree_node)=0};
};


/**
 * struct definition of the boolean_type tree node.
 * The tree walker structure of this node is:
 * #(TOK_BOOLEAN_TYPE type_node)
*/
CREATE_TREE_NODE_CLASS(boolean_type,type_node);

/**
 * This struct specifies the buffer_ref node.
 * Pascal `^` on a file.  One operand, an expression for the file.
*/
CREATE_TREE_NODE_CLASS(buffer_ref,unary_expr);


/**
 * This struct specifies the call_expr node.
 * Function call.  Operand 0 is the function.
 * Operand 1 is the argument list, a list of expressions made out of a chain of TREE_LIST nodes.
 * Operand 2 is the static chain argument, or NULL. 
 * The tree walker structure of this node is:
 * #(TOK_CALL_EXPR type fn (args)?)
*/
struct call_expr : public expr_node
{
   /// constructor
   call_expr(unsigned int i) : expr_node(i) {}

   /// fn is the operand 0 of the call expression: this is the function
   tree_nodeRef fn;

   /// The arguments of the call_expr
   std::vector<tree_nodeRef> args;

   /**
    * Add an argument to the list of arguments
    * @param arg is the argument to be added
    */
   void AddArg(const tree_nodeRef arg);

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(call_expr)

   /// Redefinition of get_kind.
   GET_KIND(call_expr)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(expr_node)=0, GETID(type), GETID(fn), GETID(args)};
};

/**
 * This struct specifies the gimple_call node.
 * Function call.  Operand 0 is the function.
 * Operand 1 is the argument list, a list of expressions made out of a chain of TREE_LIST nodes.
 * Operand 2 is the static chain argument, or NULL.
 * The tree walker structure of this node is:
 * #(TOK_GIMPLE_CALL type fn (args)?)
*/
struct gimple_call : public expr_node_vops
{
   /// constructor
   gimple_call(unsigned int i) : expr_node_vops(i) {}

   /// fn is the operand 0 of the call expression: this is the function
   tree_nodeRef fn;

   /// The arguments of the gimple_call
   std::vector<tree_nodeRef> args;

   /**
    * Add an argument to the list of arguments
    * @param arg is the argument to be added
    */
   void AddArg(const tree_nodeRef arg);

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(gimple_call)

   /// Redefinition of get_kind.
   GET_KIND(gimple_call)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(expr_node_vops)=0, GETID(type), GETID(fn), GETID(args)};
};

/**
 * This struct specifies the card_expr node.
 * Operation for Pascal sets.
*/
CREATE_TREE_NODE_CLASS(card_expr,unary_expr);

/**
 * This struct specifies the case_label_expr node.
 * Used to represent a case label. The operands are CASE_LOW (operand 0) and CASE_HIGH (operand 1), respectively. 
 * If CASE_LOW is NULL_TREE, the label is a 'default' label. If CASE_HIGH is NULL_TREE, the label is a normal case label.CASE_LABEL (operand 2) is the corresponding LABEL_DECL.
 * The tree walker structure of this node is:
 * #(TOK_CASE_LABEL_EXPR  type op (op)? (TOK_DEFAULT)? got);
*/
struct case_label_expr : public expr_node
{
   /// constructor
   case_label_expr(unsigned int i) : expr_node(i), default_flag(false) {}

   /// op0 is the operand 0 (macro CASE_LOW) of the case label expression
   tree_nodeRef op0;

   /// op1 is the operand 1 (macro CASE_HIGH) of the case label expression
   tree_nodeRef op1;

   /// default_flag is true if the label is a 'default' label
   bool default_flag;

   /// got field is the operand 2 (macro CASE_LABEL) of the case label expression
   tree_nodeRef got;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(case_label_expr)

   /// Redefinition of get_kind.
   GET_KIND(case_label_expr)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(expr_node)=0, GETID(op0), GETID(op1), GETID(got)};
};


/**
 * This struct specifies the cast_expr node.
 * The walker for this struct is
 *   #(TOK_CAST_EXPR wunary_expr);
*/
CREATE_TREE_NODE_CLASS(cast_expr,unary_expr);

/**
 * This struct specifies the catch_expr node.
 * Evaluate operand 1.  If and only if an exception is thrown during the evaluation of operand 1, evaluate operand 2.
 * This differs from TRY_FINALLY_EXPR in that operand 2 is not evaluated on a normal or jump exit, only on an exception.
*/
CREATE_TREE_NODE_CLASS(catch_expr,binary_expr);

/**
 * This struct specifies the ceil_div_expr node.
 * Division for integer result that rounds the quotient toward infinity.
*/
CREATE_TREE_NODE_CLASS(ceil_div_expr,binary_expr);

/**
 * This struct specifies the ceil_mod_expr node.
 * Kind of remainder that go with the kind of division. 
*/
CREATE_TREE_NODE_CLASS(ceil_mod_expr,binary_expr);

/**
 * struct definition of the CharType tree node.
 * The tree walker structure of this node is:
 * #(TOK_CHAR_TYPE type_node)
*/
CREATE_TREE_NODE_CLASS(CharType,type_node);

/**
 * This struct specifies the cleanup_point_expr node.
 * As a consequence, the operand of a CLEANUP_POINT_EXPR must not have
 * BLKmode, because it will not be forced out of memory.
*/
CREATE_TREE_NODE_CLASS(cleanup_point_expr,unary_expr);

/**
 * This struct specifies the complex_cst node.
 * Contents are in TREE_REALPART and TREE_IMAGPART fields, whose contents are other constant nodes. 
 * The tree walker structure of this node is:
 * #(TOK_COMPLEX_CST type real imag)
*/
struct complex_cst : public cst_node
{
   /// constructor
   complex_cst(unsigned int i) : cst_node(i) {}

   /// type field is the type of the node
   tree_nodeRef type;

   /// real is the TREE_REALPART which content is other constant node.
   tree_nodeRef real;

   /// imag is the TREE_IMAGPART which content is other constant node.
   tree_nodeRef imag;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(complex_cst)

   /// Redefinition of get_kind.
   GET_KIND(complex_cst)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(cst_node)=0, GETID(type), GETID(real), GETID(imag)};
};


/**
 * This struct specifies the complex_expr node.
 * Given two real or integer operands of the same type, returns a complex value of the corresponding complex type.
*/
CREATE_TREE_NODE_CLASS(complex_expr,binary_expr);

/**
 * struct definition of the complex_type tree node.
 * The tree walker structure of this node is:
 * #(TOK_COMPLEX_TYPE type_node);
*/
struct complex_type : public type_node
{
   /// constructor
   complex_type(unsigned int i) : type_node(i), unsigned_flag(false) {}

   /**
    * unsigned means an unsigned type
    * (macro TYPE_UNSIGNED)
    */
   bool unsigned_flag;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(complex_type)

   /// Redefinition of get_kind.
   GET_KIND(complex_type)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(type_node)=0};
};


/** Value is structure or union component.
   Operand 0 is the structure or union (an expression).
   Operand 1 is the field (a node of type FIELD_DECL).
   Operand 2, if present, is the value of DECL_FIELD_OFFSET, measured
   in units of DECL_OFFSET_ALIGN / BITS_PER_UNIT.  */
CREATE_TREE_NODE_CLASS(component_ref,ternary_expr);

/**
 * This struct specifies the compound_expr node.
 * Contains two expressions to compute, one followed by the other. The first value is ignored.  The second one's value is used.  
 * The type of the first expression need not agree with the other types.
*/
CREATE_TREE_NODE_CLASS(compound_expr,binary_expr);

/**
 * This struct specifies the compound_stmt node used to represent a brace-enclosed block.
 * The operand is COMPOUND_BODY.
*/
struct compound_stmt : public tree_node
{
   /// constructor
   compound_stmt(unsigned int i) : tree_node(i), line(-1) {}

   /// line is the line number where the compound_stmt is defined.
   int line;

   /// body is COMPOUND_BODY.
   tree_nodeRef body;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(compound_stmt)

   /// Redefinition of get_kind.
   GET_KIND(compound_stmt)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(tree_node)=0, GETID(body)};
};


/**
 * This struct specifies the cond_expr node.
 * Conditional expression ( ... ? ... : ...  in C).
 * Operand 0 is the condition. Operand 1 is the then-value. Operand 2 is the else-value.
 * Operand 0 may be of any type. Operand 1 must have the same type as the entire expression, unless
 * it unconditionally throws an exception, in which case it should have VOID_TYPE.  The same constraints apply to operand 2. 
*/
CREATE_TREE_NODE_CLASS(cond_expr,ternary_expr);


/**
 * This struct specifies the gimple_cond node.
 * Operand 0 is the condition.
*/
struct gimple_cond : public expr_node_vops
{
   /// constructor
   gimple_cond(unsigned int i) : expr_node_vops(i) {}

   /// The first operand of the ternary expression
   tree_nodeRef op0;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(gimple_cond)

   /// Redefinition of get_kind.
   GET_KIND(gimple_cond)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(expr_node_vops)=0, GETID(op0)};
};


/**
 * This struct specifies the conj_expr node.
 * Complex conjugate of operand.  Used only on complex types.
*/
CREATE_TREE_NODE_CLASS(conj_expr,unary_expr);

/**
 * struct definition of the const_decl tree node.
 * The tree walker structure of this node is:
 * #(TOK_CONST_DECL decl_node (cnst)?)
*/
struct const_decl : public decl_node
{
   /// constructor
   const_decl(unsigned int i) : decl_node(i) {}

   /*decl_node fields are in the parent class*/

   /// field cnst holds the value of a constant (tree-dump.c use the macro DECL_INITIAL)
   tree_nodeRef cnst;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(const_decl)

   /// Redefinition of get_kind.
   GET_KIND(const_decl)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(decl_node)=0, GETID(cnst)};
};


/**
 * Constructor: return an aggregate value made from specified components.
   In C, this is used only for structure and array initializers.
   The operand is a sequence of component values made out of a VEC of
   struct constructor_elt.

   For ARRAY_TYPE:
   The field INDEX of each constructor_elt is the corresponding index.
   If the index is a RANGE_EXPR, it is a short-hand for many nodes,
   one for each index in the range.  (If the corresponding field VALUE
   has side-effects, they are evaluated once for each element.  Wrap the
   value in a SAVE_EXPR if you want to evaluate side effects only once.)

   For RECORD_TYPE, UNION_TYPE, or QUAL_UNION_TYPE:
   The field INDEX of each node is a FIELD_DECL.
*/
struct constructor : public tree_node
{
   /// constructor
   constructor(unsigned int i) : tree_node(i) {}

   /// type field is the type of the node
   tree_nodeRef type;

   /// store the list initializers: <index, value>
   std::vector<std::pair< tree_nodeRef, tree_nodeRef> > list_of_idx_valu;

   /**
    * Add a pair <index value> to the list of idx_val.
    * @param idx is the index.
    * @param val is the value.
    */
   void add_idx_valu(const tree_nodeRef idx, const tree_nodeRef valu)
   {
      list_of_idx_valu.push_back(std::pair< tree_nodeRef, tree_nodeRef>(idx, valu));
   }

   /**
    * Add a pair <null, value> to the list of idx_val.
    * @param val is the value.
    */
   void add_valu(const tree_nodeRef valu)
   {
      list_of_idx_valu.push_back(std::pair< tree_nodeRef, tree_nodeRef>(tree_nodeRef(), valu));
   }

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(constructor)

   /// Redefinition of get_kind.
   GET_KIND(constructor)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(tree_node)=0, GETID(type), GETID(list_of_idx_valu)};
};


/**
 * This struct specifies the convert_expr node.
 * Represents a conversion of type of a value.
 * All conversions, including implicit ones, must be represented by CONVERT_EXPR or NOP_EXPR nodes.
*/
CREATE_TREE_NODE_CLASS(convert_expr,unary_expr);

/**
 * CTOR_INITIALIZER is a placeholder in template code for a call to
 * setup_vtbl_pointer (and appears in all functions, not just ctors)
 */
CREATE_TREE_NODE_CLASS(ctor_initializer,tree_node);

/**
 *  This struct represent a local declaration.
 */
struct decl_stmt : public tree_node
{
   /// constructor
   decl_stmt(unsigned int i) : tree_node(i), line (-1) {}

   /// line is the line number where the compound_stmt is defined.
   int line;

   /// Is the body of the declaration
   tree_nodeRef decl;

   /// Is the next statement
   tree_nodeRef next;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(decl_stmt)

   /// Redefinition of get_kind.
   GET_KIND(decl_stmt)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(tree_node)=0, GETID(decl), GETID(next)};
};

/**
 * This struct specifies the eh_filter_expr node.
 * Used to represent an exception specification. 
 * EH_FILTER_TYPES is a list of allowed types, and EH_FILTER_FAILURE is an expression to evaluate on failure.  
 * EH_FILTER_MUST_NOT_THROW controls which range type to use when expanding.  
*/
CREATE_TREE_NODE_CLASS(eh_filter_expr,binary_expr);

/**
 * struct definition of the integer_type tree node.
 * The tree walker structure of this node is:
 * #(TOK_ENUMERAL_TYPE type_node prec (TOK_UNSIGNED)? min max (csts)?);
*/
struct enumeral_type : public type_node
{
   /// constructor
   enumeral_type(unsigned int i) : type_node(i), prec(0), unsigned_flag(false) {}

   /*type_node fields are in the parent class*/

   /**
    * prec field is the number of bits used by this type
    * (tree-dump.c use the macro TYPE_PRECISION)
   */
   unsigned int prec;

   /**
    * unsigned means an unsigned type
    * (tree-dump.c use the macro TYPE_UNSIGNED)
   */
   bool unsigned_flag;

   /// min: tree-dump.c use the macro TYPE_MIN_VALUE
   tree_nodeRef min;

   /// max: tree-dump.c use the macro TYPE_MAX_VALUE
   tree_nodeRef max;

   /**
    * csts is a list in which each element's TREE_PURPOSE is a name 
    * and the TREE_VALUE is the value (an integer_cst node)
    * (tree-dump.c use the macro TYPE_VALUES)
   */
   tree_nodeRef csts;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(enumeral_type)

   /// Redefinition of get_kind.
   GET_KIND(enumeral_type)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(type_node)=0, GETID(min), GETID(max), GETID(csts)};
};


/**
 * This struct specifies the eq_expr node.
 * Relational operators. EQ_EXPR is allowed for any types.
 * The operands will have the same type, and the value is always the type used by the language for booleans. 
*/
CREATE_TREE_NODE_CLASS(eq_expr,binary_expr);

/**
 * This struct specifies the error_mark node.
 * Any erroneous construct is parsed into a node of this type.
 * This type of node is accepted without complaint in all contexts by later parsing activities, to avoid multiple error messages for one error.
 * No fields in these nodes are used.
 * The tree walker structure of this node is:
 * #(TOK_ERROR_MARK TOK_ERROR)
*/
CREATE_TREE_NODE_CLASS(error_mark,tree_node);

/**
 * This struct specifies the exact_div_expr node.
 * Division which is not supposed to need rounding. Used for pointer subtraction in C.
*/
CREATE_TREE_NODE_CLASS(exact_div_expr,binary_expr);

/**
 * This struct specifies the exc_ptr_expr node.
 * The exception object from the runtime.
 * The tree walker structure of this node is:
 * #(TOK_EXC_PTR_EXPR type TOK_EXCEPTION_OBJECT)
*/
CREATE_TREE_NODE_CLASS(exc_ptr_expr,expr_node);

/**
 * This struct specifies the exit_block_expr node.
 * Exit a labeled block, possibly returning a value.  
 * Operand 0 is a LABELED_BLOCK_EXPR to exit.  Operand 1 is the value to return. It may be left null. 
 * The tree walker structure of this node is:
 * #(TOK_EXIT_BLOCK_EXPR type (string_id)? exit)
*/
struct exit_block_expr : public expr_node
{
   /// constructor
   exit_block_expr(unsigned int i) : expr_node(i) {}

   /// string_id is the name of the label store in the operand 0.(macro: name= IDENTIFIER_POINTER (DECL_NAME (op0) and op0=LABELED_BLOCK_LABEL (EXIT_BLOCK_LABELED_BLOCK (t))))
   std::string string_id;

   /// exit is the operand O of this node: this is a LABELED_BLOCK_EXPR to exit.
   tree_nodeRef exit;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(exit_block_expr)

   /// Redefinition of get_kind.
   GET_KIND(exit_block_expr)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(expr_node)=0, GETID(exit)};
};


/**
 * This struct specifies the exit_expr node.
 * Exit the inner most loop conditionally.  The operand is the condition.
 * The type should be void and the value should be ignored.
*/
CREATE_TREE_NODE_CLASS(exit_expr,unary_expr);

/**
 * This struct represent a statement expression
 */
struct expr_stmt : public tree_node
{
   /// constructor
   expr_stmt(unsigned int i) : tree_node(i), line (-1) {}

   /// line is the line number where the compound_stmt is defined.
   int line;

   /// Is the statement given by the expression
   tree_nodeRef expr;

   /// Is the next statement
   tree_nodeRef next;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(expr_stmt)

   /// Redefinition of get_kind.
   GET_KIND(expr_stmt)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(tree_node)=0, GETID(expr), GETID(next)};
};


/**
 * This struct specifies the fdesc_expr node.
 * Operand 0 is a function constant; result is part N of a function descriptor of type ptr_mode.
*/
CREATE_TREE_NODE_CLASS(fdesc_expr,binary_expr);

/**
 * struct definition of the field_decl tree node.
 * The tree walker structure of this node is:
 * #(TOK_FIELD_DECL decl_node attr (init)? (size)? algn (bpos)?)
*/
struct field_decl : public decl_node, public attr
{
   /// constructor
   field_decl(unsigned int i): decl_node(i), algn(0) {}

   /*decl_node fields are in the parent class: decl_node*/
   /*attr fields are in the parent class: attr*/

   /**
    * init field holds the value to initialize a variable to.
    * (tree-dump.c use the macro DECL_INITIAL)
   */
   tree_nodeRef init;

   /**
    * size field holds the size of datum, in bits.
    * (tree-dump.c use the macro DECL_SIZE)
   */
   tree_nodeRef size;

   /**
    * algn field holds the alignment required for the datum, in bits.
    * (tree-dump.c use the macro DECL_ALIGN)
   */
   unsigned int algn;

   /**
    * bpos field is the field position, counting in bytes, of the byte
    * containing the bit closest to the beginning of the structure
    * (tree-dump.c use the macro DECL_FIELD_OFFSET 
    * and the function bit_position)
   */
   tree_nodeRef bpos;

   /**
    * symbol_memory_tag annotation
    */
   tree_nodeRef smt_ann;


   /**
    + Compute the offset of this field in the struct or structure
    * @return the offset for this field
   */
   long long int offset();

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(field_decl)

   /// Redefinition of get_kind.
   GET_KIND(field_decl)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(decl_node)=0, GETID(attr), GETID(init), GETID(size), GETID(bpos), GETID(smt_ann)};
};


/**
 * This struct specifies the filter_expr node.
 * The filter object from the runtime.
 * The tree walker structure of this node is:
 * #(TOK_FILTER_EXPR type TOK_FILTER_OBJECT)
*/
CREATE_TREE_NODE_CLASS(filter_expr,expr_node);

/**
 * This struct specifies the fix_ceil_expr node.
 * Conversion of real to fixed point.
*/
CREATE_TREE_NODE_CLASS(fix_ceil_expr,unary_expr);

/**
 * This struct specifies the fix_floor_expr node.
 * Conversion of real to fixed point
*/
CREATE_TREE_NODE_CLASS(fix_floor_expr,unary_expr);

/**
 * This struct specifies the fix_round_expr node.
 * Conversion of real to fixed point
*/
CREATE_TREE_NODE_CLASS(fix_round_expr,unary_expr);

/**
 * This struct specifies the fix_trunc_expr node.
 * Conversion of real to fixed point.
*/
CREATE_TREE_NODE_CLASS(fix_trunc_expr,unary_expr);

/**
 * This struct specifies the float_expr node.
 * Conversion of an integer to a real.
*/
CREATE_TREE_NODE_CLASS(float_expr,unary_expr);

/**
 * This struct specifies the floor_div_expr node.
 * Division for integer result that rounds toward minus infinity.
*/
CREATE_TREE_NODE_CLASS(floor_div_expr,binary_expr);

/**
 * This struct specifies the floor_mod_expr node.
 * Kind of remainder that go with the kind of division.
*/
CREATE_TREE_NODE_CLASS(floor_mod_expr,binary_expr);

/**
 * struct definition of the function_decl tree node.
 * The tree walker structure of this node is:
 * #(TOK_FUNCTION_DECL decl_node (TOK_C| TOK_OPERATOR (TOK_STRING)*)? attr (args)? (TOK_UNDEFINED)? (TOK_EXTERN | TOK_STATIC) (body)?)
*/
struct function_decl : public decl_node, public attr
{
   /// constructor
   function_decl(unsigned int i) :
      decl_node(i),
      attr(),
      operator_flag(false),
      fixd_flag(false),
      virt_flag(false),
      undefined_flag(false),
      static_flag(false)
   {}

   /// it is true when the function_decl is an operator
   bool operator_flag;

   /// store the list of operator names
   std::vector<std::string> list_of_op_names;

   /// flags used to know if fixd is set
   bool fixd_flag;

   /// flags used to know if  virt is set
   bool virt_flag;

   /// Is the result of THUNK_FIXED_OFFSET(t) for this tree node
   int fixd;

   /// Is the result of tree_low_cst (THUNK_VIRTUAL_OFFSET (t), 0) for this node
   int virt;

   /// fn field is the initial declaration for this function declaration
   tree_nodeRef fn;

   /**
    * tmpl_parms holds template parameters 
    * It is a TREE_LIST, his VALU field is a TREE_VEC whose LIST_OF_OP holds template parameters. 
    * The instantion of parameter "list_of_op[i]" is "list_of_op[i]" hold in tmpl_args. 
   */
   tree_nodeRef tmpl_parms;

   /**
    * tmpl_args holds template instantiations 
    * It is a TREE_VEC whose LIST_OF_OP holds template instantiations.
    * The parameter of  instantiation "list_of_op[i]" is "list_of_op[i]" hold in tmpl_parms. 
   */
   tree_nodeRef tmpl_args;

   /**
    * args field holds a chain of parm_decl nodes for the arguments.
    * (tree-dump.c use the macro DECL_ARGUMENTS)
   */
   std::vector <tree_nodeRef> list_of_args;

   /**
    * undefined_flag means external reference: 
    * do not allocate storage, and refer to a definition elsewhere.
    * (tree-dump.c use the macro DECL_EXTERNAL)
   */
   bool undefined_flag;

   /**
    * static_flag is true if function has been defined
    * (macro TREE_STATIC)
   */
   bool static_flag;

   /**
    * body field is the saved representation of the body of the entire function.
    * (macro DECL_SAVED_TREE)
   */
   tree_nodeRef body;

   /**
    * java inline body
   */
   tree_nodeRef inline_body;

   /// list of pragmas associated to the function
   std::vector<tree_nodeRef> pragmas;

   /**
    * Add a string to list of attribute.
    * @param a is the token number of the token attribute.
   */
   void add(std::string a)
   {
      list_of_op_names.push_back(a);
   }

   /**
    * Add a parameter to the list of the paramters.
    * @param a is the parameter to be added.
   */
   void AddArg(const tree_nodeRef a);

   /// return true if is a declaration of constructor
   bool is_constructor();

   /// Return true if is a declaration of destructor
   bool is_destructor();

   /// returns true if is a declaration of an operator
   bool is_operator();

   /// returns true if is a declaration of a public function
   bool is_public();

   /// returns true if is a declaration of a private function
   bool is_private();

   /// returns true if is a declaration of a protected function
   bool is_protected();

   /// Return the function mngl
   std::string get_function_mngl();

   /// Return the function name
   std::string get_function_name();

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(function_decl)

   /// Redefinition of get_kind.
   GET_KIND(function_decl)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(decl_node)=0, GETID(attr), GETID(fn), GETID(tmpl_parms), GETID(tmpl_args), GETID(list_of_args), GETID(body), GETID(inline_body), GETID(pragmas)};
};


/**
 * struct definition of the function_type tree node.
 * Type of functions.  Special fields:
 * TREE_TYPE       type of value returned.
 * TYPE_ARG_TYPES  list of types of arguments expected.
 *                 this list is made of TREE_LIST nodes.
 * Types of "Procedures" in languages where they are different from functions
 * have code FUNCTION_TYPE also, but then TREE_TYPE is zero or void type.
 * The tree walker structure of this node is:
 * #(TOK_FUNCTION_TYPE type_node retn (prms)?)
*/
struct function_type : public type_node
{
   /// constructor
   function_type(unsigned int i) : type_node(i), varargs_flag(false) {}

   /// varargs flag: tells if function is of varargs type
   bool varargs_flag;

   /**
    * retn field is the type of value returned.
    * (macro TREE_TYPE)
    */
   tree_nodeRef retn;

   /**
    * prms field is a list of types of arguments expected,
    * this list is made of tree_list nodes.
    * (macro TYPE_ARG_TYPES)
    */
   tree_nodeRef prms;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(function_type)

   /// Redefinition of get_kind.
   GET_KIND(function_type)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(type_node)=0, GETID(retn), GETID(prms)};
};


/**
 * This struct specifies the ge_expr node.
 * Relational operator. GE_EXPR is allowed only for integer (or pointer or enumeral) or real types.
 * The operands will have the same type, and the value is always the type used by the language for booleans.
*/
CREATE_TREE_NODE_CLASS(ge_expr,binary_expr);

/**
 * This struct specifies the gimple_assign node (gcc 4.3 tree node).
 * Assignment expression. The first operand is the what to set; the second, the new value.
 */
struct gimple_assign :  public expr_node_vops
{
   /// constructor
   gimple_assign(unsigned int i) : expr_node_vops(i), init_assignment(0) {}

   /// The first operand of the binary expression
   tree_nodeRef op0;

   /// The second operand of the binary expression
   tree_nodeRef op1;

   /// in case the statement comes from a phi node split points to the original gimple_phi (PandA extension)
   tree_nodeRef orig;

   bool init_assignment;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(gimple_assign)

   /// Redefinition of get_kind.
   GET_KIND(gimple_assign)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(expr_node_vops)=0, GETID(op0), GETID(op1), GETID(orig)};
};

struct gimple_nop :  public expr_node_vops
{
   /// constructor
   gimple_nop(unsigned int i) : expr_node_vops(i) {}

    /// Redefinition of get_kind_text.
   GET_KIND_TEXT(gimple_nop)

   /// Redefinition of get_kind.
   GET_KIND(gimple_nop)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(expr_node_vops)=0};
};

/**
 * This struct specifies the gimple_goto node.
 * The operand is a LABEL_DECL node or an expression.
 * The type should be void and the value should be ignored. 
*/
struct gimple_goto : public expr_node_vops
{
   /// constructor
   gimple_goto(unsigned int i) : expr_node_vops(i) {}

   /// the label
   tree_nodeRef op;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(gimple_goto)

   /// Redefinition of get_kind.
   GET_KIND(gimple_goto)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(expr_node_vops)=0, GETID(op)};
};


/**
 * This struct specifies the goto_subroutine node.
 * Used internally for cleanups in the implementation of TRY_FINALLY_EXPR.
 * (Specifically, it is created by expand_expr, not front-ends.)
 * Operand 0 is the rtx for the start of the subroutine we need to call.
 * Operand 1 is the rtx for a variable in which to store the address of where the subroutine should return to. 
*/
CREATE_TREE_NODE_CLASS(goto_subroutine,binary_expr);

/**
 * This struct specifies the gt_expr node.
 * Relational operator. GT_EXPR is allowed only for integer (or pointer or enumeral) or real types.
 * The operands will have the same type, and the value is always the type used by the language for booleans.
*/
CREATE_TREE_NODE_CLASS(gt_expr,binary_expr);

/**
 * A HANDLER wraps a catch handler for the HANDLER_TYPE.  If this is
 * CATCH_ALL_TYPE, then the handler catches all types.  The declaration of
 * the catch variable is in HANDLER_PARMS, and the body block in
 *  HANDLER_BODY.
 */
struct handler : public tree_node
{
   /// constructor
   handler(unsigned int i) : tree_node(i), line(-1) {}

   /// line is the line number where the compound_stmt is defined.
   int line;

   /// is the body of the handler
   tree_nodeRef body;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(handler)

   /// Redefinition of get_kind.
   GET_KIND(handler)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(tree_node)=0, GETID(body)};
};


/**
 * struct definition of the function_decl tree node.
 * The tree walker structure of this node is:
 * #(TOK_IDENTIFIER_NODE ((strg lngt) | TOK_OPERATOR));
*/
struct identifier_node : public tree_node
{
   /// constructors
   identifier_node(unsigned int node_id, std::string _strg, tree_manager* TM);
   identifier_node(unsigned int node_id, bool _operator_flag, tree_manager* TM);

   /// Store true if the identifier_node is an operator.
   const bool operator_flag;

   /// Store the identifier string associated with the identifier_node.
   std::string strg;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(identifier_node)

   /// Redefinition of get_kind.
   GET_KIND(identifier_node)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
   */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(tree_node)=0};
};

/**
 * This struct specifies the imagpart_expr node.
 * Used only on an operand of complex type, this return a value of the corresponding component type.  
*/
CREATE_TREE_NODE_CLASS(imagpart_expr,unary_expr);

/**
 * This struct specifies the indirect_ref node.
 * C unary `*' or Pascal `^'.  One operand, an expression for a pointer.
*/
CREATE_TREE_NODE_CLASS(indirect_ref,unary_expr);

/**
 * This struct specifies the in_expr node.
 * Operation for Pascal sets.
*/
CREATE_TREE_NODE_CLASS(in_expr,binary_expr);

/**
 * This struct specifies the init_expr node.
 * Initialization expression.  Operand 0 is the variable to initialize; Operand 1 is the initializer. 
*/
CREATE_TREE_NODE_CLASS(init_expr,binary_expr);

/**
 * This struct specifies the integer_cst node.
*/
struct integer_cst : public cst_node
{
   /// constructor
   integer_cst(unsigned int i): cst_node(i), value(0) {}

   /// type field is the type of the node
   tree_nodeRef type;

   ///The value of the integer cast
   long long int value;

   /**
    * has_high specifies if high is meaningful.
   */
   bool has_high;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(integer_cst)

   /// Redefinition of get_kind.
   GET_KIND(integer_cst)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(cst_node)=0, GETID(type)};
};


/**
 * struct definition of the integer_type tree node.
 * The tree walker structure of this node is:
 * #(TOK_INTEGER_TYPE type_node prec (TOK_STRING)? (TOK_UNSIGNED)? min max );
*/
struct integer_type : public type_node
{
   /// constructor
   integer_type(unsigned int i) : type_node(i), prec(0), unsigned_flag(false) {}

   /*type_node fields are in the parent class: type_node*/

   /**
    * prec field is the number of bits used by this type
    * (macro TYPE_PRECISION)
   */
   unsigned int prec;

   /// FIXME: add the meaning of this field
   std::string str;

   /**
    * unsigned means an unsigned type
    * (macro TYPE_UNSIGNED)
   */
   bool unsigned_flag;

   /// min: tree-dump.c use the macro TYPE_MIN_VALUE
   tree_nodeRef min;

   /// max: tree-dump.c use the macro TYPE_MAX_VALUE
   tree_nodeRef max;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(integer_type)

   /// Redefinition of get_kind.
   GET_KIND(integer_type)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(type_node)=0, GETID(min), GETID(max)};
};


/**
 * struct definition of the label_decl tree node.
 * The tree walker structure of this node is:
 * #(TOK_LABEL_DECL decl_node (uid)?)
*/
CREATE_TREE_NODE_CLASS(label_decl,decl_node);

/**
 * This struct specifies the labeled_block_expr node.
 * A labeled block. Operand 0 is the label that will be generated to mark the end of the block.
 * Operand 1 is the labeled block body. 
 * The tree walker structure of this node is:
 * #(TOK_LABELED_BLOCK type (body | (lab body)))
*/
struct labeled_block_expr : public expr_node
{
   /// constructor
   labeled_block_expr(unsigned int i) : expr_node(i) {}

   /// body is the operand 1 of this node: this is the label that will be generated to mark the end of the block.(macro LABELED_BLOCK_BODY)
   tree_nodeRef body;

   /// lab is the operand 0 of this node: this is the labeled block body.(macro LABELED_BLOCK_LABEL)
   tree_nodeRef lab;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(labeled_block_expr)

   /// Redefinition of get_kind.
   GET_KIND(labeled_block_expr)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(expr_node)=0, GETID(body), GETID(lab)};
};


/**
 * This struct specifies the gimple_label node.
 * A label definition, encapsulated as a statement.
 * The operand is the LABEL_DECL node for the label that appears here.
 * The type should be void and the value should be ignored. 
*/
struct gimple_label : public expr_node_vops
{
   /// constructor
   gimple_label(unsigned int i) : expr_node_vops(i) {}

   /// the label
   tree_nodeRef op;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(gimple_label)

   /// Redefinition of get_kind.
   GET_KIND(gimple_label)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(expr_node_vops)=0, GETID(op)};
};

/**
 * struct definition of the lang_type tree node.
 * The tree walker structure of this node is:
 * #(TOK_LANG_TYPE type_node);
*/
CREATE_TREE_NODE_CLASS(lang_type,type_node);

/**
 * This struct specifies the le_expr node.
 * Relational operator.LE_EXPR is allowed only for integer (or pointer or enumeral) or real types.
 * The operands will have the same type, and the value is always the type used by the language for booleans. 
*/
CREATE_TREE_NODE_CLASS(le_expr,binary_expr);

/**
 * A loop. The operand is the body of the loop.
 * It must contain an EXIT_EXPR or is an infinite loop.
 * The type should be void and the value should be ignored.
*/
CREATE_TREE_NODE_CLASS(loop_expr,unary_expr);

/**
 * This struct specifies the lrotate_expr node.
 * Shift operation for rotate.
 * Shift means logical shift if done on an unsigned type, arithmetic shift if done on a signed type.
 * The second operand is the number of bits to shift by; it need not be the same type as the first operand and result.
 * Note that the result is undefined if the second operand is larger than the first operand's type size. 
*/
CREATE_TREE_NODE_CLASS(lrotate_expr,binary_expr);

/**
 * This struct specifies the lshift_expr node.
 * Shift operation for shift.
 * Shift means logical shift if done on an unsigned type, arithmetic shift if done on a signed type.
 * The second operand is the number of bits to shift by; it need not be the same type as the first operand and result.
 * Note that the result is undefined if the second operand is larger than the first operand's type size. 
*/
CREATE_TREE_NODE_CLASS(lshift_expr,binary_expr);

/**
 * This struct specifies the lt_expr node.
 * Relational operator. LT_EXPR is allowed only for integer (or pointer or enumeral) or real types.
 * The operands will have the same type, and the value is always the type used by the language for booleans.
*/
CREATE_TREE_NODE_CLASS(lt_expr,binary_expr);

/**
 * This struct specifies the ltgt_expr node.
 * This is the reverse of uneq_expr.
 */
CREATE_TREE_NODE_CLASS(ltgt_expr,binary_expr);

/**
 * This struct specifies the max_expr node.
*/
CREATE_TREE_NODE_CLASS(max_expr,binary_expr);

/**
 * Memory addressing.  Operands are a pointer and a tree constant integer
 * byte offset of the pointer type that when dereferenced yields the
 * type of the base object the pointer points into and which is used for
 * TBAA purposes.
 * The type of the MEM_REF is the type the bytes at the memory location
 * are interpreted as.
 * MEM_REF <p, c> is equivalent to ((typeof(c))p)->x... where x... is a
 * chain of component references offsetting p by c.
 */
CREATE_TREE_NODE_CLASS(mem_ref,binary_expr);

/**
 * Memory Partition Memory Tag.
 */
CREATE_TREE_NODE_CLASS(memory_partition_tag,memory_tag);

/**
 * struct definition of the method_type tree node.
 * METHOD_TYPE is the type of a function which takes an extra first
 * argument for "self", which is not present in the declared argument list.
 * The TREE_TYPE is the return type of the method.  The TYPE_METHOD_BASETYPE
 * is the type of "self".  TYPE_ARG_TYPES is the real argument list, which
 * includes the hidden argument for "self".
 * The tree walker structure of this node is:
 * #(TOK_METHOD_TYPE type_node (clas)? (retn)? (prms)? )
*/
struct method_type : public function_type
{
   /// constructor
   method_type(unsigned int i) : function_type(i) {}

   //type_node fields are in the parent class: type_node
   //return type and parameters are in the parent class function_type

   /**
    * clas field is the type of "self".
    * (macro TYPE_METHOD_BASETYPE)
   */
   tree_nodeRef clas;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(method_type)

   /// Redefinition of get_kind.
   GET_KIND(method_type)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(function_type)=0, GETID(clas)};
};


/**
 * This struct specifies the min_expr node.
*/
CREATE_TREE_NODE_CLASS(min_expr,binary_expr);

/**
 * This struct specifies the minus_expr node.
 * Simple arithmetic.
*/
CREATE_TREE_NODE_CLASS(minus_expr,binary_expr);

/**
 * This struct specifies the modify_expr node.
 * Assignment expression. The first operand is the what to set; the second, the new value.
*/
CREATE_TREE_NODE_CLASS(modify_expr,binary_expr);

/**
 * This struct represent one of the bunch of tree codes for the initial,
 * superficial parsing of templates
 */
CREATE_TREE_NODE_CLASS(modop_expr,expr_node);

/**
 * This struct specifies the mult_expr node.
 * Simple arithmetic. 
*/
CREATE_TREE_NODE_CLASS(mult_expr,binary_expr);

/**
 * Name Memory Tag.
 */
CREATE_TREE_NODE_CLASS(name_memory_tag,memory_tag);

/**
 * struct definition of the label_decl tree node.
 * The tree walker structure of this node is:
 * #(TOK_NAMESPACE_DECL decl_node (dcls)?)
*/
struct namespace_decl : public decl_node
{
   /// constructor
   namespace_decl(unsigned int i) : decl_node(i) {}

   /*decl_node fields are in the parent class*/

   /// declarations
   tree_nodeRef dcls;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(namespace_decl)

   /// Redefinition of get_kind.
   GET_KIND(namespace_decl)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(decl_node)=0, GETID(dcls)};
};


/**
 * This struct specifies the ne_expr node.
 * Relational operators. NE_EXPR is allowed for any types.
 * The operands will have the same type, and the value is always the type used by the language for booleans. 
*/
CREATE_TREE_NODE_CLASS(ne_expr,binary_expr);

/**
 * This struct specifies the negate_expr node.
 * Unary negation.
*/
CREATE_TREE_NODE_CLASS(negate_expr,unary_expr);

/**
* This struct represent a 'new' expression
*/
CREATE_TREE_NODE_CLASS(new_expr,expr_node);

/**
 * This struct specifies the non_lvalue_expr node.
 * Value is same as argument, but guaranteed not an lvalue.
*/
CREATE_TREE_NODE_CLASS(non_lvalue_expr,unary_expr);

/**
 * This struct specifies the nop_expr node.
 * Represents a conversion expected to require no code to be generated.
*/
CREATE_TREE_NODE_CLASS(nop_expr,unary_expr);

/**
 * Used to represent lookup of runtime type dependent data.  Often this is
 * a reference to a vtable, but it needn't be.  Operands are:
 * OBJ_TYPE_REF_EXPR: An expression that evaluates the value to use.
 * OBJ_TYPE_REF_OBJECT: Is the object on whose behalf the lookup is
 * being performed.  Through this the optimizers may be able to statically
 * determine the dynamic type of the object.
 * OBJ_TYPE_REF_TOKEN: Something front-end specific used to resolve the
 * reference to something simpler, usually to the address of a DECL.
 * Never touched by the middle-end.  Good choices would be either an
 * identifier or a vtable index.
 */
CREATE_TREE_NODE_CLASS(obj_type_ref,ternary_expr);

/**
 * struct definition of the offset_type tree node.
 * The tree walker structure of this node is:
 * #(TOK_OFFSET_TYPE type_node)
*/
CREATE_TREE_NODE_CLASS(offset_type,type_node);

/**
 * This struct specifies the ordered_expr node.
 * Additional relational operator for floating point unordered.
*/
CREATE_TREE_NODE_CLASS(ordered_expr,binary_expr);

/**
 * This struct represents a list-like node for chaining overloading candidates
 */
struct overload : public tree_node
{
   /// constructor
   overload(unsigned int i) : tree_node(i) {}

   /// Is the current function declaration
   tree_nodeRef crnt;

   /// Is the chain
   tree_nodeRef chan;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(overload)

   /// Redefinition of get_kind.
   GET_KIND(overload)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum { GETID(tree_node)=0, GETID(crnt),  GETID(chan)};
};


/**
 * struct definition of the parm_decl tree node.
 * The tree walker structure of this node is:
 * #(TOK_PARM_DECL  decl_node (argt)? (size)? algn used (TOK_REGISTER)?)
*/
struct parm_decl : public decl_node
{
   /// constructor
   parm_decl(unsigned int i) : decl_node(i), algn(0), used(0), register_flag(false) {}

   /*decl_node fields are in the parent class: decl_node*/

   /**
    * argt field is the type in which the argument is actually passed,
    * which may be different from its type within the funtion.
    * (macro DECL_ARG_TYPE)
   */
   tree_nodeRef argt;

   /**
    * size field holds the size of datum, in bits.
    * (macro DECL_SIZE)
   */
   tree_nodeRef size;

   /**
    * algn field holds the alignment required for the datum, in bits.
    * (macro DECL_ALIGN)
   */
   unsigned int algn;

   /**
    * used is nonzero if the name is used in its scope
    * (macro TREE_USED)
   */
   int used;

   /**
    * register_flag means declared 'register'
    * (macro DECL_REGISTER)
   */
   bool register_flag;

   /**
    * symbol_memory_tag annotation
    */
   tree_nodeRef smt_ann;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(parm_decl)

   /// Redefinition of get_kind.
   GET_KIND(parm_decl)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(decl_node)=0, GETID(argt), GETID(size), GETID(smt_ann)};
};


/**
 * This struct specifies the gimple_phi node.
 * SSA PHI operator.  PHI_RESULT is the new SSA_NAME node created by the PHI node. PHI_ARG_LENGTH is the number of arguments.
 * PHI_ARG_ELT returns the Ith tuple <def, edge> from the argument list.  Each tuple contains the incoming reaching
 * definition (SSA_NAME node) and the edge via which that definition is coming through.
 * The tree walker structure of this node is:
 * #(TOK_GIMPLE_PHI res (tuple)*)
*/
struct gimple_phi : public tree_node
{
   /// constructor
   gimple_phi(unsigned int i) : tree_node(i), virtual_flag(false){}

   /// res is the new SSA_NAME node created by the PHI node.
   tree_nodeRef res;

   /// flag for virtual phi
   bool virtual_flag;

   /** store the list pairs: <def, edge>. Each tuple contains the incoming reaching
    * definition (SSA_NAME node) and the edge via which that definition is coming through.
   */
   std::vector<std::pair< tree_nodeRef, unsigned int> > list_of_def_edge;

   /**
    * Add a pair <def, edge> to the list of idx_val.
    * @param idx is the index.
    * @param val is the value.
    */
   void add_def_edge(const tree_nodeRef def, unsigned int edge);

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(gimple_phi)

   /// Redefinition of get_kind.
   GET_KIND(gimple_phi)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(tree_node)=0, GETID(res), GETID(list_of_def_edge)};
};


/**
 * This struct specifies the placeholder_expr node.
 * Such a "self-references" is done using a PLACEHOLDER_EXPR.  This is a node that will later be replaced with the object being referenced.
 * Its type is that of the object and selects which object to use from a chain of references (see below).  No other slots are used in the PLACEHOLDER_EXPR.
 * This node denotes a record to later be substituted before evaluating this expression.
 * The type of this expression is used to find the record to replace it.
*/
CREATE_TREE_NODE_CLASS(placeholder_expr,expr_node);

/**
 * This struct specifies the plus_expr node.
 * Simple arithmetic. 
*/
CREATE_TREE_NODE_CLASS(plus_expr,binary_expr);

/**
 * Pointer addition.  The first operand is always a pointer and the
 * second operand is an integer of type sizetype.
 */
CREATE_TREE_NODE_CLASS(pointer_plus_expr,binary_expr);

/**
 * struct definition of the pointer_type tree node.
 * The tree walker structure of this node is:
 * #(TOK_POINTER_TYPE type_node ptd)
*/
struct pointer_type : public type_node
{
   /// constructor
   pointer_type(unsigned int i) : type_node(i) {}

   /*type_node fields are in the parent class*/

   /**
    * ptd field points to the node for the type pointed to.
    * (macro TREE_TYPE)
   */
   tree_nodeRef ptd;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(pointer_type)

   /// Redefinition of get_kind.
   GET_KIND(pointer_type)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(type_node)=0, GETID(ptd)};
};


/**
 * This struct specifies the postdecrement_expr node.
 * struct for -- in C. The second arg is how much to increment by.
 * For a pointer, it would be the size of the object pointed to.
*/
CREATE_TREE_NODE_CLASS(postdecrement_expr,binary_expr);

/**
 * This struct specifies the postincrement_expr node.
 * struct for ++ in C. The second arg is how much to increment by.
 * For a pointer, it would be the size of the object pointed to.
*/
CREATE_TREE_NODE_CLASS(postincrement_expr,binary_expr);

/**
 * This struct specifies the predecrement_expr node.
 * struct for -- in C. The second arg is how much to increment by.
 * For a pointer, it would be the size of the object pointed to.
*/
CREATE_TREE_NODE_CLASS(predecrement_expr,binary_expr);

/**
 * This struct specifies the preincrement_expr node.
 * struct for ++ in C. The second arg is how much to increment by.
 * For a pointer, it would be the size of the object pointed to.
*/
CREATE_TREE_NODE_CLASS(preincrement_expr,binary_expr);

/**
 * struct definition of the qual_union_type tree node.
 * The tree walker structure of this node is:
 * #(TOK_QUAL_UNION_TYPE type_node);
*/
CREATE_TREE_NODE_CLASS(qual_union_type,type_node);

/**
 * This struct specifies the range_expr node.
 * Operation for Pascal sets. 
*/
CREATE_TREE_NODE_CLASS(range_expr,binary_expr);

/**
 * This struct specifies the rdiv_expr node.
 * Division for real result.
*/
CREATE_TREE_NODE_CLASS(rdiv_expr,binary_expr);

/**
 * This struct specifies the real_cst node.
 * Contents are in TREE_REAL_CST field.
 * The tree walker structure of this node is:
 * #(TOK_REAL_CST type (TOK_OVERFLOW)? valr valx)
*/
struct real_cst : public cst_node
{
   /// constructor
   real_cst(unsigned int i) : cst_node(i), overflow_flag(false) {}

   /// type field is the type of the node
   tree_nodeRef type;

   /**
    * overflow_flag  means there was an overflow in folding, and no warning has been issued for this subexpression.  
    * TREE_OVERFLOW implies TREE_CONSTANT_OVERFLOW, but not vice versa. 
   */
   bool overflow_flag;

   /// valr is the real value
   std::string valr;

   /// valx field
   std::string valx;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(real_cst)

   /// Redefinition of get_kind.
   GET_KIND(real_cst)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(cst_node)=0, GETID(type)};
};


/**
 * This struct specifies the realpart_expr node.
 * Used only on an operand of complex type, this return a value of the corresponding component type. 
*/
CREATE_TREE_NODE_CLASS(realpart_expr,unary_expr);

/**
 * struct definition of the real_type tree node.
 * The tree walker structure of this node is:
 * #(TOK_REAL_TYPE type_node prec);
*/
struct real_type : public type_node
{
   /// constructor
   real_type(unsigned int i) : type_node(i), prec(0) {}

   /*type_node fields are in the parent class*/

   /**
    * prec field is the number of bits used by this type.
    * (macro TYPE_PRECISION)
   */
   unsigned int prec;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(real_type)

   /// Redefinition of get_kind.
   GET_KIND(real_type)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(type_node)=0};
};


/**
 * struct definition of the record_type tree node.
 * The tree walker structure of this node is:
 * #(TOK_RECORD_TYPE type_node (vfld)? (TOK_SPEC)? TOK_STRUCT (flds)? (fncs)? (binf)?)
*/
struct record_type :  public type_node
{
   /// constructor
   record_type(unsigned int i): type_node(i), spec_flag(false) , ptrmem_flag(false), struct_flag(false) {}

   /**
    * list_of_flds is a chain of field_decl for the fields of the struct,
    * and var_decl, type_decl and const_decl for record-scope variables, types and enumerators.
    * (macro  TYPE_FIELDS)
   */
   std::vector<tree_nodeRef> list_of_flds;

   /**
    * list_of_fncs is a chain of methods_decl for the fields of the struct.
    * (macro TYPE_METHODS)
   */
   std::vector<tree_nodeRef> list_of_fncs;

   /// If pointer mem point to type
   tree_nodeRef ptd;

   /// If pointer mem struct type
   tree_nodeRef cls;

   /**
    * binf field are informations about this type, as a base type for itself.
    * It is a binfo node.
    * (macro TYPE_BINFO)
   */
   tree_nodeRef bfld;

   /// type context
   tree_nodeRef binf;

   /// true when a spec is considered
   bool spec_flag;

   /// true when ptrmem obj is considered
   bool ptrmem_flag;

   /// true when a struct is considered
   bool struct_flag;

   /// FIXME: check the meaning
   tree_nodeRef vfld;

   /** tmpl_parms holds template parameters
    * It is a TREE_LIST, his VALU field is a TREE_VEC whose LIST_OF_OP holds template parameters. 
    * The instantion of parameter "list_of_op[i]" is "list_of_op[i]" hold in tmpl_args. 
   */
   tree_nodeRef tmpl_parms;

   /** tmpl_args holds template instantiations
    * It is a TREE_VEC whose LIST_OF_OP holds template instantiations.
    * The parameter of  instantiation "list_of_op[i]" is "list_of_op[i]" hold in tmpl_parms. 
   */
   tree_nodeRef tmpl_args;

   /**
    * Add a field_decl to list of flds.
    * @param a is a NODE_ID.
   */
   void add_flds(const tree_nodeRef a)
   {
      list_of_flds.push_back(a);
   }

   /**
    * Add a methods_decl to list of fncs.
    * @param a is a NODE_ID.
   */
   void add_fncs(const tree_nodeRef a)
   {
      list_of_fncs.push_back(a);
   }

   /**
    * returns tree_nodeRef of the field specified by offset
    * @param offset is the offset of the field from the base address of the record_type
    * @return the tree_nodeRef if the offset is valid else null pointer
   */
   tree_nodeRef get_field(long long int offset);

   /**
    * returns the name of the struct represented by this node if there is one else
    * returns the string \#UNKNOWN\#
    */
   std::string get_maybe_name() const;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(record_type)

   /// Redefinition of get_kind.
   GET_KIND(record_type)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(type_node)=0, GETID(vfld), GETID(list_of_flds),GETID(list_of_fncs),GETID(ptd),GETID(cls),GETID(bfld),GETID(binf),GETID(tmpl_parms),GETID(tmpl_args)};
};


/**
 * This struct specifies the reference_expr node.
 * Non-lvalue reference or pointer to an object.
*/
CREATE_TREE_NODE_CLASS(reference_expr,unary_expr);

/**
 * struct definition of the reference_type tree node.
 * The tree walker structure of this node is:
 * #(TOK_REFERENCE_TYPE type_node refd)
*/
struct reference_type : public type_node
{
   /// constructor
   reference_type(unsigned int i) : type_node(i) {}

   /*type_node fields are in the parent class*/

   /**
    * refd field references to the node for the type referenced to.
    * (macro TREE_TYPE)
   */
   tree_nodeRef refd;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(reference_type)

   /// Redefinition of get_kind.
   GET_KIND(reference_type)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(type_node)=0, GETID(refd)};
};


/**
 * This struct represent one of tree codes for the initial,
 * superficial parsing of templates
 */
CREATE_TREE_NODE_CLASS(reinterpret_cast_expr,unary_expr);

/**
 * struct definition of the result_decl tree node.
 * The tree walker structure of this node is:
 * #(TOK_RESULT_DECL decl_node (init)? size algn)
*/
struct result_decl : public decl_node
{
   /// constructor
   result_decl(unsigned int i): decl_node(i), algn(0) {}

   /*decl_node fields are in the parent class*/

   /**
    * init field holds the value to initialize a variable to.
    * (macro DECL_INITIAL)
   */
   tree_nodeRef init;

   /**
    * size field holds the size of datum, in bits.
    * (macro DECL_SIZE)
   */
   tree_nodeRef size;

   /**
    * algn field holds the alignment required for the datum, in bits.
    * (macro DECL_ALIGN)
   */
   unsigned int algn;

   /**
    * symbol_memory_tag annotation
    */
   tree_nodeRef smt_ann;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(result_decl)

   /// Redefinition of get_kind.
   GET_KIND(result_decl)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(decl_node)=0, GETID(init), GETID(size),GETID(smt_ann)};
};


/**
 * This struct specifies the gimple_resx node.
 * Resume execution after an exception.  The perand is a 
 * number indicating the exception region that is being left. 
*/
CREATE_TREE_NODE_CLASS(gimple_resx,unary_expr);

/**
 * This struct specifies the gimple_return node.
 * Evaluates the operand, then returns from the current function.
 * Presumably that operand is an assignment that stores into the RESULT_DECL that hold the value to be returned.
 * The operand may be null. The type should be void and the value should be ignored. 
 * The tree walker structure of this node is:
 * #(TOK_GIMPLE_RETURN type (op)?)
*/
struct gimple_return : public expr_node_vops
{
   /// constructor
   gimple_return(unsigned int i) : expr_node_vops(i) {}

   /// op field is the operand of this node
   tree_nodeRef op;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(gimple_return)

  /// Redefinition of get_kind.
   GET_KIND(gimple_return)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(expr_node_vops)=0, GETID(op)};
};


/**
 * This struct represent a 'return' statement
 */
struct return_stmt : public tree_node
{
   /// constructor
   return_stmt(unsigned int i): tree_node(i), line (-1) {}

   /// line is the line number where the compound_stmt is defined.
   int line;

   /// Is the expression of the statement
   tree_nodeRef expr;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(return_stmt)

   /// Redefinition of get_kind.
   GET_KIND(return_stmt)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(tree_node)=0, GETID(expr)};
};


/**
 * This struct specifies the round_div_expr node.
 * Division for integer result that rounds toward nearest integer.
*/
CREATE_TREE_NODE_CLASS(round_div_expr,binary_expr);

/**
 * This struct specifies the round_mod_expr node.
 * Kind of remainder that go with the kind of division. 
*/
CREATE_TREE_NODE_CLASS(round_mod_expr,binary_expr);

/**
 * This struct specifies the rrotate_expr node.
 * Shift operation for rotate.
 * Shift means logical shift if done on an unsigned type, arithmetic shift if done on a signed type.
 * The second operand is the number of bits to shift by; it need not be the same type as the first operand and result.
 * Note that the result is undefined if the second operand is larger than the first operand's type size. 
*/
CREATE_TREE_NODE_CLASS(rrotate_expr,binary_expr);

/**
 * This struct specifies the rshift_expr node.
 * Shift operation for shift.
 * Shift means logical shift if done on an unsigned type, arithmetic shift if done on a signed type.
 * The second operand is the number of bits to shift by; it need not be the same type as the first operand and result.
 * Note that the result is undefined if the second operand is larger than the first operand's type size. 
*/
CREATE_TREE_NODE_CLASS(rshift_expr,binary_expr);

/**
 * This struct specifies the save_expr node.
 * Represents something we computed once and will use multiple times.
 * First operand is that expression.  Second is the function decl in which the SAVE_EXPR was created.  
 * The third operand is the RTL, nonzero only after the expression has been computed.
*/
CREATE_TREE_NODE_CLASS(save_expr,ternary_expr);

/**
 * This struct specifies reference to particular overloaded struct method
 * The tree walker structure of this node is:
 * #(TOK_SCOPE_REF)
*/
struct scope_ref : public expr_node
{
   /// constructor
   scope_ref(unsigned int i) : expr_node(i) {}

   /// The first operand of the binary expression
   tree_nodeRef op0;

   /// The second operand of the binary expression
   tree_nodeRef op1;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(scope_ref)

   /// Redefinition of get_kind.
   GET_KIND(scope_ref)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(expr_node)=0, GETID(op0), GETID(op1)};
};


/**
 * This struct specifies the set_le_expr node.
 * Operation for Pascal sets.
*/
CREATE_TREE_NODE_CLASS(set_le_expr,binary_expr);

/**
 * struct definition of the set_type tree node.
 * The tree walker structure of this node is:
 * #(TOK_SET_TYPE type_node)
*/
CREATE_TREE_NODE_CLASS(set_type,type_node);

/**
 * This struct specifies the sizeof_expr node.
 * Represents the size value of the operand.
*/
CREATE_TREE_NODE_CLASS(sizeof_expr,unary_expr);

/**
 * This struct specifies the ssa_name node.
 * Variable references for SSA analysis.  New SSA names are created every time a variable is assigned a new value.  
 * The SSA builder uses SSA_NAME nodes to implement SSA versioning.  
 * The tree walker structure of this node is:
 * #(TOK_SSA_NAME var vers)
*/
struct ssa_name : public tree_node
{
   /// constructor
   ssa_name(unsigned int i) :
      tree_node(i),
      vers(0),
      volatile_flag(false),
      virtual_flag(false),
      default_flag(false),
      uses(0)
   {}

   /// var is the variable being referenced (macro SSA_NAME_VAR).
   tree_nodeRef var;

   /// vers is the SSA version number of this SSA name.(macro SSA_NAME_VERSION). Note that in tree SSA, version numbers are not per variable and may be recycled
   unsigned int vers;

   /// in alias analysis represent the may alias?
   tree_nodeRef ptr_info;

   /// in case a ssa_name is never defined this boolean member is true
   bool volatile_flag;

   /// flag for virtual phi
   bool virtual_flag;

   /**
    * Nonzero if this SSA_NAME is the default definition for the
    * underlying symbol.  A default SSA name is created for symbol S if
    * the very first reference to S in the function is a read operation.
    * Default definitions are always created by an empty statement and
    * belong to no basic block.
    * */
   bool default_flag;

   /// Number of uses of this ssa_name
   size_t uses;

   /// Map containing, as indexes, the references to statements in which this ssa_name is used, and as indexed objects vectors of the nodes which diretctly use the ssa_name in the indexing statement (e.g. a binary expression node contained in a statement)
   std::tr1::unordered_map<tree_nodeRef, std::vector<tree_nodeRef> > use_nodes;
 
   /// Vector containing the references to statements in which this ssa_name is used
   std::vector<tree_nodeRef> use_stmts;

   /// in case ssa_name is not volatile the statement which defines it; statements could be more than one because of gimple_phi splitting
   std::vector<tree_nodeRef> def_stmts;

   /**
    * Add a def stmt.
    * @param a is a def statement.
    */
   void add_def(const tree_nodeRef a)
   {
      def_stmts.push_back(a);
   }

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(ssa_name)

   /// Redefinition of get_kind.
   GET_KIND(ssa_name)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(tree_node)=0, GETID(var), GETID(ptr_info), GETID(def_stmts)};
};


/**
 * This struct specifies the statement_list node.
 * Used to chain children of container statements together. 
 * The tree walker structure of this node is:
 * #(TOK_STATEMENT_LIST( (stmt)+ | (bloc)+ )?)
*/
struct statement_list : public tree_node
{
   /// constructor
   statement_list(unsigned int i) : tree_node(i) {}

   /// list_of_stmt field is the list of statements. If this field is null then the list_of_bloc field is not null.
   std::vector<tree_nodeRef> list_of_stmt;

   /// list_of_bloc field is the list of basic block. If this field is null then the list_of_stmt field is not null.
   std::map<unsigned int, blocRef> list_of_bloc;

   /**
    * Add a value to list of basic block.
    * @param a is a NODE_ID.
   */
   void add_bloc(const blocRef a);

   /**
    * Add a value to list of stmt.
    * @param a is a NODE_ID.
   */
   void add_stmt(const tree_nodeRef a)
   {
      list_of_stmt.push_back(a);
   }

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(statement_list)

   /// Redefinition of get_kind.
   GET_KIND(statement_list)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(tree_node)=0, GETID(list_of_stmt), GETID(list_of_bloc)};
};


/**
 * This struct specifies the static_cast_expr node.
 * The walker for this struct is
 *   #(TOK_STATIC_CAST_EXPR wunary_expr);
*/
CREATE_TREE_NODE_CLASS(static_cast_expr,unary_expr);

/**
 * This struct specifies the string_cst node.
 * Contents are TREE_STRING_LENGTH and TREE_STRING_POINTER fields.
 * The tree walker structure of this node is:
 * #(TOK_STRING_CST type strg lngt)
*/
struct string_cst : public cst_node
{
   /// constructor
   string_cst(unsigned int i): cst_node(i), lngt(-1) {}

   /// type field is the type of the node
   tree_nodeRef type;

   /// strg is the TREE_STRING_POINTER.
   std::string strg;

   /// lngt is the TREE_STRING_LENGTH.
   int lngt;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(string_cst)

   /// Redefinition of get_kind.
   GET_KIND(string_cst)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(cst_node)=0, GETID(type)};
};


/**
 * Struct Field Memory Tag.
*/
CREATE_TREE_NODE_CLASS(struct_field_tag,memory_tag);

/**
 * Symbol Memory Tag.
 */
CREATE_TREE_NODE_CLASS(symbol_memory_tag,memory_tag);

/**
 * This struct specifies the gimple_switch node.
 * Switch expression.
 * TREE_TYPE is the original type of the condition, before any language required type conversions.  
 * It may be NULL, in which case the original type and final types are assumed to be the same.
 * Operand 0 is the expression used to perform the branch,
 * Operand 1 is the body of the switch, which probably contains CASE_LABEL_EXPRs.  It may also be NULL, in which case operand 2 must not be NULL.
 * Operand 2 is either NULL_TREE or a TREE_VEC of the CASE_LABEL_EXPRs of all the cases. (NO MORE USED)
 * The tree walker structure of this node is:
 * #(TOK_GIMPLE_SWITCH ternary_expr);
*/
struct gimple_switch : public expr_node_vops
{
   /// constructor
   gimple_switch(unsigned int i) : expr_node_vops(i) {}

   /// the branch var
   tree_nodeRef op0;

   /// the vec of CASE_LABEL_EXPRs
   tree_nodeRef op1;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(gimple_switch)

   /// Redefinition of get_kind.
   GET_KIND(gimple_switch)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(expr_node_vops)=0, GETID(op0), GETID(op1)};
};


/**
 * This struct implements the target_expr node.
 * For TARGET_EXPR, operand 0 is the target of an initialization; operand 1 is the initializer for the target;
 * and operand 2 is the cleanup for this node, if any; operand 3 is the saved initializer after this node has been expanded once, this is so we can re-expand the tree later. 
 * The tree walker structure of this node is:
 * #(TOK_TARGET_EXPR type decl (((init) (clnp)?) | ((clnp) (init)?))?)
*/
struct target_expr : public expr_node
{
   /// constructor
   target_expr(unsigned int i) : expr_node(i) {}

   /// decl is the operand 0 of the target_expr: this is the target of an initialization.
   tree_nodeRef decl;

   /// init1 is the operand 1 of the target_expr: this is the initializer for the target.
   tree_nodeRef init1;

   /// clnp is the operand 2 of the target_expr: this is the cleanup for this node, if any.
   tree_nodeRef clnp;

   /// init3 is the operand 3 of the target_expr: this is the saved initializer after this node has been expanded once.
   tree_nodeRef init3;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(target_expr)

   /// Redefinition of get_kind.
   GET_KIND(target_expr)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(expr_node)=0, GETID(decl), GETID(init1), GETID(clnp), GETID(init3)};
};


/** 
 * Low-level memory addressing.  Operands are SYMBOL (static or global
 * variable), BASE (register), INDEX (register), STEP (integer constant),
 * OFFSET (integer constant).  Corresponding address is
 * SYMBOL + BASE + STEP * INDEX + OFFSET.  Only variations and values valid on
 * the target are allowed.
 * 
 * The type of STEP, INDEX and OFFSET is sizetype.  The type of BASE is
 * sizetype or a pointer type (if SYMBOL is NULL).
 * 
 * The sixth argument is the reference to the original memory access, which
 * is preserved for the purposes of the RTL alias analysis.  The seventh
 * argument is a tag representing results of the tree level alias analysis.
 */
struct target_mem_ref : public WeightedNode
{
   /// constructor
   target_mem_ref(unsigned int i) : WeightedNode(i) {}

   /// type of the expression
   tree_nodeRef type;

   /// static or global variable
   tree_nodeRef symbol;

   /// BASE register
   tree_nodeRef base;

   /// INDEX register
   tree_nodeRef idx;

   /// STEP integer constant
   tree_nodeRef step;

   /// OFFSET integer constant
   tree_nodeRef offset;

   /// original memory access
   tree_nodeRef orig;

   /// result of the tree level alias analysis
   tree_nodeRef tag;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(target_mem_ref)

   /// Redefinition of get_kind.
   GET_KIND(target_mem_ref)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(WeightedNode)=0, GETID(type), GETID(symbol), GETID(base), GETID(idx), GETID(step), GETID(offset), GETID(orig), GETID(tag)};
};

/**
 * Low-level memory addressing.  Operands are BASE (address of static or
 * global variable or register), OFFSET (integer constant),
 * INDEX (register), STEP (integer constant), INDEX2 (register),
 * The corresponding address is BASE + STEP * INDEX + INDEX2 + OFFSET.
 * Only variations and values valid on the target are allowed.
 *
 * The type of STEP, INDEX and INDEX2 is sizetype.
 *
 * The type of BASE is a pointer type.  If BASE is not an address of
 * a static or global variable INDEX2 will be NULL.
 *
 * The type of OFFSET is a pointer type and determines TBAA the same as
 * the constant offset operand in MEM_REF.
 */
struct target_mem_ref461 : public WeightedNode
{
   /// constructor
   target_mem_ref461(unsigned int i) : WeightedNode(i) {}

   /// type of the expression
   tree_nodeRef type;

   /// BASE register
   tree_nodeRef base;

   /// OFFSET integer constant
   tree_nodeRef offset;

   /// INDEX register
   tree_nodeRef idx;

   /// STEP integer constant
   tree_nodeRef step;

   /// INDEX register
   tree_nodeRef idx2;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(target_mem_ref461)

   /// Redefinition of get_kind.
   GET_KIND(target_mem_ref461)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(WeightedNode)=0, GETID(type), GETID(base), GETID(idx), GETID(idx2), GETID(step), GETID(offset)};
};

/**
 * struct definition of the template_decl tree node.
 * The tree walker structure of this node is:
 * #(TOK_TEMPLATE_DECL decl_node (rslt)? (inst)? (spcs)? (prms)?)
 */
struct template_decl : public decl_node
{
   /// constructor
   template_decl(unsigned int i) : decl_node(i) {}

   /*decl_node fields are in the parent class: decl_node*/

   /// rslt is null for struct templates and declaration for object to be created for non-struct templates
   tree_nodeRef rslt;

   /// inst field holds the template instantiatin vector.
   tree_nodeRef inst;

   /// prms field holds the specialization parameters vector.
   tree_nodeRef spcs;

   /// prms field holds the template parameters vector.
   tree_nodeRef prms;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(template_decl)

   /// Redefinition of get_kind.
   GET_KIND(template_decl)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(decl_node)=0, GETID(rslt), GETID(inst), GETID(base), GETID(spcs), GETID(prms)};
};


/**
 * This struct represents a template identifier
*/
CREATE_TREE_NODE_CLASS(template_id_expr,expr_node);

/**
 * This struct represents a template parameter index
*/
CREATE_TREE_NODE_CLASS(template_parm_index,tree_node);

/**
 * struct definition of the template_type_parm tree node.
 * The tree walker structure of this node is:
 * #(TOK_TEMPLATE_TYPE_PARM type_node)
*/
CREATE_TREE_NODE_CLASS(template_type_parm,type_node);

/**
 * This struct represents a throw expression
*/
CREATE_TREE_NODE_CLASS(throw_expr,unary_expr);

/**
 * Represents a trait expression during template expansion.
 */
CREATE_TREE_NODE_CLASS(trait_expr,tree_node);

/**
 * struct definition of the translation_unit_decl tree node.
 * The tree walker structure of this node is: 
 * #(TOK_TRANSLATION_UNIT_DECL decl_node)
*/
CREATE_TREE_NODE_CLASS(translation_unit_decl,decl_node);

/**
 * This struct specifies the tree_list node.
 * This node has the TREE_VALUE and TREE_PURPOSE fields. These nodes are made into lists by chaining through the TREE_CHAIN field.
 * The elements of the list live in the TREE_VALUE fields, while TREE_PURPOSE fields are occasionally used as well to get the effect of Lisp association lists. 
 * The tree walker structure of this node is:
 * #(TOK_TREE_LIST  (purp)? valu (chan)?);
*/
struct tree_list : public WeightedNode
{
   /// constructor
   tree_list(unsigned int i) : WeightedNode(i) {}

   /// purp is the TREE_PURPOSE field occasionally used as well to get the effect of Lisp association lists.
   tree_nodeRef purp;

   /// purp is the TREE_VALUE field which stores the elements of the list.
   tree_nodeRef valu;

   /// purp is the TREE_CHAIN field: tree_list nodes are made into lists by chaining through the TREE_CHAIN field.
   tree_nodeRef chan;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(tree_list)

   /// Redefinition of get_kind.
   GET_KIND(tree_list)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(tree_node)=0, GETID(purp), GETID(valu), GETID(chan)};
};


/**
 * This struct specifies the tree_vec node.
 * These nodes contain an array of tree nodes.
 * The tree walker structure of this node is:
 * #(TOK_TREE_VEC lngt ((op)+)?)
*/
struct tree_vec : public tree_node
{
   /// constructor
   tree_vec(unsigned int i): tree_node(i), lngt(0) {}

   /// lngt is the lenght of the array (list_of_op) stored in tree_vec node.(macro TREE_VEC_LENGTH)
   size_t lngt;

   /// list_of_op is the array of tree node stored in tree_vec node.(macro TREE_VEC_ELT)
   std::vector<tree_nodeRef> list_of_op;

   /**
    * Add a value to list of operands.
    * @param a is a NODE_ID.
   */
   void add_op(const tree_nodeRef a)
   {
      list_of_op.push_back(a);
   }

   /// returns the number of operands
   size_t get_number_of_op()
   {
      return list_of_op.size();
   }

   /// return the n-th (from 0 to size() -1) operand of tree_vec
   tree_nodeRef get_op(size_t n)
   {
      return list_of_op[n];
   }

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(tree_vec)

   /// Redefinition of get_kind.
   GET_KIND(tree_vec)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(tree_node)=0, GETID(list_of_op)};
};


/**
 * This struct specifies the trunc_div_expr node.
 * Division for integer result that rounds the quotient toward zero.
*/
CREATE_TREE_NODE_CLASS(trunc_div_expr,binary_expr);

/**
 * This struct specifies the trunc_mod_expr node.
 * Kind of remainder that go with the kind of division.
*/
CREATE_TREE_NODE_CLASS(trunc_mod_expr,binary_expr);

/**
 * This struct specifies the truth_and_expr node.
 * AND always compute the second operand whether its value is needed or not (for side effects).
 * The operand may have BOOLEAN_TYPE or INTEGER_TYPE. The argument will be either zero or one.
*/
CREATE_TREE_NODE_CLASS(truth_and_expr,binary_expr);

/**
 * This struct specifies the truth_andif_expr node.
 * ANDIF allow the second operand not to be computed if the value of the expression is determined from the first operand.  
 * The operand may have BOOLEAN_TYPE or INTEGER_TYPE. The argument will be either zero or one.  
*/
CREATE_TREE_NODE_CLASS(truth_andif_expr,binary_expr);

/**
 * This struct specifies the truth_not_expr node.
 * The operand may have BOOLEAN_TYPE or INTEGER_TYPE.  In either case, the argument will be
 * either zero or one. TRUTH_NOT_EXPR will never have an INTEGER_TYPE VAR_DECL as its argument.
*/
CREATE_TREE_NODE_CLASS(truth_not_expr,unary_expr);

/**
 * This struct specifies the truth_or_expr node.
 * The operand may have BOOLEAN_TYPE or INTEGER_TYPE. the argument will be either zero or one.  
*/
CREATE_TREE_NODE_CLASS(truth_or_expr,binary_expr);

/**
 * This struct specifies the truth_orif_expr node.
 * ORIF allow the second operand not to be computed if the value of the expression is determined from the first operand.  
 * The operand may have BOOLEAN_TYPE or INTEGER_TYPE. The argument will be either zero or one.
*/
CREATE_TREE_NODE_CLASS(truth_orif_expr,binary_expr);

/**
 * This struct specifies the truth_xor_expr node.
 * XOR always compute the second operand whether its value is needed or not (for side effects).
 * The operand may have BOOLEAN_TYPE or INTEGER_TYPE. The argument will be either zero or one.
*/
CREATE_TREE_NODE_CLASS(truth_xor_expr,binary_expr);

/**
* This struct represents a try-block statement
*/
struct try_block : public tree_node
{
   /// constructor
   try_block(unsigned int i) : tree_node(i), line(-1) {}

   /// line is the line number where the compound_stmt is defined.
   int line;

   /// Is the body of the statement
   tree_nodeRef body;

   /// Is the handler of the statement
   tree_nodeRef hdlr;

   /// Is the next statement
   tree_nodeRef next;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(try_block)

   /// Redefinition of get_kind.
   GET_KIND(try_block)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(tree_node)=0, GETID(body), GETID(hdlr), GETID(next)};
};

/**
 * This struct specifies the try_catch_expr node.
 * Evaluate operand 1.  If and only if an exception is thrown during the evaluation of operand 1, evaluate operand 2.
 * This differs from TRY_FINALLY_EXPR in that operand 2 is not evaluated on a normal or jump exit, only on an exception.  
*/
CREATE_TREE_NODE_CLASS(try_catch_expr,binary_expr);

/**
 * This struct specifies the try_finally node.
 * Evaluate the first operand. 
 * The second operand is a cleanup expression which is evaluated on any exit (normal, exception, or jump out) from this expression. 
*/
CREATE_TREE_NODE_CLASS(try_finally,binary_expr);

/**
 * struct definition of the type_decl tree node.
 * The tree walker structure of this node is:
 * #(TOK_TYPE_DECL decl_node)
*/
struct type_decl : public decl_node
{
   /// constructor
   type_decl(unsigned int i) : decl_node(i) {}

   /** tmpl_parms holds template parameters
    * It is a TREE_LIST, his VALU field is a TREE_VEC whose LIST_OF_OP holds template parameters. 
    * The instantion of parameter "list_of_op[i]" is "list_of_op[i]" hold in tmpl_args. 
   */
   tree_nodeRef tmpl_parms;

   /** tmpl_args holds template instantiations
    * It is a TREE_VEC whose LIST_OF_OP holds template instantiations.
    * The parameter of  instantiation "list_of_op[i]" is "list_of_op[i]" hold in tmpl_parms. 
   */
   tree_nodeRef tmpl_args;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(type_decl)

   /// Redefinition of get_kind.
   GET_KIND(type_decl)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(decl_node)=0, GETID(tmpl_parms), GETID(tmpl_args)};
};


/**
 * struct definition of the typename_type tree node.
 * The tree walker structure of this node is:
 * #(TOK_TYPENAME_TYPE type_node)
*/
CREATE_TREE_NODE_CLASS(typename_type,type_node);

/**
 * This struct specifies the uneq_expr node.
 * This is equivalent to unordered or ... 
*/
CREATE_TREE_NODE_CLASS(uneq_expr,binary_expr);

/**
 * This struct specifies the unge_expr node.
 * This is equivalent to unordered or ...
*/
CREATE_TREE_NODE_CLASS(unge_expr,binary_expr);

/**
 * This struct specifies the ungt_expr node.
 * This is equivalent to unordered or ...
*/
CREATE_TREE_NODE_CLASS(ungt_expr,binary_expr);

/**
 * struct definition of the union_type tree node.
 * The tree walker structure of this node is:
 * #(TOK_UNION_TYPE type_node TOK_UNION (flds)? (fncs)? (binf)?);
*/
struct union_type : public type_node
{
   /// constructor
   union_type(unsigned int i) : type_node(i) {}

   /*type_node fields are in the parent class*/

   /**
    * list_of_flds is a chain of field_decl for the fields of the struct,
    * and var_decl, type_decl and const_decl for record-scope variables, types and enumerators.
    * (macro  TYPE_FIELDS)
   */
   std::vector<tree_nodeRef> list_of_flds;

   /**
    * list_of_fncs is a chain of methods_decl for the fields of the struct.
    * (macro TYPE_METHODS)
   */
   std::vector<tree_nodeRef> list_of_fncs;

   /**
    * binf field are informations about this type, as a base type for itself.
    * It is a binfo node.
    * (macro TYPE_BINFO)
   */
   tree_nodeRef binf;

   /**
    * Add a field_decl to list of flds.
    * @param a is a NODE_ID.
   */
   void add_flds(const tree_nodeRef a)
   {
      list_of_flds.push_back(a);
   }

   /**
    * Add a methods_decl to list of fncs.
    * @param a is a NODE_ID.
   */
   void add_fncs(const tree_nodeRef a)
   {
      list_of_fncs.push_back(a);
   }

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(union_type)

   /// Redefinition of get_kind.
   GET_KIND(union_type)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(type_node)=0, GETID(list_of_flds), GETID(list_of_fncs), GETID(binf)};
};


/**
 * This struct specifies the unle_expr node.
 * This is equivalent to unordered or ... 
*/
CREATE_TREE_NODE_CLASS(unle_expr,binary_expr);

/**
 * This struct specifies the unlt_expr node.
 * This is equivalent to unordered or ... 
*/
CREATE_TREE_NODE_CLASS(unlt_expr,binary_expr);

/**
 * This struct specifies the unordered_expr node.
 * Additional relational operator for floating point unordered.
*/
CREATE_TREE_NODE_CLASS(unordered_expr,binary_expr);

/**
 * This struct specifies the unsave_expr node.
 * The operand is the value to unsave.  By unsave, we
 * mean that all _EXPRs such as TARGET_EXPRs, SAVE_EXPRs,
 * CALL_EXPRs and RTL_EXPRs, that are protected
 * from being evaluated more than once should be reset so that a new
 * expand_expr call of this expr will cause those to be re-evaluated.
 * This is useful when we want to reuse a tree in different places,
 * but where we must re-expand. 
*/
CREATE_TREE_NODE_CLASS(unsave_expr,unary_expr);

/**
 * This struct specifies the va_arg_expr node.
 * Used to implement `va_arg'.
*/
CREATE_TREE_NODE_CLASS(va_arg_expr,unary_expr);

/**
 * struct definition of the label_decl tree node.
 * The tree walker structure of this node is:
 * #(TOK_VAR_DECL decl_node attr (TOK_STATIC TOK_STATIC | TOK_STATIC | TOK_EXTERN)? (init)? (size)? algn used (TOK_REGISTER)?)
*/
struct var_decl : public decl_node, public attr
{
   /// constructor
   var_decl(unsigned int i) : decl_node(i), use_tmpl(-1), static_static_flag(false), static_flag(false), extern_flag(false), algn(0), used(0), register_flag(false) {}

   /*decl_node fields are in the parent class: decl_node*/

   /** use_tmpl indicates whether or not (and how) a template was expanded for this VAR_DECL.
    * 0=normal declaration, e.g. int min (int, int);
    * 1=implicit template instantiation
    * 2=explicit template specialization, e.g. int min<int> (int, int);
    * 3=explicit template instantiation, e.g. template int min<int> (int, int);  
   */
   int use_tmpl;

   /// to manage C++ code with static member
   bool static_static_flag;

   /// to manage standard static attribute
   bool static_flag;

   /// a variable can be extern
   bool extern_flag;

   /*attr fields are in the parent class: attr*/

   /**
    * init field holds the value to initialize a variable to.
    * (macro DECL_INITIAL)
   */
   tree_nodeRef init;

   /**
    * size field holds the size of datum, in bits.
    * (macro DECL_SIZE)
   */
   tree_nodeRef size;

   /**
    * algn field holds the alignment required for the datum, in bits.
    * (macro DECL_ALIGN)
   */
   unsigned int algn;

   /**
    * used is nonzero if the name is used in its scope
    * (macro TREE_USED)
   */
   int used;

   /**
    * register_flag means declared 'register'
    * (macro DECL_REGISTER)
   */
   bool register_flag;

   /**
    * symbol_memory_tag annotation
    */
   tree_nodeRef smt_ann;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(var_decl)

   /// Redefinition of get_kind.
   GET_KIND(var_decl)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(decl_node)=0, GETID(attr), GETID(init), GETID(size), GETID(smt_ann)};
};


/**
 * This struct represents a vector new expression
 */
CREATE_TREE_NODE_CLASS(vec_new_expr,expr_node);

/**
 * This struct specifies the vector_cst node.
 * Contents are in TREE_VECTOR_CST_ELTS field.
 * The tree walker structure of this node is:
 * #(TOK_VECTOR_CST type (valu)+)
*/
struct vector_cst : public cst_node
{
   /// constructor
   vector_cst(unsigned int i) : cst_node(i) {}

   /// type field is the type of the node
   tree_nodeRef type;

   /// list_of_valu is a list of value of the TREE_VECTOR_CST_ELTS vector elements.
   std::vector<tree_nodeRef> list_of_valu;

   /**
    * Add a value to list of value.
    * @param a is a NODE_ID.
   */
   void add_valu(const tree_nodeRef a)
   {
      list_of_valu.push_back(a);
   }

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(vector_cst)

   /// Redefinition of get_kind.
   GET_KIND(vector_cst)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(cst_node)=0, GETID(type), GETID(list_of_valu)};
};


/**
 * struct definition of the vector_type tree node.
 * The tree walker structure of this node is:
 * #(TOK_VECTOR_TYPE type_node elts)
*/

struct vector_type : public type_node
{
   /// constructor
   vector_type(unsigned int i) : type_node(i) {}

   /// field elts is the type of an vector element (tree-dump.c use the macro TREE_TYPE)
   tree_nodeRef elts;

   /// Redefinition of get_kind_text.
   GET_KIND_TEXT(vector_type)

   /// Redefinition of get_kind.
   GET_KIND(vector_type)

   /**
    * virtual function used to traverse the tree_node data structure.
    * @param v is a reference to the tree_node visitor class
    */
   virtual void visit(tree_node_visitor * const v) const;

   /// visitor enum
   enum {GETID(type_node)=0, GETID(elts)};
};

/**
 * This struct specifies the view_convert_expr node.
 * Represents viewing something of one type as being of a second type. This corresponds to an "Unchecked Conversion" in Ada and roughly to
 * the idiom *(type2 *)&X in C.  The only operand is the value to be viewed as being of another type.  It is undefined if the type of the
 * input and of the expression have different sizes.
 *
 * This code may also be used within the LHS of a MODIFY_EXPR, in which case no actual data motion may occur.  TREE_ADDRESSABLE will be set in
 * this case and GCC must abort if it could not do the operation without generating insns. 
*/
CREATE_TREE_NODE_CLASS(view_convert_expr,unary_expr);

/**
 * struct definition of the void_type tree node.
 * The tree walker structure of this node is:
 * #(TOK_VOID_TYPE type_node);
*/
CREATE_TREE_NODE_CLASS(void_type,type_node);

/**
 * This struct specifies the vtable_ref node.
 * Vtable indexing.  Carries data useful for emitting information for vtable garbage collection.
 * Operand 0: an array_ref (or equivalent expression)
 * Operand 1: the vtable base (must be a var_decl)
 * Operand 2: index into vtable (must be an integer_cst). 
*/
CREATE_TREE_NODE_CLASS(vtable_ref,ternary_expr);

/**
 * This struct specifies the with_cleanup_expr node.
 * Specify a value to compute along with its corresponding cleanup.
 * Operand 0 argument is an expression whose value needs a cleanup.
 * Operand 1 is the cleanup expression for the object.
 * Operand 2 is an RTL_EXPR which will eventually represent that value. The RTL_EXPR is used in this expression, which is how the expression manages to act on the proper value.
 * The cleanup is executed by the first enclosing CLEANUP_POINT_EXPR, if it exists, otherwise it is the responsibility of the caller to manually
 * call expand_start_target_temps/expand_end_target_temps, as needed.
 * This differs from TRY_CATCH_EXPR in that operand 2 is always evaluated when an exception isn't thrown when cleanups are run.
*/
CREATE_TREE_NODE_CLASS(with_cleanup_expr,ternary_expr);

/**
 * Records the size for an expression of variable size type.  This is
 * for use in contexts in which we are accessing the entire object,
 * such as for a function call, or block copy.
 * Operand 0 is the real expression.
 * Operand 1 is the size of the type in the expression.
*/
CREATE_TREE_NODE_CLASS(with_size_expr,binary_expr);

/**
 * GIMPLE_PREDICT <PREDICT, OUTCOME> specifies a hint for branch prediction.
 * PREDICT is one of the predictors from predict.def.
 * OUTCOME is NOT_TAKEN or TAKEN.
 */
CREATE_TREE_NODE_CLASS(gimple_predict,unary_expr);

/**
 *  Widening summation.
 * The first argument is of type t1.
 * The second argument is of type t2, such that t2 is at least twice
 * the size of t1. The type of the entire expression is also t2.
 * WIDEN_SUM_EXPR is equivalent to first widening (promoting)
 * the first argument from type t1 to type t2, and then summing it
 * with the second argument.
 */
CREATE_TREE_NODE_CLASS(widen_sum_expr,binary_expr);

/**
 * Widening multiplication.
 * The two arguments are of type t1.
 * The result is of type t2, such that t2 is at least twice
 * the size of t1. WIDEN_MULT_EXPR is equivalent to first widening (promoting)
 * the arguments from type t1 to type t2, and then multiplying them.
 */
CREATE_TREE_NODE_CLASS(widen_mult_expr,binary_expr);

#endif
