/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file tree_nodes_merger.hpp
 * @brief tree node merger classess.
 *
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef TREE_NODES_MERGER_HPP
#define TREE_NODES_MERGER_HPP

#include <ostream>
#include <tr1/unordered_map>
#include <tr1/unordered_set>
#include <boost/preprocessor/seq/for_each.hpp>
#include <boost/preprocessor/facilities/empty.hpp>
#include "refcount.hpp"
#include "tree_node.hpp"
#include "tree_node_mask.hpp"

/**
 * @name forward declarations
 */
//@{
REF_FORWARD_DECL(tree_node_reached);
REF_FORWARD_DECL(tree_node);
REF_FORWARD_DECL(vops);
REF_FORWARD_DECL(vuse);
REF_FORWARD_DECL(vdef);
REF_FORWARD_DECL(bloc);
//@}

struct tree_node_reached : public tree_node_mask
{
      ///default constructor
      tree_node_reached(std::tr1::unordered_map<unsigned int, unsigned int> &_remap, std::tr1::unordered_set<unsigned int> &_not_yet_remapped, tree_manager &_TM) : remap(_remap), TM(_TM), not_yet_remapped(_not_yet_remapped) {}
      ///tree_node visitors
      BOOST_PP_SEQ_FOR_EACH(OPERATOR_MACRO_DECL, BOOST_PP_EMPTY, OBJ_SPECIALIZED_SEQ);


   private:
      ///remap old indexes in new indexes
      std::tr1::unordered_map<unsigned int, unsigned int> &remap;
      ///tree manager
      tree_manager &TM;
      ///tree_node not yet added to the Tree Manager
      std::tr1::unordered_set<unsigned int> &not_yet_remapped;
};

struct tree_node_index_factory : public tree_node_mask
{
      ///default constructor
   tree_node_index_factory(std::tr1::unordered_map<unsigned int, unsigned int> &_remap, tree_manager &_TM) : remap(_remap), TM(_TM), curr_tree_node_ptr(0) {}
      ///tree_node visitors
   BOOST_PP_SEQ_FOR_EACH(OPERATOR_MACRO_DECL, BOOST_PP_EMPTY, OBJ_SPECIALIZED_SEQ);

      /**
    * Factory method.
    * It creates a tree_node of type equal tree_node_type by using a relation (tree_node_schema) between tree node fields and their values.
    * @param node_id is the node id of the created object.
    * @param tree_node_type is the type of the node added to the tree_manager expressed as a treeVocabularyTokenTypes.
       * For example the following code:
    * std::map<int, std::string> identifier_schema;
    * int identifier_node_id = TM->get_tree_node_n();
    * identifier_schema[TOK(TOK_STRG)]= "my_identifier";
    * tree_node_factory TNF(identifier_schema, TM);
    * tree_node_factory TNF(identifier_schema,*TM);
    * TNF.create_tree_node(identifier_node_id, TOK(TOK_IDENTIFIER_NODE));
    * * will add an identifier node to the tree_manager TM.
       */
   void create_tree_node(const unsigned int node_id, const tree_nodeRef &tn);

   private:
      ///remap old indexes in new indexes
      std::tr1::unordered_map<unsigned int, unsigned int> &remap;
      ///tree manager
      tree_manager &TM;
      ///current tree node filled according to the tree_node_schema
      tree_node* curr_tree_node_ptr;
      ///current vops pointer
      vops* curr_vops_ptr;
      ///current vuse pointer
      vuse* curr_vuse_ptr;
      ///current vdef pointer
      vdef* curr_vdef_ptr;
      ///current basic block pointer
      bloc* curr_bloc;
      ///current tree_node source
      tree_nodeRef source_tn;
      ///current virtual operands source
      vopsRef source_vops;
      ///current virtual operands source
      vuseRef source_vuse;
      ///current virtual operands source
      vdefRef source_vdef;
      ///current basic block source
      blocRef source_bloc;
};

#endif
