/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file var_pp_functor.cpp
 * @brief Helper for reading data about internal representation after behavioral_manager analysis about specification produced from tree
 *
 * @author Marco Lattuada <lattuada@elet.polimi.it>
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 */
///Header include
#include "var_pp_functor.hpp"

#include "behavioral_helper.hpp"

pointer_var_pp_functor::pointer_var_pp_functor(const behavioral_helperConstRef _BH, const std::tr1::unordered_set<unsigned int> vars, bool _add_restrict):
        pointer_based_variables(vars),
        BH(_BH),
        std_functor(std_var_pp_functor(_BH)),
        add_restrict(_add_restrict)
{
}

std::string std_var_pp_functor::operator() (unsigned int var) const
{
    if (BH->is_an_indirect_ref(var))
    {
        return BH->PrintVariable(var);
    }
    if (BH->is_an_array_ref(var))
    {
        unsigned int array = BH->get_array_ref_array(var);
        unsigned int index = BH->get_array_ref_index(var);
        if(BH->is_a_mem_ref(array))
        {
           unsigned int base = BH->get_mem_ref_base(array);
           unsigned int offset = BH->get_mem_ref_offset(array);
           unsigned int type = BH->get_type(array);
           std::string offset_str = this->operator()(offset);
           if(offset_str == "0")
              return "(" + this->operator()(base) + ")";
           else
              return "((" + BH->print_type(type)+ "*)((unsigned char*)" + this->operator()(base) + " + " + offset_str + "))";
        }
        else
        return this->operator()(array) +"["+this->operator()(index)+"]";
    }
    if (BH->is_a_component_ref(var))
    {
        unsigned int record = BH->get_component_ref_record(var);
        unsigned int field = BH->get_component_ref_field(var);
        return "("+this->operator()(record)+")."+this->operator()(field);
    }
    if (BH->is_an_addr_expr(var))
    {
        unsigned int pointed = BH->get_operand_from_unary_expr(var);
        if(BH->is_a_mem_ref(pointed))
        {
           unsigned int base = BH->get_mem_ref_base(pointed);
           unsigned int offset = BH->get_mem_ref_offset(pointed);
           unsigned int type = BH->get_type(pointed);
           return "((" + BH->print_type(type)+ "*)((unsigned char*)" + this->operator()(base) + " + " + this->operator()(offset) + "))";
        }
        else if (BH->is_an_array(pointed))
            return this->operator()(pointed);
        else
            return "&(" + this->operator()(pointed) + ")";
    }
    if (BH->is_a_realpart_expr(var))
    {
        unsigned int complex = BH->get_operand_from_unary_expr(var);
        return "__real__ " + this->operator()(complex);
    }
    if (BH->is_a_imagpart_expr(var))
    {
        unsigned int complex = BH->get_operand_from_unary_expr(var);
        return "__imag__ " + this->operator()(complex);
    }
    return BH->PrintVariable(var);
}

std::string pointer_var_pp_functor::operator() (unsigned int var) const
{

    if (pointer_based_variables.find(var) == pointer_based_variables.end())
    {
        if (BH->is_an_indirect_ref(var))
        {
            unsigned int pointer = BH->get_indirect_ref_var(var);
            if (pointer_based_variables.find(pointer) != pointer_based_variables.end())
            {
                if (add_restrict)
                    return "*__restrict__ "+BH->PrintVariable(var);
                else
                    return "*"+BH->PrintVariable(var);
            }
            else
            {
                return BH->PrintVariable(var);
            }
        }
        if (BH->is_an_array_ref(var))
        {
            unsigned int array = BH->get_array_ref_array(var);
            unsigned int index = BH->get_array_ref_index(var);
            return std_functor(array)+"["+this->operator()(index)+"]";
        }
        if (BH->is_a_component_ref(var))
        {
            unsigned int record = BH->get_component_ref_record(var);
            unsigned int field = BH->get_component_ref_field(var);
            return "("+this->operator()(record)+")."+this->operator()(field);
        }
        /*if (BH->is_a_result_decl(var))
        {
           return this->operator()(BH->get_function_return());
        }*/
        if (BH->is_an_addr_expr(var))
        {
            unsigned int pointed = BH->get_operand_from_unary_expr(var);
            if (BH->is_an_array(pointed))
                return this->operator()(pointed);
            else
                return "&(" + this->operator()(pointed) + ")";
        }
        if (BH->is_a_realpart_expr(var))
        {
            unsigned int complex = BH->get_operand_from_unary_expr(var);
            return "__real__ " + this->operator()(complex);
        }
        if (BH->is_a_imagpart_expr(var))
        {
            unsigned int complex = BH->get_operand_from_unary_expr(var);
            return "__imag__ " + this->operator()(complex);
        }
        return BH->PrintVariable(var);
    }
    else
    {
        if (BH->is_an_array(var))
            return BH->PrintVariable(var);
        else if (add_restrict)
            return "*__restrict__ "+BH->PrintVariable(var);
        else
            return "*"+BH->PrintVariable(var);
    }
}

address_var_pp_functor::address_var_pp_functor(const behavioral_helperConstRef _BH, const std::tr1::unordered_set<unsigned int> vars, const std::tr1::unordered_set<unsigned int> pointer_vars) :
        addr_based_variables(vars),
        pointer_based_variables(pointer_vars),
        BH(_BH),
        readOnly(false)
{

}

std::string address_var_pp_functor::operator() (unsigned int var) const
{
    ///pointer_based_variables are the I/O parameters of the function
    ///addr_based_variables are the I/O parameters of the called task
    if (pointer_based_variables.find(var) != pointer_based_variables.end())
    {
        if (addr_based_variables.find(var) != addr_based_variables.end())
        {
            return BH->PrintVariable(var);
        }
        else
        {
            return "*"+BH->PrintVariable(var);
        }
    }
    else
    {
        if (addr_based_variables.find(var) != addr_based_variables.end())
        {
            if (BH->is_an_array(var))
                return BH->PrintVariable(var);
            else
                return "&"+BH->PrintVariable(var);
        }
        else
        {
            return BH->PrintVariable(var);
        }
    }
    //not reachable point
    return "";
}
