/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file modelsimWrapper.hpp
 * @brief Wrapper to modelsim.
 *
 * A object used to invoke the modelsim compiler and simulator
 *
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef MODELSIMWRAPPER_HPP
#define MODELSIMWRAPPER_HPP

#include "SimulationTool.hpp"
#include "refcount.hpp"
CONSTREF_FORWARD_DECL(Parameter);

#include <string>

/**
 * @class modelsimWrapper
 * Main class for wrapping the modesilm compiler and simulator.
 */
class modelsimWrapper : public SimulationTool
{
      /**
       * Generates the proper simulation script
       */
      void GenerateScript(std::ostringstream& script, const std::string& top_filename, const std::string& FileList);

   public:

      /**
       * Constructor
       * @param Param is the set of parameters
       */
      modelsimWrapper(const ParameterConstRef Param);

      /**
       * Destructor
       */
      ~modelsimWrapper();

      /**
       * Checks if the current specification can be executed or not
       */
      virtual void CheckExecution();

};
/// Refcount definition for the modelsimWrapper class
typedef refcount<modelsimWrapper> modelsimWrapperRef;

#endif
