/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file AlteraBackendFlow.hpp
 * @brief Wrapper to implement a synthesis tools by Altera
 *
 * A object used to invoke synthesis tools by Altera
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Date: 2012-03-19 12:10:41 +0100 (Mon, 19 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef _ALTERA_BACKEND_FLOW_HPP_
#define _ALTERA_BACKEND_FLOW_HPP_

///superclass include
#include "BackendFlow.hpp"

#include "refcount.hpp"
REF_FORWARD_DECL(target_device);

class AlteraBackendFlow : public BackendFlow
{
      ///design values reported by the tools
      std::map<std::string, double> design_values;

      /**
       * Writes the proper flow configuration in the output script
       */
      void WriteFlowConfiguration(std::ostream& script);

      /**
       * Parses device utilization
       */
      void xparse_utilization(const std::string& fn);

      /**
       * Checks the synthesis results and fills the corresponding datastructures
       */
      void CheckSynthesisResults();

      /**
       * Creates the constraint file
       */
      void create_sdc(const DesignParametersRef dp, bool time_constrained);

   public:

      /**
       * Constructor
       * @param Param is the reference to the class containing all the parameters
       * @param device is the reference to the datastructure representing the target device
       */
      AlteraBackendFlow(const ParameterConstRef Param, const std::string & flow_name, const target_deviceRef device);

      /**
       * Destructor
       */
      virtual ~AlteraBackendFlow();

      /**
       * Initializes the parameters
       */
      void InitDesignParameters(const DesignParametersRef dp);

      /**
       * Checks if the execution can be performed and, in case, performs the synthesis
       */
      void ExecuteSynthesis();

};
///Refcount definition for the class
typedef refcount<AlteraBackendFlow> AlteraBackendFlowRef;

#endif
