/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file XilinxBackendFlow.hpp
 * @brief Wrapper to implement a synthesis tools by Xilinx
 *
 * A object used to invoke synthesis tools by Xilinx
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Date: 2012-03-19 12:10:41 +0100 (Mon, 19 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef _XILINX_BACKEND_FLOW_HPP_
#define _XILINX_BACKEND_FLOW_HPP_

///superclass include
#include "BackendFlow.hpp"

#include "refcount.hpp"
REF_FORWARD_DECL(target_device);

#define PARAM_ucf_file           "ucf_file"
#define PARAM_xcf_file           "xcf_file"
#define PARAM_power_optimization "power_optimization"
#define PARAM_connect_iob        "connect_iob"

class XilinxBackendFlow : public BackendFlow
{
      ///results from the synthesis
      std::map<std::string, double> area_values;

      /**
       * Creates the UCF file
       */
      void create_cf(const DesignParametersRef dp, bool xst);

      /**
       * Writes the proper flow configuration in the output script
       */
      void WriteFlowConfiguration(std::ostream& script);

      /**
       * Checks the synthesis results and fills the corresponding datastructures
       */
      void CheckSynthesisResults();

      /**
       * Parses the utilization file in XML format
       */
      void xparse_xst_utilization(const std::string& fn);

      /**
       * Parses the utilization file in XML format
       */
      void xparse_map_utilization(const std::string& fn);

      /**
       * Fixed the parsing of timing results from trce
       */
      void xparse_timing(const std::string& fn, bool post);

      /**
       * Fixed the parsing of timing results from xst
       */
      void parse_timing(const std::string& fn);

   public:

      /**
       * Constructor
       * @param Param is the reference to the class containing all the parameters
       * @param device is the reference to the datastructure representing the target device
       */
      XilinxBackendFlow(const ParameterConstRef Param, const std::string& flow_name, const target_deviceRef device);

      /**
       * Destructor
       */
      virtual ~XilinxBackendFlow();

      /**
       * Initializes the parameters
       */
      void InitDesignParameters(const DesignParametersRef dp);

      /**
       * Checks if the execution can be performed and, in case, performs the synthesis
       */
      void ExecuteSynthesis();

};
///Refcount definition for the class
typedef refcount<XilinxBackendFlow> XilinxBackendFlowRef;

#endif
