/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file cdfc_module_binding.hpp
 * @brief Module binding based on the analysis of the control data flow chained graph
 *
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * @version $Revision: 10608 $
 * @date $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
*/

#ifndef CDFC_MODULE_BINDING_HPP
#define CDFC_MODULE_BINDING_HPP

#include <map>
#include <string>

#include "refcount.hpp"
#include "graph.hpp"
#include "application_manager.hpp"

#include "fu_binding_creator.hpp"


/**
 * Class managing the module allocation.
 */
class cdfc_module_binding : public fu_binding_creator
{
   public:

      /// Module binding algorithms
      typedef enum
      {
         TTT_FAST = 0,
         TTT_FULL,
         COLORING,
         WEIGHTED_COLORING,
         BIPARTITE_MATCHING
      } type_t;

      /**
       * This is the constructor of the class.
       */
      cdfc_module_binding(const ParameterRef Param, const hlsRef HLS, type_t _clique_covering_method);

      /**
       * Destructor.
       */
      ~cdfc_module_binding();

      /**
       * Performs module binding exploiting the control data flow chained graph
       */
      void exec();

      /**
       * Returns the name of the implemented algorithm
       */
      std::string get_kind_text() const;

   private:

      type_t clique_covering_method;
      void get_reg_in(std::vector<application_manager::io_binding_type> &vars_read, vertex state, vertex op, std::list<unsigned int> &r_eg);
      /**
       * compute the output register in case the variable is in the liveout of a given state, otherwise the negated var_written is returned
       * @param var_written is the variable written
       * @param state is the state in which the variable is written
       * @param reg_out is the computed register index
       */
      void get_reg_out(unsigned int var_written, vertex state, unsigned int &reg_out);

      static const int CD_EDGE = 1;
      static const int COMPATIBILITY_EDGE = 2;


};

#endif
