/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file FSM_NI_SSA_liveness.hpp
 * @brief liveness analysis exploiting the SSA form of the IR
 *
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef FSM_NI_SSA_LIVENESS_HPP
#define FSM_NI_SSA_LIVENESS_HPP

#include "refcount.hpp"

#include "liveness_computer.hpp"
/**
 * @name Forward declarations.
 */
//@{
REF_FORWARD_DECL(hls);
REF_FORWARD_DECL(Parameter);
//@}

class FSM_NI_SSA_liveness : public liveness_computer
{

   public:

      /**
       * Constructor
       * @param Param is the refcount to the class containing all the parameters
       * @param HLS is the class containing all the HLS intermediate results
       */
      FSM_NI_SSA_liveness(const ParameterRef Param, const hlsRef HLS);

      /**
       * Destructor
       */
      virtual ~FSM_NI_SSA_liveness();
    
      /**
       * This function computes dataflow analysis based on State Transition Graph 
       * based on dataflow equations.
       */
      virtual void exec();

      /**
       * Returns the name of the algorithms
       */
      virtual std::string get_kind_text() const;

};

#endif

