/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEI
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2012 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file tree_node_finder.hpp
 * @brief tree node finder. This class exploiting the visitor design pattern find a tree node in a tree_manager.
 *
 * @author Fabrizio Ferrandi <ferrandi@elet.polimi.it>
 * $Revision: 10608 $
 * $Date: 2012-03-12 10:46:42 +0100 (Mon, 12 Mar 2012) $
 * Last modified by $Author: ferrandi $
 *
*/
#ifndef TREE_NODE_FINDER_HPP
#define TREE_NODE_FINDER_HPP

///STD include
#include <ostream>

///Tree include
#include "token_interface.hpp"
#include "tree_node.hpp"
#include "tree_node_mask.hpp"

///Utility include
#include <boost/preprocessor/seq/for_each.hpp>
#include <boost/preprocessor/facilities/empty.hpp>
#include "refcount.hpp"

/**
 * @name forward declarations
 */
//@{
REF_FORWARD_DECL(tree_node_finder);
//@}

struct tree_node_finder : public tree_node_mask
{
      ///default constructor
      tree_node_finder(const std::map<treeVocabularyTokenTypes::token_enum, std::string> &_tree_node_schema) : find_res(true), tree_node_schema(_tree_node_schema), os(std::cout) {}
      ///tree_node visitors
      BOOST_PP_SEQ_FOR_EACH(OPERATOR_MACRO_DECL, BOOST_PP_EMPTY, OBJ_SPECIALIZED_SEQ);

      ///Return true in case the tree node is compatible with the tree_node_schema. Usually called by tree_manager::create_tree_node.
      bool check(const tree_nodeRef &t)
      {
         find_res=true;
         t->visit(this);
         return find_res;
      }
   private:
      ///result of the search
      bool find_res;
      ///tree_node_schema expresses the value of the fields of the tree node we are looking for.
      const std::map<treeVocabularyTokenTypes::token_enum, std::string> &tree_node_schema;

      ///os is used for debugging purpose
      std::ostream& os;
      /**
       * Write the field when t is not null
       * @param str is the string key associated to t
       * @param t is the tree_nodeRef
       */
      void write_when_not_null(const std::string str, const tree_nodeRef & t);
      /**
       * Write the field when t is not null
       * @param str is the string key associated to t
       * @param t is the blocRef
       */
      void write_when_not_null_bloc(const std::string str, const blocRef & t);

      /**
       * Write the field when t is not null
       * @param t is the vopsRef
       */
      void write_when_not_null_vops(const vopsRef & t);

      /**
       * Write the field when t is not null
       * @param t is the vuseRef
       */
      void write_when_not_null_vuse(const vuseRef & t);

      /**
       * Write the field when t is not null
       * @param t is the vdefRef
       */
      void write_when_not_null_vdef(const vdefRef & t);
};

#endif
